var CustomReporter = require('./customReporter');

const errorshot = require('wdio-errorshot-reporter');

var path = require('path');
var VisualRegressionCompare = require('wdio-visual-regression-service/compare');
var getScreenshotName = require('./getScreenshotName');
// enable require text files for testing
var fs = require('fs');
require.extensions['.txt'] = function(module, filename) {
    module.exports = fs.readFileSync(filename, 'utf8');
};

const debug = process.env.DEBUG;
const defaultTimeoutInterval = 180000;
var defaultMaxInstances = process.env.MANAGER_MAX_INSTANCES || 3;

var diffDir = process.env.SCREENSHOT_DIRECTORY + '/diff' || 'screenshots/diff/';
var refDir =
    process.env.SCREENSHOT_DIRECTORY + '/reference' || 'screenshots/reference/';
var screenDir =
    process.env.SCREENSHOT_DIRECTORY + '/screen' || 'screenshots/screen/';
var errorDir = process.env.SCREENSHOT_DIRECTORY + '/error' || './errorShots/';

var config = {
    //
    // ==================
    // Specify Test Files
    // ==================
    // Define which test specs should run. The pattern is relative to the directory
    // from which `wdio` was called. Notice that, if you are calling `wdio` from an
    // NPM script (see https://docs.npmjs.com/cli/run-script) then the current working
    // directory is where your package.json resides, so `wdio` will be called from there.
    //
    // specs: [
    //     './specs/**/results.logic.spec.js'
    // ],
    specs: [
        process.env.SPEC_FILE_PATTERN ||
            './local/specs/core/**/querypage.spec.js',
    ],

    // Patterns to exclude.
    exclude: [
        // 'path/to/excluded/files'
    ],
    //
    // ============
    // Capabilities
    // ============
    // Define your capabilities here. WebdriverIO can run multiple capabilities at the same
    // time. Depending on the number of capabilities, WebdriverIO launches several test
    // sessions. Within your capabilities you can overwrite the spec and exclude options in
    // order to group specific specs to a specific capability.
    //
    // First, you can define how many instances should be started at the same time. Let's
    // say you have 3 different capabilities (Chrome, Firefox, and Safari) and you have
    // set maxInstances to 1; wdio will spawn 3 processes. Therefore, if you have 10 spec
    // files and you set maxInstances to 10, all spec files will get tested at the same time
    // and 30 processes will get spawned. The property handles how many capabilities
    // from the same test should run tests.
    //
    maxInstances: debug ? 1 : defaultMaxInstances,
    //
    // If you have trouble getting all important capabilities together, check out the
    // Sauce Labs platform configurator - a great tool to configure your capabilities:
    // https://docs.saucelabs.com/reference/platforms-configurator
    //
    capabilities: [
        {
            //browserName: 'chrome',
            chromeOptions: {
                args: [
                    '--disable-composited-antialiasing',
                    '--allow-insecure-localhost',
                ].concat(
                    (function() {
                        return process.env.HEADLESS_CHROME
                            ? [
                                  '--headless',
                                  '--no-sandbox',
                                  '--disable-gpu',
                                  '--disable-setuid-sandbox',
                              ]
                            : [];
                    })()
                ),
            },

            os: 'OS X',
            os_version: 'High Sierra',
            browser: 'Chrome',
            browser_version: '74.0 beta',
            resolution: '1600x1200',
        },
    ],

    IECapabilties: [
        {
            os: 'Windows',
            os_version: '10',
            browser: 'IE',
            browser_version: '11.0',
            'browserstack.selenium_version': '3.5.2',
            resolution: '1600x1200',
            'browserstack.local': true,
        },
    ],

    // wdio.conf.js

    //
    // ===================
    // Test Configurations
    // ===================
    // Define all options that are relevant for the WebdriverIO instance here
    //
    // By default WebdriverIO commands are executed in a synchronous way using
    // the wdio-sync package. If you still want to run your tests in an async way
    // e.g. using promises you can set the sync option to false.
    sync: true,

    //
    // Level of logging verbosity: silent | verbose | command | data | result | error
    logLevel: 'error',
    //
    // Enables colors for log output.
    coloredLogs: true,
    //
    // If you only want to run your tests until a specific amount of tests have failed use
    // bail (default is 0 - don't bail, run all tests).
    bail: 0,
    //
    // Saves a screenshot to a given path if a command fails.
    screenshotPath: errorDir,
    //
    // Set a base URL in order to shorten url command calls. If your url parameter starts
    // with "/", then the base url gets prepended.
    baseUrl: 'http://localhost',
    //
    // Default timeout for all waitFor* commands.
    waitforTimeout: 30000,
    //
    // Default timeout in milliseconds for request
    // if Selenium Grid doesn't send response
    connectionRetryTimeout: 90000,
    //
    // Default request retries count
    connectionRetryCount: 3,
    //
    // Initialize the browser instance with a WebdriverIO plugin. The object should have the
    // plugin name as key and the desired plugin options as properties. Make sure you have
    // the plugin installed before running any tests. The following plugins are currently
    // available:
    // WebdriverCSS: https://github.com/webdriverio/webdrivercss
    // WebdriverRTC: https://github.com/webdriverio/webdriverrtc
    // Browserevent: https://github.com/webdriverio/browserevent
    // plugins: {
    //     webdrivercss: {
    //         screenshotRoot: 'my-shots',
    //         failedComparisonsRoot: 'diffs',
    //         misMatchTolerance: 0.05,
    //         screenWidth: [320,480,640,1024]
    //     },
    //     webdriverrtc: {},
    //     browserevent: {}
    // },
    //
    // Test runner services
    // Services take over a specific job you don't want to take care of. They enhance
    // your test setup with almost no effort. Unlike plugins, they don't add new
    // commands. Instead, they hook themselves up into the test process.

    services: ['visual-regression'],

    // FIXME: the browser name 'chrome' is passed for screenshot name evaluation
    // Reason is a bug with headless chrome reporting as 'safari'. This should be
    // changed when adding tests for other browsers.
    // See: https://github.com/zinserjan/wdio-visual-regression-service/issues/81
    visualRegression: {
        compare: new VisualRegressionCompare.LocalCompare({
            referenceName: getScreenshotName(
                path.join(process.cwd(), refDir),
                'chrome'
            ),
            screenshotName: getScreenshotName(
                path.join(process.cwd(), screenDir),
                'chrome'
            ),
            diffName: getScreenshotName(
                path.join(process.cwd(), diffDir),
                'chrome'
            ),
            misMatchTolerance: 0.01,
            ignoreComparison: 'antialiasing',
        }),
        viewportChangePause: 300,
        viewports: [{ width: 1600, height: 1000 }],
        orientations: ['landscape', 'portrait'],
    },
    // Framework you want to run your specs with.
    // The following are supported: Mocha, Jasmine, and Cucumber
    // see also: http://webdriver.io/guide/testrunner/frameworks.html
    //
    // Make sure you have the wdio adapter package for the specific framework installed
    // before running any tests.
    framework: 'mocha',
    //
    // Test reporter for stdout.
    // The only one supported by default is 'dot'
    // see also: http://webdriver.io/guide/testrunner/reporters.html
    reporters: ['spec', 'junit', CustomReporter, errorshot],
    reporterOptions: {
        junit: {
            outputDir: process.env.JUNIT_REPORT_PATH || './',
            outputFileFormat: function(opts) {
                // optional
                return `results-${opts.cid}.${opts.capabilities}.xml`;
            },
        },
        custom: {
            outputDir: process.env.JUNIT_REPORT_PATH || './',
            outputFileFormat: function(opts) {
                // optional
                return `custom-results-${opts.cid}.${opts.capabilities}.xml`;
            },
            errorshotReporter: {
                template: 'foobar-%capId%_%timestamp%_%parent%-%title%',
            },
        },
    },
    //
    // Options to be passed to Mocha.
    // See the full list at http://mochajs.org/
    mochaOpts: {
        ui: 'bdd',
        timeout: debug ? 20000000 : defaultTimeoutInterval, // make big when using browser.debug()
    },
    //
    // =====
    // Hooks
    // =====
    // WebdriverIO provides several hooks you can use to interfere with the test process in order to enhance
    // it and to build services around it. You can either apply a single function or an array of
    // methods to it. If one of them returns with a promise, WebdriverIO will wait until that promise got
    // resolved to continue.
    /**
     * Gets executed once before all workers get launched.
     * @param {Object} config wdio configuration object
     * @param {Array.<Object>} capabilities list of capabilities details
     */
    // onPrepare: function (config, capabilities) {
    // },
    /**
     * Gets executed just before initialising the webdriver session and test framework. It allows you
     * to manipulate configurations depending on the capability or spec.
     * @param {Object} config wdio configuration object
     * @param {Array.<Object>} capabilities list of capabilities details
     * @param {Array.<String>} specs List of spec file paths that are to be run
     */
    // beforeSession: function (config, capabilities, specs) {
    // },
    /**
     * Gets executed before test execution begins. At this point you can access to all global
     * variables like `browser`. It is the perfect place to define custom commands.
     * @param {Array.<Object>} capabilities list of capabilities details
     * @param {Array.<String>} specs List of spec file paths that are to be run
     */
    // before: function (capabilities, specs) {
    // },
    //
    /**
     * Hook that gets executed before the suite starts
     * @param {Object} suite suite details
     */
    // beforeSuite: function (suite) {
    // },
    /**
     * Hook that gets executed _before_ a hook within the suite starts (e.g. runs before calling
     * beforeEach in Mocha)
     */
    // beforeHook: function () {
    // },
    /**
     * Hook that gets executed _after_ a hook within the suite starts (e.g. runs after calling
     * afterEach in Mocha)
     */
    // afterHook: function () {
    // },
    /**
     * Function to be executed before a test (in Mocha/Jasmine) or a step (in Cucumber) starts.
     * @param {Object} test test details
     */
    // beforeTest: function (test) {
    // },
    /**
     * Runs before a WebdriverIO command gets executed.
     * @param {String} commandName hook command name
     * @param {Array} args arguments that command would receive
     */
    // beforeCommand: function (commandName, args) {
    // },
    /**
     * Runs after a WebdriverIO command gets executed
     * @param {String} commandName hook command name
     * @param {Array} args arguments that command would receive
     * @param {Number} result 0 - command success, 1 - command error
     * @param {Object} error error object if any
     */
    // afterCommand: function (commandName, args, result, error) {
    // },
    /**
     * Function to be executed after a test (in Mocha/Jasmine) or a step (in Cucumber) starts.
     * @param {Object} test test details
     */
    afterTest: function(test) {
        var networkLog = browser.execute(function() {
            Object.keys(window.ajaxRequests).forEach(key => {
                window.ajaxRequests[key].end = Date.now();
                window.ajaxRequests[key].duration =
                    window.ajaxRequests[key].end -
                    window.ajaxRequests[key].started;
            });

            return JSON.stringify(window.ajaxRequests);
        }).value;

        process.send({
            event: 'custom-report',
            data: { test: test, network: JSON.parse(networkLog) },
        });
    },
    /**
     * Hook that gets executed after the suite has ended
     * @param {Object} suite suite details
     */
    // afterSuite: function (suite) {
    // },
    /**
     * Gets executed after all tests are done. You still have access to all global variables from
     * the test.
     * @param {Number} result 0 - test pass, 1 - test fail
     * @param {Array.<Object>} capabilities list of capabilities details
     * @param {Array.<String>} specs List of spec file paths that ran
     */
    // after: function (result, capabilities, specs) {
    // },
    /**
     * Gets executed right after terminating the webdriver session.
     * @param {Object} config wdio configuration object
     * @param {Array.<Object>} capabilities list of capabilities details
     * @param {Array.<String>} specs List of spec file paths that ran
     */
    // afterSession: function (config, capabilities, specs) {
    // },
    /**
     * Gets executed after all workers got shut down and the process is about to exit. It is not
     * possible to defer the end of the process using a promise.
     * @param {Object} exitCode 0 - success, 1 - fail
     */
    // onComplete: function(exitCode) {
    // }
};

if (process.env.TEST_IE11 === 'true') {
    config.capabilities = config.IECapabilties;
}

if (process.env.TEST_BROWSERSTACK === 'true') {
    config.services = ['visual-regression', 'browserstack'];
    config.browserstackLocal = true;
    config.user = process.env.BROWSERSTACK_USER;
    config.key = process.env.BROWSERSTACK_KEY;
}

//config.specs = ['./remote/specs/core/studyview.spec.js'];

exports.config = config;
