// SharedMutationsTable.tsx

import React, { useState, useEffect, useRef } from 'react';
import './SharedMutationsTable.css';

// Typdefinitionen für Mutationen und Patienten (können auch in einer separaten Datei ausgelagert werden)
type Mutation = {
  gen: string;
  proteinChange: string;
  pathway: string;
  allelfrequenz: number;
};

type Patient = {
  id: string;
  alter: number;
  overallSurvival: number;
  ort: string;
  artDerTumorprobe: "Primaertumor" | "Metastase";
  similarityScore?: number;
  mutationen: Mutation[];
  therapien: { name: string; reasoning: string; comment: string }[];
};

// Funktion zur Ermittlung von gemeinsamen und unterschiedlichen Mutationen
const getSharedAndDistinctMutations = (patient1: Patient, patient2: Patient) => {
  const sharedMutations = {
    identical: [] as Mutation[],
    sameGene: [] as Mutation[],
    samePathway: [] as Mutation[],
  };
  const distinctMutations = {
    patient1: [] as Mutation[],
    patient2: [] as Mutation[],
  };

  patient1.mutationen.forEach((mut1) => {
    const match = patient2.mutationen.find((mut2) => mut1.gen === mut2.gen && mut1.proteinChange === mut2.proteinChange);
    if (match) {
      sharedMutations.identical.push(mut1);
    } else {
      const sameGene = patient2.mutationen.find((mut2) => mut1.gen === mut2.gen && mut1.proteinChange !== mut2.proteinChange);
      const samePathway = patient2.mutationen.find((mut2) => mut1.pathway === mut2.pathway && mut1.gen !== mut2.gen);
      if (sameGene) {
        sharedMutations.sameGene.push(mut1);
      } else if (samePathway) {
        sharedMutations.samePathway.push(mut1);
      } else {
        distinctMutations.patient1.push(mut1);
      }
    }
  });

  patient2.mutationen.forEach((mut2) => {
    const isDistinct = !sharedMutations.identical.find((mut) => mut.gen === mut2.gen && mut.proteinChange === mut2.proteinChange) &&
                       !sharedMutations.sameGene.find((mut) => mut.gen === mut2.gen) &&
                       !sharedMutations.samePathway.find((mut) => mut.pathway === mut2.pathway);

    if (isDistinct) {
      distinctMutations.patient2.push(mut2);
    }
  });

  return { sharedMutations, distinctMutations };
};

// Patientendaten definieren
const patients: Patient[] = [
    {
      id: "1",
      alter: 65,
      overallSurvival: 15,
      ort: "Lunge",
      artDerTumorprobe: "Primaertumor",
      mutationen: [
        { gen: "TP53", proteinChange: "R273H", pathway: "Cell Cycle", allelfrequenz: 0.45 },
        { gen: "MDC1", proteinChange: "L858R", pathway: "mTor", allelfrequenz: 0.35 },
        { gen: "KRAS", proteinChange: "G12D", pathway: "Growth Signaling", allelfrequenz: 0.42 },
        { gen: "BRCA1", proteinChange: "185delAG", pathway: "DNA Repair", allelfrequenz: 0.38 },
        { gen: "PIK3CA", proteinChange: "E545K", pathway: "Growth Signaling", allelfrequenz: 0.33 },
        { gen: "PTEN", proteinChange: "R135Q", pathway: "Growth Signaling", allelfrequenz: 0.4 },
        { gen: "ALK", proteinChange: "F1174L", pathway: "Growth Signaling", allelfrequenz: 0.36 },
        { gen: "CDKN2A", proteinChange: "V126D", pathway: "Cell Cycle", allelfrequenz: 0.41 },
        { gen: "SMAD4", proteinChange: "D351H", pathway: "TGF-Beta Signaling", allelfrequenz: 0.29 },
        { gen: "NOTCH1", proteinChange: "P2515S", pathway: "Cell Cycle", allelfrequenz: 0.27 },
        { gen: "CTNNB1", proteinChange: "S37R", pathway: "Cell Cycle", allelfrequenz: 0.43 },
        { gen: "AKT1", proteinChange: "E17K", pathway: "Growth Signaling", allelfrequenz: 0.39 },
        { gen: "KMT2D", proteinChange: "V600E", pathway: "mTOR", allelfrequenz: 0.44 },
        { gen: "MET", proteinChange: "Y1253D", pathway: "Growth Signaling", allelfrequenz: 0.4 },
        { gen: "ERBB2", proteinChange: "S310F", pathway: "Growth Signaling", allelfrequenz: 0.34 },
        { gen: "FGFR3", proteinChange: "R248C", pathway: "Growth Signaling", allelfrequenz: 0.31 },
        { gen: "HRAS", proteinChange: "G12V", pathway: "Growth Signaling", allelfrequenz: 0.37 },
        { gen: "IDH1", proteinChange: "R132H", pathway: "Metabolism", allelfrequenz: 0.28 },
        { gen: "JAK2", proteinChange: "V617F", pathway: "Growth Signaling", allelfrequenz: 0.32 },
        { gen: "MLH1", proteinChange: "I219V", pathway: "DNA Repair", allelfrequenz: 0.39 },
        { gen: "NRAS", proteinChange: "Q61L", pathway: "Growth Signaling", allelfrequenz: 0.42 },
        { gen: "PIK3R1", proteinChange: "K567E", pathway: "Growth Signaling", allelfrequenz: 0.3 },
        { gen: "RET", proteinChange: "M918T", pathway: "Growth Signaling", allelfrequenz: 0.38 },
        { gen: "SMARCB1", proteinChange: "R374Q", pathway: "Chromatin Remodeling", allelfrequenz: 0.41 },
        { gen: "VHL", proteinChange: "L188V", pathway: "Metabolism", allelfrequenz: 0.37 },
        { gen: "MDC1", proteinChange: "R85S", pathway: "mTor", allelfrequenz: 0.35 },
      ],
      therapien: [
        { name: "Therapie Aa", reasoning: "", comment: "" },
        { name: "Therapie Bb", reasoning: "", comment: "" },
      ],
    },
    {
      id: "2",
      alter: 58,
      overallSurvival: 12,
      ort: "Lunge",
      artDerTumorprobe: "Metastase",
      similarityScore: 95,
      mutationen: [
        { gen: "TP53", proteinChange: "R273H", pathway: "Cell Cycle", allelfrequenz: 0.41 }, // identisch
        { gen: "EGFR", proteinChange: "L858R", pathway: "Growth Signaling", allelfrequenz: 0.29 }, // identisch
        { gen: "KRAS", proteinChange: "G13D", pathway: "Growth Signaling", allelfrequenz: 0.47 }, // anderes ProteinChange
        { gen: "BRCA1", proteinChange: "5382insC", pathway: "DNA Repair", allelfrequenz: 0.4 }, // anderes ProteinChange
        { gen: "PIK3CA", proteinChange: "E545K", pathway: "Growth Signaling", allelfrequenz: 0.37 }, // identisch
        { gen: "CTNNB1", proteinChange: "S33P", pathway: "Cell Cycle", allelfrequenz: 0.36 }, // identisch
        { gen: "PTEN", proteinChange: "R130Q", pathway: "Growth Signaling", allelfrequenz: 0.37 }, // identisch
        { gen: "BRAF", proteinChange: "V600E", pathway: "Growth Signaling", allelfrequenz: 0.39 }, // identisch
        { gen: "ALK", proteinChange: "F1174L", pathway: "Growth Signaling", allelfrequenz: 0.33 }, // identisch
        { gen: "SMAD4", proteinChange: "D351H", pathway: "TGF-Beta Signaling", allelfrequenz: 0.32 }, // identisch
        // 10 weitere Mutationen ohne Überschneidung
        { gen: "ATM", proteinChange: "R337H", pathway: "DNA Damage Response", allelfrequenz: 0.34 },
        { gen: "NTRK1", proteinChange: "M918T", pathway: "Neurotrophic Signaling", allelfrequenz: 0.31 },
        { gen: "EPHA2", proteinChange: "S897A", pathway: "Cell Adhesion", allelfrequenz: 0.3 },
        { gen: "AXIN1", proteinChange: "R434X", pathway: "Wnt Signaling", allelfrequenz: 0.29 },
        { gen: "GATA3", proteinChange: "D335N", pathway: "Transcription Regulation", allelfrequenz: 0.36 },
        { gen: "MDM2", proteinChange: "S395F", pathway: "p53 Regulation", allelfrequenz: 0.35 },
        { gen: "NFE2L2", proteinChange: "E79Q", pathway: "Oxidative Stress Response", allelfrequenz: 0.32 },
        { gen: "PBRM1", proteinChange: "G973E", pathway: "Chromatin Remodeling", allelfrequenz: 0.38 },
        { gen: "RB1", proteinChange: "R661W", pathway: "Cell Cycle", allelfrequenz: 0.34 },
        { gen: "TSC1", proteinChange: "Q715X", pathway: "mTOR Signaling", allelfrequenz: 0.37 },
      ],
      therapien: [
        { name: "Therapie A", reasoning: "", comment: "" },
        { name: "Therapie B", reasoning: "", comment: "" },
      ],
    },
    {
      id: "3",
      alter: 70,
      overallSurvival: 10,
      ort: "Lunge",
      artDerTumorprobe: "Primaertumor",
      similarityScore: 92,
      mutationen: [
        { gen: "TP53", proteinChange: "R273H", pathway: "Cell Cycle", allelfrequenz: 0.44 }, // identisch
        { gen: "KRAS", proteinChange: "G12D", pathway: "Growth Signaling", allelfrequenz: 0.43 }, // identisch
        { gen: "EGFR", proteinChange: "T790M", pathway: "Growth Signaling", allelfrequenz: 0.36 }, // anderes ProteinChange
        { gen: "BRCA1", proteinChange: "C61G", pathway: "DNA Repair", allelfrequenz: 0.4 }, // anderes ProteinChange
        { gen: "PIK3CA", proteinChange: "H1047R", pathway: "Growth Signaling", allelfrequenz: 0.35 }, // anderes ProteinChange
        { gen: "PTEN", proteinChange: "G129E", pathway: "Growth Signaling", allelfrequenz: 0.32 }, // gleicher Pathway
        { gen: "ALK", proteinChange: "R1275Q", pathway: "Growth Signaling", allelfrequenz: 0.31 }, // gleicher Pathway
        { gen: "CTNNB1", proteinChange: "S45F", pathway: "Cell Cycle", allelfrequenz: 0.39 }, // gleicher Pathway
        { gen: "BRAF", proteinChange: "K601E", pathway: "Growth Signaling", allelfrequenz: 0.37 }, // gleicher Pathway
        { gen: "MET", proteinChange: "D1246N", pathway: "Growth Signaling", allelfrequenz: 0.38 }, // gleicher Pathway
        // 10 weitere Mutationen ohne Überschneidung
        { gen: "SMO", proteinChange: "W535L", pathway: "Hedgehog Signaling", allelfrequenz: 0.31 },
        { gen: "FLT3", proteinChange: "D835Y", pathway: "Growth Signaling", allelfrequenz: 0.32 },
        { gen: "CCND1", proteinChange: "P287L", pathway: "Cell Cycle", allelfrequenz: 0.34 },
        { gen: "EZH2", proteinChange: "Y641N", pathway: "Epigenetic Regulation", allelfrequenz: 0.33 },
        { gen: "GNAS", proteinChange: "R201C", pathway: "G Protein Signaling", allelfrequenz: 0.37 },
        { gen: "TET2", proteinChange: "R550X", pathway: "DNA Methylation", allelfrequenz: 0.36 },
        { gen: "FOXL2", proteinChange: "C134W", pathway: "Transcription Regulation", allelfrequenz: 0.38 },
        { gen: "SDHB", proteinChange: "R46Q", pathway: "Metabolism", allelfrequenz: 0.3 },
        { gen: "DNMT3A", proteinChange: "R882H", pathway: "DNA Methylation", allelfrequenz: 0.35 },
        { gen: "WT1", proteinChange: "R394W", pathway: "Transcription Regulation", allelfrequenz: 0.4 },
      ],
      therapien: [
        { name: "Therapie A", reasoning: "", comment: "" },
        { name: "Therapie B", reasoning: "", comment: "" },
      ],
    },
    {
      id: "4",
      alter: 62,
      overallSurvival: 8,
      ort: "Leber",
      artDerTumorprobe: "Metastase",
      similarityScore: 85,
      mutationen: [
        { gen: "TP53", proteinChange: "R273H", pathway: "Cell Cycle", allelfrequenz: 0.42 }, // identisch
        { gen: "BRCA1", proteinChange: "185delAG", pathway: "DNA Repair", allelfrequenz: 0.39 }, // identisch
        { gen: "KRAS", proteinChange: "G12C", pathway: "Growth Signaling", allelfrequenz: 0.4 }, // anderes ProteinChange
        { gen: "EGFR", proteinChange: "L861Q", pathway: "Growth Signaling", allelfrequenz: 0.36 }, // anderes ProteinChange
        { gen: "PIK3CA", proteinChange: "H1047R", pathway: "Growth Signaling", allelfrequenz: 0.38 }, // identisch
        { gen: "ERBB2", proteinChange: "G776S", pathway: "Growth Signaling", allelfrequenz: 0.32 }, // gleicher Pathway
        { gen: "FGFR3", proteinChange: "Y373C", pathway: "Growth Signaling", allelfrequenz: 0.34 }, // gleicher Pathway
        { gen: "NOTCH1", proteinChange: "L1576P", pathway: "Cell Cycle", allelfrequenz: 0.3 }, // gleicher Pathway
        { gen: "SMAD4", proteinChange: "R361C", pathway: "TGF-Beta Signaling", allelfrequenz: 0.28 }, // gleicher Pathway
        { gen: "IDH1", proteinChange: "G97D", pathway: "Metabolism", allelfrequenz: 0.35 }, // gleicher Pathway
        // 10 weitere Mutationen ohne Überschneidung
        { gen: "SMO", proteinChange: "W535L", pathway: "Hedgehog Signaling", allelfrequenz: 0.31 },
        { gen: "FLT3", proteinChange: "D835Y", pathway: "Growth Signaling", allelfrequenz: 0.32 },
        { gen: "CCND1", proteinChange: "P287L", pathway: "Cell Cycle", allelfrequenz: 0.34 },
        { gen: "EZH2", proteinChange: "Y641N", pathway: "Epigenetic Regulation", allelfrequenz: 0.33 },
        { gen: "GNAS", proteinChange: "R201C", pathway: "G Protein Signaling", allelfrequenz: 0.37 },
        { gen: "TET2", proteinChange: "R550X", pathway: "DNA Methylation", allelfrequenz: 0.36 },
        { gen: "FOXL2", proteinChange: "C134W", pathway: "Transcription Regulation", allelfrequenz: 0.38 },
        { gen: "SDHB", proteinChange: "R46Q", pathway: "Metabolism", allelfrequenz: 0.3 },
        { gen: "DNMT3A", proteinChange: "R882H", pathway: "DNA Methylation", allelfrequenz: 0.35 },
        { gen: "WT1", proteinChange: "R394W", pathway: "Transcription Regulation", allelfrequenz: 0.4 },
      ],
      therapien: [
        { name: "Therapie A", reasoning: "", comment: "" },
        { name: "Therapie B", reasoning: "", comment: "" },
      ],
    },
    {
        id: "4",
        alter: 62,
        overallSurvival: 8,
        ort: "Leber",
        artDerTumorprobe: "Metastase",
        similarityScore: 85,
        mutationen: [
          { gen: "TP53", proteinChange: "R273H", pathway: "Cell Cycle", allelfrequenz: 0.42 }, // identisch
          { gen: "BRCA1", proteinChange: "185delAG", pathway: "DNA Repair", allelfrequenz: 0.39 }, // identisch
          { gen: "KRAS", proteinChange: "G12C", pathway: "Growth Signaling", allelfrequenz: 0.4 }, // anderes ProteinChange
          { gen: "EGFR", proteinChange: "L861Q", pathway: "Growth Signaling", allelfrequenz: 0.36 }, // anderes ProteinChange
          { gen: "PIK3CA", proteinChange: "H1047R", pathway: "Growth Signaling", allelfrequenz: 0.38 }, // identisch
          { gen: "ERBB2", proteinChange: "G776S", pathway: "Growth Signaling", allelfrequenz: 0.32 }, // gleicher Pathway
          { gen: "FGFR3", proteinChange: "Y373C", pathway: "Growth Signaling", allelfrequenz: 0.34 }, // gleicher Pathway
          { gen: "NOTCH1", proteinChange: "L1576P", pathway: "Cell Cycle", allelfrequenz: 0.3 }, // gleicher Pathway
          { gen: "SMAD4", proteinChange: "R361C", pathway: "TGF-Beta Signaling", allelfrequenz: 0.28 }, // gleicher Pathway
          { gen: "IDH1", proteinChange: "G97D", pathway: "Metabolism", allelfrequenz: 0.35 }, // gleicher Pathway
          // 10 weitere Mutationen ohne Überschneidung
          { gen: "SMO", proteinChange: "W535L", pathway: "Hedgehog Signaling", allelfrequenz: 0.31 },
          { gen: "FLT3", proteinChange: "D835Y", pathway: "Growth Signaling", allelfrequenz: 0.32 },
          { gen: "CCND1", proteinChange: "P287L", pathway: "Cell Cycle", allelfrequenz: 0.34 },
          { gen: "EZH2", proteinChange: "Y641N", pathway: "Epigenetic Regulation", allelfrequenz: 0.33 },
          { gen: "GNAS", proteinChange: "R201C", pathway: "G Protein Signaling", allelfrequenz: 0.37 },
          { gen: "TET2", proteinChange: "R550X", pathway: "DNA Methylation", allelfrequenz: 0.36 },
          { gen: "FOXL2", proteinChange: "C134W", pathway: "Transcription Regulation", allelfrequenz: 0.38 },
          { gen: "SDHB", proteinChange: "R46Q", pathway: "Metabolism", allelfrequenz: 0.3 },
          { gen: "DNMT3A", proteinChange: "R882H", pathway: "DNA Methylation", allelfrequenz: 0.35 },
          { gen: "WT1", proteinChange: "R394W", pathway: "Transcription Regulation", allelfrequenz: 0.4 },
        ],
        therapien: [
          { name: "Therapie A", reasoning: "", comment: "" },
          { name: "Therapie B", reasoning: "", comment: "" },
        ],
      },
];


const SharedMutationsTable: React.FC = () => {
  const referencePatient = patients[0];
  const similarPatients = patients
    .filter((p) => p.id !== "1")
    .sort((a, b) => (b.similarityScore ?? 0) - (a.similarityScore ?? 0));

  const [currentIndex, setCurrentIndex] = useState(0);
  const selectedPatient = similarPatients[currentIndex];

  const { sharedMutations, distinctMutations } = getSharedAndDistinctMutations(
    referencePatient,
    selectedPatient
  );

  // Definieren der festen Spaltenbreiten
  const columnWidths = {
    pathway: '250px',
    gene: '120px',
    proteinChange: '150px',
    alleleFreq: '90px',
  };

  const handleNextPatient = () => {
    setCurrentIndex((prevIndex) => (prevIndex + 1) % similarPatients.length);
  };

  const handlePreviousPatient = () => {
    setCurrentIndex((prevIndex) => (prevIndex - 1 + similarPatients.length) % similarPatients.length);
  };

  return (
    <div className="shared-mutations-container">
      <div className="shared-mutations-buttons">
        <button className="shared-mutations-button" onClick={handlePreviousPatient}>
          Previous Patient
        </button>
        <button className="shared-mutations-button" onClick={handleNextPatient}>
          Next Patient
        </button>
      </div>

      <h1>
        Shared Mutations between {referencePatient.id} and {selectedPatient.id}
      </h1>

      <div className="shared-mutations-tables">
        {/* Shared Mutations Table */}
        <table className="shared-mutations-table">
          <thead>
            <tr>
              <th className="table-column-border" style={{ width: '250px' }}>
                Pathway
              </th>
              <th colSpan={3} className="table-column-border">
                Current Patient ({referencePatient.id})
              </th>
              <th colSpan={3} className="column-header">
                Similar Patient ({selectedPatient.id})
              </th>
            </tr>
            <tr>
              <th className="table-column-border" style={{ width: '250px' }}>
                Pathway
              </th>
              <th style={{ width: '120px' }}>Gene</th>
              <th style={{ width: '150px' }}>Protein Change</th>
              <th className="table-column-border" style={{ width: '90px' }}>
                Allele Freq
              </th>
              <th style={{ width: '120px' }}>Gene</th>
              <th style={{ width: '150px' }}>Protein Change</th>
              <th style={{ width: '90px' }}>Allele Freq</th>
            </tr>
          </thead>
          <tbody>
            {sharedMutations.identical.map((mut, index) => (
              <tr key={index} className={index % 2 === 0 ? 'row-background-odd' : 'row-background-even'}>
                <td className="table-column-border">{mut.pathway}</td>
                <td>{mut.gen}</td>
                <td>{mut.proteinChange}</td>
                <td className="table-column-border">{mut.allelfrequenz.toFixed(2)}</td>
                <td>{mut.gen}</td>
                <td>{mut.proteinChange}</td>
                <td>{mut.allelfrequenz.toFixed(2)}</td>
              </tr>
            ))}
            {sharedMutations.sameGene.map((mut, index) => (
              <tr key={index} className={index % 2 === 0 ? 'row-background-same-gene-odd' : 'row-background-same-gene-even'}>
                <td className="table-column-border">{mut.pathway}</td>
                <td>{mut.gen}</td>
                <td>{mut.proteinChange}</td>
                <td className="table-column-border">{mut.allelfrequenz.toFixed(2)}</td>
                <td>{mut.gen}</td>
                <td>{selectedPatient.mutationen.find((m) => m.gen === mut.gen)?.proteinChange}</td>
                <td>{selectedPatient.mutationen.find((m) => m.gen === mut.gen)?.allelfrequenz.toFixed(2)}</td>
              </tr>
            ))}
            {sharedMutations.samePathway.map((mut, index) => (
              <tr key={index} className={index % 2 === 0 ? 'row-background-same-pathway-odd' : 'row-background-same-pathway-even'}>
                <td className="table-column-border">{mut.pathway}</td>
                <td>{mut.gen}</td>
                <td>{mut.proteinChange}</td>
                <td className="table-column-border">{mut.allelfrequenz.toFixed(2)}</td>
                <td>{selectedPatient.mutationen.find((m) => m.pathway === mut.pathway)?.gen}</td>
                <td>{selectedPatient.mutationen.find((m) => m.pathway === mut.pathway)?.proteinChange}</td>
                <td>{selectedPatient.mutationen.find((m) => m.pathway === mut.pathway)?.allelfrequenz.toFixed(2)}</td>
              </tr>
            ))}
          </tbody>
        </table>

        <table className="additional-info-table">
          <tbody>
            <tr>
              <td>Age (Years)</td>
              <td className="column-separator">{selectedPatient.alter}</td>
            </tr>
            <tr>
              <td>Overall survival (Months)</td>
              <td className="column-separator">{selectedPatient.overallSurvival}</td>
            </tr>
            <tr>
              <td>Ort der Gewebeentnahme</td>
              <td className="column-separator">{selectedPatient.ort}</td>
            </tr>
            <tr>
              <td>Art der Tumorprobe</td>
              <td className="column-separator">{selectedPatient.artDerTumorprobe}</td>
            </tr>
            {selectedPatient.therapien.length > 0 && (
              <>
                <tr>
                  <td>Therapy</td>
                  <td className="column-separator">{selectedPatient.therapien[0].name}</td>
                </tr>
                <tr>
                  <td>Reasoning</td>
                  <td className="column-separator">{selectedPatient.therapien[0].reasoning || ""}</td>
                </tr>
                <tr>
                  <td>Comment</td>
                  <td className="column-separator">{selectedPatient.therapien[0].comment || ""}</td>
                </tr>
              </>
            )}
          </tbody>
        </table>
      </div>
        
      <h2>Distinct mutations of {referencePatient.id} and {selectedPatient.id}</h2>

{/* Distinct Mutations Table */}
<table className="shared-mutations-table" style={{ marginLeft: columnWidths.pathway }}>
  <thead>
    <tr>
      <th colSpan={3} className="table-column-border" style={{ textAlign: 'center' }}>
        Current Patient ({referencePatient.id})
      </th>
      <th colSpan={3} style={{ textAlign: 'center' }}>
        Similar Patient ({selectedPatient.id})
      </th>
    </tr>
    <tr>
      <th style={{ width: columnWidths.gene }}>Gene</th>
      <th style={{ width: columnWidths.proteinChange }}>Protein Change</th>
      <th className="table-column-border" style={{ width: columnWidths.alleleFreq }}>Allele Freq</th>
      <th style={{ width: columnWidths.gene }}>Gene</th>
      <th style={{ width: columnWidths.proteinChange }}>Protein Change</th>
      <th style={{ width: columnWidths.alleleFreq }}>Allele Freq</th>
    </tr>
  </thead>
  <tbody>
    {distinctMutations.patient1.map((mut, index) => (
      <tr key={index} className={index % 2 === 0 ? 'row-background-default-odd' : 'row-background-default-even'}>
        <td style={{ width: columnWidths.gene }}>{mut.gen}</td>
        <td style={{ width: columnWidths.proteinChange }}>{mut.proteinChange}</td>
        <td className="table-column-border" style={{ width: columnWidths.alleleFreq }}>{mut.allelfrequenz.toFixed(2)}</td>
        <td style={{ width: columnWidths.gene }}>{distinctMutations.patient2[index]?.gen || ''}</td>
        <td style={{ width: columnWidths.proteinChange }}>{distinctMutations.patient2[index]?.proteinChange || ''}</td>
        <td style={{ width: columnWidths.alleleFreq }}>{distinctMutations.patient2[index]?.allelfrequenz?.toFixed(2) || ''}</td>
      </tr>
    ))}
    {distinctMutations.patient2.length > distinctMutations.patient1.length &&
      distinctMutations.patient2
        .slice(distinctMutations.patient1.length)
        .map((mut, index) => (
          <tr key={index} className={index % 2 === 0 ? 'row-background-default-odd' : 'row-background-default-even'}>
            <td style={{ width: columnWidths.gene }}></td>
            <td style={{ width: columnWidths.proteinChange }}></td>
            <td className="table-column-border" style={{ width: columnWidths.alleleFreq }}></td>
            <td style={{ width: columnWidths.gene }}>{mut.gen}</td>
            <td style={{ width: columnWidths.proteinChange }}>{mut.proteinChange}</td>
            <td style={{ width: columnWidths.alleleFreq }}>{mut.allelfrequenz.toFixed(2)}</td>
          </tr>
        ))}
  </tbody>
</table>
</div>
  );
};

export default SharedMutationsTable;

