import { assert } from 'chai';
import _ from 'lodash';
import { PatientSurvival } from 'shared/model/PatientSurvival';
import {
    convertScatterDataToDownloadData,
    downSampling,
    filterScatterData,
    getDownloadContent,
    getSurvivalSummaries,
    getLineData,
    getMedian,
    getScatterData,
    getScatterDataWithOpacity,
    getStats,
    ScatterData,
    parseSurvivalData,
    getSurvivalStatusBoolean,
    isNullSurvivalClinicalDataValue,
    createSurvivalAttributeIdsDict,
    getNumPatientsAtRisk,
    sortPatientSurvivals,
    floorScatterData,
    generateStudyViewSurvivalPlotTitle,
} from './SurvivalUtil';

const exampleAlteredPatientSurvivals = [
    {
        uniquePatientKey: 'TCGA-OR-A5J2',
        patientId: 'TCGA-OR-A5J2',
        studyId: 'acc_tcga',
        entryMonths: 0,
        months: 0.09,
        status: true,
    },
    {
        uniquePatientKey: 'TCGA-OR-A5J1',
        patientId: 'TCGA-OR-A5J1',
        studyId: 'acc_tcga',
        entryMonths: 0,
        months: 5.09,
        status: false,
    },
];

const exampleUnalteredPatientSurvivals = [
    {
        uniquePatientKey: 'TCGA-OR-A5J3',
        patientId: 'TCGA-OR-A5J3',
        studyId: 'acc_tcga',
        entryMonths: 0,
        months: 0,
        status: false,
    },
    {
        uniquePatientKey: 'TCGA-2F-A9KP',
        patientId: 'TCGA-2F-A9KP',
        studyId: 'blca_tcga',
        entryMonths: 0,
        months: 0.13,
        status: false,
    },
    {
        uniquePatientKey: 'TCGA-2F-A9KO',
        patientId: 'TCGA-2F-A9KO',
        studyId: 'blca_tcga',
        entryMonths: 0,
        months: 63.83,
        status: true,
    },
    {
        uniquePatientKey: 'TCGA-2F-A9KQ',
        patientId: 'TCGA-2F-A9KQ',
        studyId: 'blca_tcga',
        entryMonths: 0,
        months: 182.19,
        status: true,
    },
];

const allScatterData: ScatterData[] = [
    {
        x: 0,
        y: 10,
        patientId: '',
        uniquePatientKey: '',
        studyId: '',
        status: true,
    },
    {
        x: 0.5,
        y: 9,
        patientId: '',
        uniquePatientKey: '',
        studyId: '',
        status: true,
    },
    {
        x: 1,
        y: 8,
        patientId: '',
        uniquePatientKey: '',
        studyId: '',
        status: true,
    },
    {
        x: 0,
        y: 10,
        patientId: '',
        uniquePatientKey: '',
        studyId: '',
        status: true,
    },
    {
        x: 0,
        y: 10,
        patientId: '',
        uniquePatientKey: '',
        studyId: '',
        status: true,
    },
];

// data from gbm_columbia_2019 study
const cbioExamplePatientSurvivals = [
    {
        months: 1.4,
        entryMonths: 0,
        status: false,
    },
    {
        months: 4.3,
        entryMonths: 0,
        status: false,
    },
    {
        months: 4.5,
        entryMonths: 0,
        status: false,
    },
    {
        months: 8,
        entryMonths: 0,
        status: false,
    },
    {
        months: 8.9,
        entryMonths: 0,
        status: true,
    },
    {
        months: 9.2,
        entryMonths: 0,
        status: false,
    },
    {
        months: 13.1,
        entryMonths: 0,
        status: false,
    },
    {
        months: 14.4,
        entryMonths: 0,
        status: true,
    },
    {
        months: 14.8,
        entryMonths: 0,
        status: false,
    },
    {
        months: 14.8,
        entryMonths: 0,
        status: false,
    },
    {
        months: 15.3,
        entryMonths: 0,
        status: false,
    },
    {
        months: 15.9,
        entryMonths: 0,
        status: false,
    },
    {
        months: 19.6,
        entryMonths: 0,
        status: false,
    },
    {
        months: 20.6,
        entryMonths: 0,
        status: true,
    },
    {
        months: 23,
        entryMonths: 0,
        status: false,
    },
    {
        months: 23,
        entryMonths: 0,
        status: false,
    },
    {
        months: 24,
        entryMonths: 0,
        status: true,
    },
    {
        months: 25.4,
        entryMonths: 0,
        status: true,
    },
    {
        months: 26.6,
        entryMonths: 0,
        status: true,
    },
    {
        months: 31.3,
        entryMonths: 0,
        status: false,
    },
    {
        months: 34,
        entryMonths: 0,
        status: false,
    },
    {
        months: 36.6,
        entryMonths: 0,
        status: true,
    },
    {
        months: 36.9,
        entryMonths: 0,
        status: false,
    },
    {
        months: 43.1,
        entryMonths: 0,
        status: false,
    },
    {
        months: 45.4,
        entryMonths: 0,
        status: true,
    },
    {
        months: 45.6,
        entryMonths: 0,
        status: false,
    },
    {
        months: 46.5,
        entryMonths: 0,
        status: true,
    },
    {
        months: 46.8,
        entryMonths: 0,
        status: true,
    },
    {
        months: 55.6,
        entryMonths: 0,
        status: false,
    },
] as PatientSurvival[];

// ground truth from R survival package
/*
library(survival)
time = c(1.4,4.3,4.5,8,8.9,9.2,13.1,14.4,14.8,14.8,15.3,15.9,19.6,20.6,23,23,24,25.4,26.6,31.3,34,36.6,36.9,43.1,45.4,45.6,46.5,46.8,55.6)
status = c(1,1,1,1,2,1,1,2,1,1,1,1,1,2,1,1,2,2,2,1,1,2,1,1,2,1,2,2,1)
summary(survfit(Surv(time, status) ~ 1))
*/
const cbioExampleSurvivalSummaries = [
    {
        survivalFunctionEstimate: 1,
        atRisk: 29,
    },
    {
        survivalFunctionEstimate: 1,
        atRisk: 28,
    },
    {
        survivalFunctionEstimate: 1,
        atRisk: 27,
    },
    {
        survivalFunctionEstimate: 1,
        atRisk: 26,
    },
    {
        survivalFunctionEstimate: 0.96,
        standardError: 0.03919183588453085,
        low95ConfidenceInterval: 0.8861782272999308,
        high95ConfidenceInterval: 1,
        atRisk: 25,
    },
    {
        survivalFunctionEstimate: 0.96,
        atRisk: 24,
    },
    {
        survivalFunctionEstimate: 0.96,
        atRisk: 23,
    },
    {
        survivalFunctionEstimate: 0.9163636363636364,
        standardError: 0.05671964347934269,
        low95ConfidenceInterval: 0.8116737574073527,
        high95ConfidenceInterval: 1,
        atRisk: 22,
    },
    {
        survivalFunctionEstimate: 0.9163636363636364,
        atRisk: 21,
    },
    {
        survivalFunctionEstimate: 0.9163636363636364,
        atRisk: 20,
    },
    {
        survivalFunctionEstimate: 0.9163636363636364,
        atRisk: 19,
    },
    {
        survivalFunctionEstimate: 0.9163636363636364,
        atRisk: 18,
    },
    {
        survivalFunctionEstimate: 0.9163636363636364,
        atRisk: 17,
    },
    {
        survivalFunctionEstimate: 0.8590909090909091,
        standardError: 0.07682903121584796,
        low95ConfidenceInterval: 0.7209674254729507,
        high95ConfidenceInterval: 1,
        atRisk: 16,
    },
    {
        survivalFunctionEstimate: 0.8590909090909091,
        atRisk: 15,
    },
    {
        survivalFunctionEstimate: 0.8590909090909091,
        atRisk: 14,
    },
    {
        survivalFunctionEstimate: 0.7930069930069931,
        standardError: 0.09518757265448993,
        low95ConfidenceInterval: 0.6267641342567718,
        high95ConfidenceInterval: 1,
        atRisk: 13,
    },
    {
        survivalFunctionEstimate: 0.7269230769230769,
        standardError: 0.10778053305175259,
        low95ConfidenceInterval: 0.5436022117301463,
        high95ConfidenceInterval: 0.9720658753052113,
        atRisk: 12,
    },
    {
        survivalFunctionEstimate: 0.6608391608391608,
        standardError: 0.11649295252831449,
        low95ConfidenceInterval: 0.4677843335458505,
        high95ConfidenceInterval: 0.9335678114491659,
        atRisk: 11,
    },
    {
        survivalFunctionEstimate: 0.6608391608391608,
        atRisk: 10,
    },
    {
        survivalFunctionEstimate: 0.6608391608391608,
        atRisk: 9,
    },
    {
        survivalFunctionEstimate: 0.5782342657342657,
        standardError: 0.12790863672863112,
        low95ConfidenceInterval: 0.37481184465886863,
        high95ConfidenceInterval: 0.8920605654113074,
        atRisk: 8,
    },
    {
        survivalFunctionEstimate: 0.5782342657342657,
        atRisk: 7,
    },
    {
        survivalFunctionEstimate: 0.5782342657342657,
        atRisk: 6,
    },
    {
        survivalFunctionEstimate: 0.4625874125874126,
        standardError: 0.1454996635668725,
        low95ConfidenceInterval: 0.24972477313930544,
        high95ConfidenceInterval: 0.8568918157148447,
        atRisk: 5,
    },
    {
        survivalFunctionEstimate: 0.4625874125874126,
        atRisk: 4,
    },
    {
        survivalFunctionEstimate: 0.3083916083916084,
        standardError: 0.15893348873798227,
        low95ConfidenceInterval: 0.11231173743871747,
        high95ConfidenceInterval: 0.8467982625436375,
        atRisk: 3,
    },
    {
        survivalFunctionEstimate: 0.1541958041958042,
        standardError: 0.13491899968648588,
        low95ConfidenceInterval: 0.0277513466677814,
        high95ConfidenceInterval: 0.8567636848843272,
        atRisk: 2,
    },
    {
        survivalFunctionEstimate: 0.1541958041958042,
        atRisk: 1,
    },
];

// data from lung dataset: https://rdrr.io/cran/survival/man/lung.html
const largeExamplePatientSurvivalsFromRSurvivalPackage = [
    {
        patientId: '57',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '57',
        months: 5,
        status: true,
    },
    {
        patientId: '73',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '73',
        months: 11,
        status: true,
    },
    {
        patientId: '79',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '79',
        months: 11,
        status: true,
    },
    {
        patientId: '108',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '108',
        months: 11,
        status: true,
    },
    {
        patientId: '30',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '30',
        months: 12,
        status: true,
    },
    {
        patientId: '116',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '116',
        months: 13,
        status: true,
    },
    {
        patientId: '215',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '215',
        months: 13,
        status: true,
    },
    {
        patientId: '111',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '111',
        months: 15,
        status: true,
    },
    {
        patientId: '32',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '32',
        months: 26,
        status: true,
    },
    {
        patientId: '96',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '96',
        months: 30,
        status: true,
    },
    {
        patientId: '149',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '149',
        months: 31,
        status: true,
    },
    {
        patientId: '35',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '35',
        months: 53,
        status: true,
    },
    {
        patientId: '53',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '53',
        months: 53,
        status: true,
    },
    {
        patientId: '124',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '124',
        months: 54,
        status: true,
    },
    {
        patientId: '202',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '202',
        months: 59,
        status: true,
    },
    {
        patientId: '64',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '64',
        months: 60,
        status: true,
    },
    {
        patientId: '128',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '128',
        months: 60,
        status: true,
    },
    {
        patientId: '19',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '19',
        months: 61,
        status: true,
    },
    {
        patientId: '144',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '144',
        months: 62,
        status: true,
    },
    {
        patientId: '56',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '56',
        months: 65,
        status: true,
    },
    {
        patientId: '66',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '66',
        months: 65,
        status: true,
    },
    {
        patientId: '14',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '14',
        months: 71,
        status: true,
    },
    {
        patientId: '201',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '201',
        months: 79,
        status: true,
    },
    {
        patientId: '22',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '22',
        months: 81,
        status: true,
    },
    {
        patientId: '192',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '192',
        months: 81,
        status: true,
    },
    {
        patientId: '20',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '20',
        months: 88,
        status: true,
    },
    {
        patientId: '93',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '93',
        months: 88,
        status: true,
    },
    {
        patientId: '127',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '127',
        months: 92,
        status: true,
    },
    {
        patientId: '214',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '214',
        months: 92,
        status: false,
    },
    {
        patientId: '39',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '39',
        months: 93,
        status: true,
    },
    {
        patientId: '46',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '46',
        months: 95,
        status: true,
    },
    {
        patientId: '82',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '82',
        months: 95,
        status: true,
    },
    {
        patientId: '206',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '206',
        months: 105,
        status: true,
    },
    {
        patientId: '226',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '226',
        months: 105,
        status: false,
    },
    {
        patientId: '34',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '34',
        months: 107,
        status: true,
    },
    {
        patientId: '104',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '104',
        months: 107,
        status: true,
    },
    {
        patientId: '189',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '189',
        months: 110,
        status: true,
    },
    {
        patientId: '223',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '223',
        months: 116,
        status: true,
    },
    {
        patientId: '28',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '28',
        months: 118,
        status: true,
    },
    {
        patientId: '36',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '36',
        months: 122,
        status: true,
    },
    {
        patientId: '193',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '193',
        months: 131,
        status: true,
    },
    {
        patientId: '58',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '58',
        months: 132,
        status: true,
    },
    {
        patientId: '74',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '74',
        months: 132,
        status: true,
    },
    {
        patientId: '200',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '200',
        months: 135,
        status: true,
    },
    {
        patientId: '164',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '164',
        months: 142,
        status: true,
    },
    {
        patientId: '16',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '16',
        months: 144,
        status: true,
    },
    {
        patientId: '44',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '44',
        months: 145,
        status: true,
    },
    {
        patientId: '179',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '179',
        months: 145,
        status: true,
    },
    {
        patientId: '88',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '88',
        months: 147,
        status: true,
    },
    {
        patientId: '42',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '42',
        months: 153,
        status: true,
    },
    {
        patientId: '106',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '106',
        months: 156,
        status: true,
    },
    {
        patientId: '155',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '155',
        months: 156,
        status: true,
    },
    {
        patientId: '65',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '65',
        months: 163,
        status: true,
    },
    {
        patientId: '90',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '90',
        months: 163,
        status: true,
    },
    {
        patientId: '148',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '148',
        months: 163,
        status: true,
    },
    {
        patientId: '10',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '10',
        months: 166,
        status: true,
    },
    {
        patientId: '100',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '100',
        months: 166,
        status: true,
    },
    {
        patientId: '84',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '84',
        months: 167,
        status: true,
    },
    {
        patientId: '11',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '11',
        months: 170,
        status: true,
    },
    {
        patientId: '210',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '210',
        months: 173,
        status: false,
    },
    {
        patientId: '227',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '227',
        months: 174,
        status: false,
    },
    {
        patientId: '63',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '63',
        months: 175,
        status: true,
    },
    {
        patientId: '220',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '220',
        months: 175,
        status: false,
    },
    {
        patientId: '80',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '80',
        months: 176,
        status: true,
    },
    {
        patientId: '105',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '105',
        months: 177,
        status: true,
    },
    {
        patientId: '228',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '228',
        months: 177,
        status: false,
    },
    {
        patientId: '97',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '97',
        months: 179,
        status: true,
    },
    {
        patientId: '159',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '159',
        months: 179,
        status: true,
    },
    {
        patientId: '177',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '177',
        months: 180,
        status: true,
    },
    {
        patientId: '112',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '112',
        months: 181,
        status: true,
    },
    {
        patientId: '169',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '169',
        months: 181,
        status: true,
    },
    {
        patientId: '154',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '154',
        months: 182,
        status: true,
    },
    {
        patientId: '218',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '218',
        months: 183,
        status: true,
    },
    {
        patientId: '213',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '213',
        months: 185,
        status: false,
    },
    {
        patientId: '178',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '178',
        months: 186,
        status: true,
    },
    {
        patientId: '224',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '224',
        months: 188,
        status: false,
    },
    {
        patientId: '52',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '52',
        months: 189,
        status: true,
    },
    {
        patientId: '225',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '225',
        months: 191,
        status: false,
    },
    {
        patientId: '217',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '217',
        months: 192,
        status: false,
    },
    {
        patientId: '167',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '167',
        months: 194,
        status: true,
    },
    {
        patientId: '83',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '83',
        months: 196,
        status: false,
    },
    {
        patientId: '173',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '173',
        months: 197,
        status: true,
    },
    {
        patientId: '221',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '221',
        months: 197,
        status: false,
    },
    {
        patientId: '122',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '122',
        months: 199,
        status: true,
    },
    {
        patientId: '114',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '114',
        months: 201,
        status: true,
    },
    {
        patientId: '141',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '141',
        months: 201,
        status: true,
    },
    {
        patientId: '132',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '132',
        months: 202,
        status: true,
    },
    {
        patientId: '204',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '204',
        months: 202,
        status: false,
    },
    {
        patientId: '222',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '222',
        months: 203,
        status: false,
    },
    {
        patientId: '126',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '126',
        months: 207,
        status: true,
    },
    {
        patientId: '67',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '67',
        months: 208,
        status: true,
    },
    {
        patientId: '4',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '4',
        months: 210,
        status: true,
    },
    {
        patientId: '219',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '219',
        months: 211,
        status: false,
    },
    {
        patientId: '117',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '117',
        months: 212,
        status: true,
    },
    {
        patientId: '9',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '9',
        months: 218,
        status: true,
    },
    {
        patientId: '212',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '212',
        months: 221,
        status: false,
    },
    {
        patientId: '143',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '143',
        months: 222,
        status: true,
    },
    {
        patientId: '216',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '216',
        months: 222,
        status: false,
    },
    {
        patientId: '62',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '62',
        months: 223,
        status: true,
    },
    {
        patientId: '207',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '207',
        months: 224,
        status: false,
    },
    {
        patientId: '194',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '194',
        months: 225,
        status: false,
    },
    {
        patientId: '196',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '196',
        months: 225,
        status: false,
    },
    {
        patientId: '75',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '75',
        months: 226,
        status: true,
    },
    {
        patientId: '151',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '151',
        months: 229,
        status: true,
    },
    {
        patientId: '70',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '70',
        months: 230,
        status: true,
    },
    {
        patientId: '205',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '205',
        months: 235,
        status: false,
    },
    {
        patientId: '209',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '209',
        months: 237,
        status: false,
    },
    {
        patientId: '92',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '92',
        months: 239,
        status: true,
    },
    {
        patientId: '208',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '208',
        months: 239,
        status: true,
    },
    {
        patientId: '203',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '203',
        months: 240,
        status: false,
    },
    {
        patientId: '197',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '197',
        months: 243,
        status: false,
    },
    {
        patientId: '94',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '94',
        months: 245,
        status: true,
    },
    {
        patientId: '54',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '54',
        months: 246,
        status: true,
    },
    {
        patientId: '211',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '211',
        months: 252,
        status: false,
    },
    {
        patientId: '188',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '188',
        months: 259,
        status: false,
    },
    {
        patientId: '166',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '166',
        months: 266,
        status: false,
    },
    {
        patientId: '135',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '135',
        months: 267,
        status: true,
    },
    {
        patientId: '162',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '162',
        months: 268,
        status: true,
    },
    {
        patientId: '180',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '180',
        months: 269,
        status: false,
    },
    {
        patientId: '195',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '195',
        months: 269,
        status: true,
    },
    {
        patientId: '191',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '191',
        months: 270,
        status: true,
    },
    {
        patientId: '184',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '184',
        months: 272,
        status: false,
    },
    {
        patientId: '199',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '199',
        months: 276,
        status: false,
    },
    {
        patientId: '198',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '198',
        months: 279,
        status: false,
    },
    {
        patientId: '113',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '113',
        months: 283,
        status: true,
    },
    {
        patientId: '86',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '86',
        months: 284,
        status: true,
    },
    {
        patientId: '182',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '182',
        months: 284,
        status: false,
    },
    {
        patientId: '170',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '170',
        months: 285,
        status: true,
    },
    {
        patientId: '187',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '187',
        months: 285,
        status: true,
    },
    {
        patientId: '190',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '190',
        months: 286,
        status: true,
    },
    {
        patientId: '119',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '119',
        months: 288,
        status: true,
    },
    {
        patientId: '158',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '158',
        months: 291,
        status: true,
    },
    {
        patientId: '163',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '163',
        months: 292,
        status: false,
    },
    {
        patientId: '185',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '185',
        months: 292,
        status: false,
    },
    {
        patientId: '131',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '131',
        months: 293,
        status: true,
    },
    {
        patientId: '176',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '176',
        months: 296,
        status: false,
    },
    {
        patientId: '181',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '181',
        months: 300,
        status: false,
    },
    {
        patientId: '21',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '21',
        months: 301,
        status: true,
    },
    {
        patientId: '171',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '171',
        months: 301,
        status: false,
    },
    {
        patientId: '47',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '47',
        months: 303,
        status: true,
    },
    {
        patientId: '175',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '175',
        months: 303,
        status: false,
    },
    {
        patientId: '72',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '72',
        months: 305,
        status: true,
    },
    {
        patientId: '1',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '1',
        months: 306,
        status: true,
    },
    {
        patientId: '7',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '7',
        months: 310,
        status: true,
    },
    {
        patientId: '98',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '98',
        months: 310,
        status: true,
    },
    {
        patientId: '153',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '153',
        months: 315,
        status: false,
    },
    {
        patientId: '168',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '168',
        months: 320,
        status: true,
    },
    {
        patientId: '156',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '156',
        months: 329,
        status: true,
    },
    {
        patientId: '186',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '186',
        months: 332,
        status: false,
    },
    {
        patientId: '140',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '140',
        months: 337,
        status: true,
    },
    {
        patientId: '150',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '150',
        months: 340,
        status: true,
    },
    {
        patientId: '60',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '60',
        months: 345,
        status: true,
    },
    {
        patientId: '172',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '172',
        months: 348,
        status: true,
    },
    {
        patientId: '183',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '183',
        months: 350,
        status: true,
    },
    {
        patientId: '110',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '110',
        months: 351,
        status: true,
    },
    {
        patientId: '133',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '133',
        months: 353,
        status: true,
    },
    {
        patientId: '147',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '147',
        months: 353,
        status: true,
    },
    {
        patientId: '146',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '146',
        months: 356,
        status: false,
    },
    {
        patientId: '8',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '8',
        months: 361,
        status: true,
    },
    {
        patientId: '78',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '78',
        months: 363,
        status: true,
    },
    {
        patientId: '120',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '120',
        months: 363,
        status: true,
    },
    {
        patientId: '103',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '103',
        months: 364,
        status: true,
    },
    {
        patientId: '157',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '157',
        months: 364,
        status: false,
    },
    {
        patientId: '24',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '24',
        months: 371,
        status: true,
    },
    {
        patientId: '137',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '137',
        months: 371,
        status: true,
    },
    {
        patientId: '160',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '160',
        months: 376,
        status: false,
    },
    {
        patientId: '174',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '174',
        months: 382,
        status: false,
    },
    {
        patientId: '161',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '161',
        months: 384,
        status: false,
    },
    {
        patientId: '138',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '138',
        months: 387,
        status: true,
    },
    {
        patientId: '29',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '29',
        months: 390,
        status: true,
    },
    {
        patientId: '25',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '25',
        months: 394,
        status: true,
    },
    {
        patientId: '142',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '142',
        months: 404,
        status: false,
    },
    {
        patientId: '165',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '165',
        months: 413,
        status: false,
    },
    {
        patientId: '76',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '76',
        months: 426,
        status: true,
    },
    {
        patientId: '69',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '69',
        months: 428,
        status: true,
    },
    {
        patientId: '109',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '109',
        months: 429,
        status: true,
    },
    {
        patientId: '43',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '43',
        months: 433,
        status: true,
    },
    {
        patientId: '121',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '121',
        months: 442,
        status: true,
    },
    {
        patientId: '61',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '61',
        months: 444,
        status: true,
    },
    {
        patientId: '152',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '152',
        months: 444,
        status: false,
    },
    {
        patientId: '102',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '102',
        months: 450,
        status: true,
    },
    {
        patientId: '2',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '2',
        months: 455,
        status: true,
    },
    {
        patientId: '139',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '139',
        months: 457,
        status: true,
    },
    {
        patientId: '145',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '145',
        months: 458,
        status: false,
    },
    {
        patientId: '41',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '41',
        months: 460,
        status: true,
    },
    {
        patientId: '31',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '31',
        months: 473,
        status: true,
    },
    {
        patientId: '99',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '99',
        months: 477,
        status: true,
    },
    {
        patientId: '134',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '134',
        months: 511,
        status: false,
    },
    {
        patientId: '136',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '136',
        months: 511,
        status: false,
    },
    {
        patientId: '48',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '48',
        months: 519,
        status: true,
    },
    {
        patientId: '26',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '26',
        months: 520,
        status: true,
    },
    {
        patientId: '115',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '115',
        months: 524,
        status: true,
    },
    {
        patientId: '118',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '118',
        months: 524,
        status: true,
    },
    {
        patientId: '107',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '107',
        months: 529,
        status: false,
    },
    {
        patientId: '33',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '33',
        months: 533,
        status: true,
    },
    {
        patientId: '130',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '130',
        months: 543,
        status: false,
    },
    {
        patientId: '123',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '123',
        months: 550,
        status: true,
    },
    {
        patientId: '129',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '129',
        months: 551,
        status: false,
    },
    {
        patientId: '125',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '125',
        months: 558,
        status: true,
    },
    {
        patientId: '101',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '101',
        months: 559,
        status: false,
    },
    {
        patientId: '15',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '15',
        months: 567,
        status: true,
    },
    {
        patientId: '27',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '27',
        months: 574,
        status: true,
    },
    {
        patientId: '45',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '45',
        months: 583,
        status: true,
    },
    {
        patientId: '95',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '95',
        months: 588,
        status: false,
    },
    {
        patientId: '17',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '17',
        months: 613,
        status: true,
    },
    {
        patientId: '23',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '23',
        months: 624,
        status: true,
    },
    {
        patientId: '87',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '87',
        months: 641,
        status: true,
    },
    {
        patientId: '49',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '49',
        months: 643,
        status: true,
    },
    {
        patientId: '12',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '12',
        months: 654,
        status: true,
    },
    {
        patientId: '91',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '91',
        months: 655,
        status: true,
    },
    {
        patientId: '59',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '59',
        months: 687,
        status: true,
    },
    {
        patientId: '55',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '55',
        months: 689,
        status: true,
    },
    {
        patientId: '77',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '77',
        months: 705,
        status: true,
    },
    {
        patientId: '18',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '18',
        months: 707,
        status: true,
    },
    {
        patientId: '13',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '13',
        months: 728,
        status: true,
    },
    {
        patientId: '40',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '40',
        months: 731,
        status: true,
    },
    {
        patientId: '51',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '51',
        months: 735,
        status: true,
    },
    {
        patientId: '89',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '89',
        months: 740,
        status: false,
    },
    {
        patientId: '50',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '50',
        months: 765,
        status: true,
    },
    {
        patientId: '81',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '81',
        months: 791,
        status: true,
    },
    {
        patientId: '85',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '85',
        months: 806,
        status: false,
    },
    {
        patientId: '37',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '37',
        months: 814,
        status: true,
    },
    {
        patientId: '68',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '68',
        months: 821,
        status: false,
    },
    {
        patientId: '71',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '71',
        months: 840,
        status: false,
    },
    {
        patientId: '5',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '5',
        months: 883,
        status: true,
    },
    {
        patientId: '38',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '38',
        months: 965,
        status: false,
    },
    {
        patientId: '3',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '3',
        months: 1010,
        status: false,
    },
    {
        patientId: '6',
        studyId: '1',
        entryMonths: 0,
        uniquePatientKey: '6',
        months: 1022,
        status: false,
    },
];

// ground truth from R survival package
/*
library(survival)
summary(survfit(Surv(time, status) ~ 1, data=lung))
*/
const largeExampleSurvivalSummaries = [
    {
        survivalFunctionEstimate: 0.9956140350877193,
        standardError: 0.004376335998553152,
        low95ConfidenceInterval: 0.9870734167405145,
        high95ConfidenceInterval: 1,
        atRisk: 228,
    },
    {
        survivalFunctionEstimate: 0.9912280701754386,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 227,
    },
    {
        survivalFunctionEstimate: 0.9868421052631579,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 226,
    },
    {
        survivalFunctionEstimate: 0.9824561403508771,
        standardError: 0.008694642592690387,
        low95ConfidenceInterval: 0.9655618970753633,
        high95ConfidenceInterval: 0.9996459788199428,
        atRisk: 225,
    },
    {
        survivalFunctionEstimate: 0.9780701754385964,
        standardError: 0.00969918321667634,
        low95ConfidenceInterval: 0.9592436769100913,
        high95ConfidenceInterval: 0.9972661703269686,
        atRisk: 224,
    },
    {
        survivalFunctionEstimate: 0.9736842105263157,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 223,
    },
    {
        survivalFunctionEstimate: 0.9692982456140351,
        standardError: 0.011424649532723746,
        low95ConfidenceInterval: 0.9471630031311906,
        high95ConfidenceInterval: 0.9919507897209449,
        atRisk: 222,
    },
    {
        survivalFunctionEstimate: 0.9649122807017544,
        standardError: 0.012185800489276179,
        low95ConfidenceInterval: 0.9413217146007166,
        high95ConfidenceInterval: 0.9890940525514066,
        atRisk: 221,
    },
    {
        survivalFunctionEstimate: 0.9605263157894737,
        standardError: 0.012895584798847694,
        low95ConfidenceInterval: 0.9355810726270365,
        high95ConfidenceInterval: 0.9861366698382243,
        atRisk: 220,
    },
    {
        survivalFunctionEstimate: 0.956140350877193,
        standardError: 0.01356206983336428,
        low95ConfidenceInterval: 0.9299252668892969,
        high95ConfidenceInterval: 0.9830944519162026,
        atRisk: 219,
    },
    {
        survivalFunctionEstimate: 0.9517543859649122,
        standardError: 0.014191357445529183,
        low95ConfidenceInterval: 0.9243423391699586,
        high95ConfidenceInterval: 0.979979357017088,
        atRisk: 218,
    },
    {
        survivalFunctionEstimate: 0.9473684210526315,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 217,
    },
    {
        survivalFunctionEstimate: 0.9429824561403508,
        standardError: 0.015356372406043017,
        low95ConfidenceInterval: 0.9133597795943134,
        high95ConfidenceInterval: 0.9735658745379081,
        atRisk: 216,
    },
    {
        survivalFunctionEstimate: 0.9385964912280701,
        standardError: 0.015898956972633615,
        low95ConfidenceInterval: 0.9079467094428935,
        high95ConfidenceInterval: 0.9702809252827124,
        atRisk: 215,
    },
    {
        survivalFunctionEstimate: 0.9342105263157895,
        standardError: 0.016418488032799494,
        low95ConfidenceInterval: 0.9025787990057462,
        high95ConfidenceInterval: 0.9669508174140794,
        atRisk: 214,
    },
    {
        survivalFunctionEstimate: 0.9298245614035088,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 213,
    },
    {
        survivalFunctionEstimate: 0.9254385964912281,
        standardError: 0.017396561544658157,
        low95ConfidenceInterval: 0.8919624428310942,
        high95ConfidenceInterval: 0.9601711403423211,
        atRisk: 212,
    },
    {
        survivalFunctionEstimate: 0.9210526315789473,
        standardError: 0.017858444540481477,
        low95ConfidenceInterval: 0.8867074506937553,
        high95ConfidenceInterval: 0.9567281175710984,
        atRisk: 211,
    },
    {
        survivalFunctionEstimate: 0.9166666666666666,
        standardError: 0.018304070212790808,
        low95ConfidenceInterval: 0.8814842999075682,
        high95ConfidenceInterval: 0.9532532546137108,
        atRisk: 210,
    },
    {
        survivalFunctionEstimate: 0.9122807017543859,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 209,
    },
    {
        survivalFunctionEstimate: 0.9078947368421052,
        standardError: 0.01915104821194575,
        low95ConfidenceInterval: 0.8711247066201793,
        high95ConfidenceInterval: 0.9462168239764874,
        atRisk: 208,
    },
    {
        survivalFunctionEstimate: 0.9035087719298245,
        standardError: 0.019554318293626145,
        low95ConfidenceInterval: 0.8659845059869966,
        high95ConfidenceInterval: 0.942659014463242,
        atRisk: 207,
    },
    {
        survivalFunctionEstimate: 0.8991228070175438,
        standardError: 0.019945208381001268,
        low95ConfidenceInterval: 0.8608685484842601,
        high95ConfidenceInterval: 0.9390769630537713,
        atRisk: 206,
    },
    {
        survivalFunctionEstimate: 0.894736842105263,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 205,
    },
    {
        survivalFunctionEstimate: 0.8903508771929823,
        standardError: 0.020692632882034462,
        low95ConfidenceInterval: 0.850703906610012,
        high95ConfidenceInterval: 0.9318455908792737,
        atRisk: 204,
    },
    {
        survivalFunctionEstimate: 0.8859649122807016,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 203,
    },
    {
        survivalFunctionEstimate: 0.8815789473684209,
        standardError: 0.02139821960592849,
        low95ConfidenceInterval: 0.8406211828565318,
        high95ConfidenceInterval: 0.9245323057435421,
        atRisk: 202,
    },
    {
        survivalFunctionEstimate: 0.8771929824561402,
        standardError: 0.021736606481725963,
        low95ConfidenceInterval: 0.8356080266650664,
        high95ConfidenceInterval: 0.9208474594736283,
        atRisk: 201,
    },
    {
        survivalFunctionEstimate: 0.8771929824561402,
        atRisk: 200,
    },
    {
        survivalFunctionEstimate: 0.8727849775191746,
        standardError: 0.022069805522652957,
        low95ConfidenceInterval: 0.8305833667271929,
        high95ConfidenceInterval: 0.9171308353846988,
        atRisk: 199,
    },
    {
        survivalFunctionEstimate: 0.868376972582209,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 198,
    },
    {
        survivalFunctionEstimate: 0.8639689676452434,
        standardError: 0.022710230434161833,
        low95ConfidenceInterval: 0.8205848920812573,
        high95ConfidenceInterval: 0.9096467461895121,
        atRisk: 197,
    },
    {
        survivalFunctionEstimate: 0.859560962708278,
        standardError: 0.023018178956455825,
        low95ConfidenceInterval: 0.8156096611974171,
        high95ConfidenceInterval: 0.9058806973022666,
        atRisk: 196,
    },
    {
        survivalFunctionEstimate: 0.859560962708278,
        atRisk: 195,
    },
    {
        survivalFunctionEstimate: 0.8551302360963796,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 194,
    },
    {
        survivalFunctionEstimate: 0.8506995094844813,
        standardError: 0.023618347014532283,
        low95ConfidenceInterval: 0.8056453359016672,
        high95ConfidenceInterval: 0.8982732515011627,
        atRisk: 193,
    },
    {
        survivalFunctionEstimate: 0.8462687828725829,
        standardError: 0.02390731788957698,
        low95ConfidenceInterval: 0.8006849239055442,
        high95ConfidenceInterval: 0.8944477802471132,
        atRisk: 192,
    },
    {
        survivalFunctionEstimate: 0.8418380562606846,
        standardError: 0.024189236295403024,
        low95ConfidenceInterval: 0.7957383140762694,
        high95ConfidenceInterval: 0.8906085083906636,
        atRisk: 191,
    },
    {
        survivalFunctionEstimate: 0.8374073296487863,
        standardError: 0.024464346043651622,
        low95ConfidenceInterval: 0.7908050285896571,
        high95ConfidenceInterval: 0.8867559137808481,
        atRisk: 190,
    },
    {
        survivalFunctionEstimate: 0.8329766030368879,
        standardError: 0.024732874338405486,
        low95ConfidenceInterval: 0.7858846221620299,
        high95ConfidenceInterval: 0.8828904417267226,
        atRisk: 189,
    },
    {
        survivalFunctionEstimate: 0.8285458764249896,
        standardError: 0.02499503329948401,
        low95ConfidenceInterval: 0.7809766790650879,
        high95ConfidenceInterval: 0.8790125079825091,
        atRisk: 188,
    },
    {
        survivalFunctionEstimate: 0.8241151498130912,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 187,
    },
    {
        survivalFunctionEstimate: 0.8196844232011928,
        standardError: 0.025501024206286953,
        low95ConfidenceInterval: 0.7711966521914113,
        high95ConfidenceInterval: 0.8712207862021563,
        atRisk: 186,
    },
    {
        survivalFunctionEstimate: 0.8152536965892946,
        standardError: 0.02574521634972202,
        low95ConfidenceInterval: 0.7663238624353613,
        high95ConfidenceInterval: 0.8673077042000258,
        atRisk: 185,
    },
    {
        survivalFunctionEstimate: 0.8108229699773962,
        standardError: 0.025983761566129887,
        low95ConfidenceInterval: 0.7614621201100749,
        high95ConfidenceInterval: 0.8633835765171467,
        atRisk: 184,
    },
    {
        survivalFunctionEstimate: 0.8063922433654979,
        standardError: 0.026216813999113575,
        low95ConfidenceInterval: 0.7566111230778935,
        high95ConfidenceInterval: 0.8594487053200444,
        atRisk: 183,
    },
    {
        survivalFunctionEstimate: 0.8019615167535995,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 182,
    },
    {
        survivalFunctionEstimate: 0.7975307901417011,
        standardError: 0.026667013167259244,
        low95ConfidenceInterval: 0.7469402423914959,
        high95ConfidenceInterval: 0.8515478550031967,
        atRisk: 181,
    },
    {
        survivalFunctionEstimate: 0.7931000635298028,
        standardError: 0.026884426256158216,
        low95ConfidenceInterval: 0.7421198366144994,
        high95ConfidenceInterval: 0.8475823980672283,
        atRisk: 180,
    },
    {
        survivalFunctionEstimate: 0.7886693369179044,
        standardError: 0.027096880445726126,
        low95ConfidenceInterval: 0.7373091295792802,
        high95ConfidenceInterval: 0.843607244290396,
        atRisk: 179,
    },
    {
        survivalFunctionEstimate: 0.7842386103060061,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 178,
    },
    {
        survivalFunctionEstimate: 0.7798078836941078,
        standardError: 0.02750736905168702,
        low95ConfidenceInterval: 0.7277159159467492,
        high95ConfidenceInterval: 0.8356287421312647,
        atRisk: 177,
    },
    {
        survivalFunctionEstimate: 0.7753771570822094,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 176,
    },
    {
        survivalFunctionEstimate: 0.7709464304703111,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 175,
    },
    {
        survivalFunctionEstimate: 0.7665157038584127,
        standardError: 0.028088614549899322,
        low95ConfidenceInterval: 0.71339353405228,
        high95ConfidenceInterval: 0.8235935654254198,
        atRisk: 174,
    },
    {
        survivalFunctionEstimate: 0.7620849772465144,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 173,
    },
    {
        survivalFunctionEstimate: 0.757654250634616,
        standardError: 0.028454215832805715,
        low95ConfidenceInterval: 0.703888089637702,
        high95ConfidenceInterval: 0.8155273145766189,
        atRisk: 172,
    },
    {
        survivalFunctionEstimate: 0.7532235240227176,
        standardError: 0.0286307024133906,
        low95ConfidenceInterval: 0.6991477110928089,
        high95ConfidenceInterval: 0.8114818487418157,
        atRisk: 171,
    },
    {
        survivalFunctionEstimate: 0.7487927974108193,
        standardError: 0.028803083404654848,
        low95ConfidenceInterval: 0.6944153581040133,
        high95ConfidenceInterval: 0.8074283595702632,
        atRisk: 170,
    },
    {
        survivalFunctionEstimate: 0.7487927974108193,
        atRisk: 169,
    },
    {
        survivalFunctionEstimate: 0.7487927974108193,
        atRisk: 168,
    },
    {
        survivalFunctionEstimate: 0.7443090082047664,
        standardError: 0.0289775050803304,
        low95ConfidenceInterval: 0.6896269426384071,
        high95ConfidenceInterval: 0.8033269372789578,
        atRisk: 167,
    },
    {
        survivalFunctionEstimate: 0.7443090082047664,
        atRisk: 166,
    },
    {
        survivalFunctionEstimate: 0.7397980445186769,
        standardError: 0.02915088300448111,
        low95ConfidenceInterval: 0.6848139057644397,
        high95ConfidenceInterval: 0.7991968943193706,
        atRisk: 165,
    },
    {
        survivalFunctionEstimate: 0.7352870808325874,
        standardError: 0.029320081732531107,
        low95ConfidenceInterval: 0.6800090367596312,
        high95ConfidenceInterval: 0.7950586860074556,
        atRisk: 164,
    },
    {
        survivalFunctionEstimate: 0.7352870808325874,
        atRisk: 163,
    },
    {
        survivalFunctionEstimate: 0.7307482716916455,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 162,
    },
    {
        survivalFunctionEstimate: 0.7262094625507035,
        standardError: 0.029652399306138524,
        low95ConfidenceInterval: 0.6703565535202378,
        high95ConfidenceInterval: 0.7867159360623157,
        atRisk: 161,
    },
    {
        survivalFunctionEstimate: 0.7216706534097617,
        standardError: 0.029812419468637323,
        low95ConfidenceInterval: 0.665542307130596,
        high95ConfidenceInterval: 0.7825325699252303,
        atRisk: 160,
    },
    {
        survivalFunctionEstimate: 0.7171318442688198,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 159,
    },
    {
        survivalFunctionEstimate: 0.7125930351278779,
        standardError: 0.030120508671315792,
        low95ConfidenceInterval: 0.6559371648215311,
        high95ConfidenceInterval: 0.7741424955710832,
        atRisk: 158,
    },
    {
        survivalFunctionEstimate: 0.708054225986936,
        standardError: 0.030268699355940832,
        low95ConfidenceInterval: 0.6511460301448384,
        high95ConfidenceInterval: 0.7699360262189463,
        atRisk: 157,
    },
    {
        survivalFunctionEstimate: 0.7035154168459942,
        standardError: 0.03041306399374996,
        low95ConfidenceInterval: 0.6463623694859835,
        high95ConfidenceInterval: 0.7657220857915765,
        atRisk: 156,
    },
    {
        survivalFunctionEstimate: 0.7035154168459942,
        atRisk: 155,
    },
    {
        survivalFunctionEstimate: 0.6989471349184229,
        standardError: 0.030556746434366584,
        low95ConfidenceInterval: 0.6415511474681542,
        high95ConfidenceInterval: 0.7614780198564322,
        atRisk: 154,
    },
    {
        survivalFunctionEstimate: 0.6989471349184229,
        atRisk: 153,
    },
    {
        survivalFunctionEstimate: 0.6943487985044859,
        standardError: 0.03069975624822632,
        low95ConfidenceInterval: 0.6367117813051313,
        high95ConfidenceInterval: 0.7572032874849175,
        atRisk: 152,
    },
    {
        survivalFunctionEstimate: 0.6943487985044859,
        atRisk: 151,
    },
    {
        survivalFunctionEstimate: 0.6943487985044859,
        atRisk: 150,
    },
    {
        survivalFunctionEstimate: 0.6896887394541202,
        standardError: 0.03084537596468369,
        low95ConfidenceInterval: 0.6318068386265672,
        high95ConfidenceInterval: 0.7528733914369057,
        atRisk: 149,
    },
    {
        survivalFunctionEstimate: 0.6896887394541202,
        atRisk: 148,
    },
    {
        survivalFunctionEstimate: 0.6849969793217793,
        standardError: 0.030990311251877862,
        low95ConfidenceInterval: 0.6268721795556091,
        high95ConfidenceInterval: 0.7485112228342844,
        atRisk: 147,
    },
    {
        survivalFunctionEstimate: 0.6849969793217793,
        atRisk: 146,
    },
    {
        survivalFunctionEstimate: 0.6802728622230084,
        standardError: 0.031134571657969453,
        low95ConfidenceInterval: 0.6219071502619764,
        high95ConfidenceInterval: 0.7441161705282586,
        atRisk: 145,
    },
    {
        survivalFunctionEstimate: 0.6755487451242376,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 144,
    },
    {
        survivalFunctionEstimate: 0.6708246280254667,
        standardError: 0.03141077002867532,
        low95ConfidenceInterval: 0.6120011509890957,
        high95ConfidenceInterval: 0.7353020183674845,
        atRisk: 143,
    },
    {
        survivalFunctionEstimate: 0.6661005109266959,
        standardError: 0.0315428158924731,
        low95ConfidenceInterval: 0.6070599689817391,
        high95ConfidenceInterval: 0.7308831307078855,
        atRisk: 142,
    },
    {
        survivalFunctionEstimate: 0.6661005109266959,
        atRisk: 141,
    },
    {
        survivalFunctionEstimate: 0.6661005109266959,
        atRisk: 140,
    },
    {
        survivalFunctionEstimate: 0.6613084209200291,
        standardError: 0.03167781291272837,
        low95ConfidenceInterval: 0.6020464948312763,
        high95ConfidenceInterval: 0.7264037434555679,
        atRisk: 139,
    },
    {
        survivalFunctionEstimate: 0.6565163309133621,
        standardError: 0.03180866233964095,
        low95ConfidenceInterval: 0.5970411161533592,
        high95ConfidenceInterval: 0.7219162652195477,
        atRisk: 138,
    },
    {
        survivalFunctionEstimate: 0.6517242409066952,
        standardError: 0.03193541515522963,
        low95ConfidenceInterval: 0.592043732696558,
        high95ConfidenceInterval: 0.7174207963503666,
        atRisk: 137,
    },
    {
        survivalFunctionEstimate: 0.6517242409066952,
        atRisk: 136,
    },
    {
        survivalFunctionEstimate: 0.646896653937016,
        standardError: 0.03206166662641572,
        low95ConfidenceInterval: 0.5870126010017608,
        high95ConfidenceInterval: 0.7128897747011944,
        atRisk: 135,
    },
    {
        survivalFunctionEstimate: 0.6420690669673368,
        standardError: 0.03218379785658331,
        low95ConfidenceInterval: 0.581989509110171,
        high95ConfidenceInterval: 0.7083507181884041,
        atRisk: 134,
    },
    {
        survivalFunctionEstimate: 0.6420690669673368,
        atRisk: 133,
    },
    {
        survivalFunctionEstimate: 0.6372049073690994,
        standardError: 0.03230546711091228,
        low95ConfidenceInterval: 0.5769315524532485,
        high95ConfidenceInterval: 0.7037751571199863,
        atRisk: 132,
    },
    {
        survivalFunctionEstimate: 0.6372049073690994,
        atRisk: 131,
    },
    {
        survivalFunctionEstimate: 0.6323033311585678,
        standardError: 0.03242667957372857,
        low95ConfidenceInterval: 0.5718379060086294,
        high95ConfidenceInterval: 0.6991622947573333,
        atRisk: 130,
    },
    {
        survivalFunctionEstimate: 0.6323033311585678,
        atRisk: 129,
    },
    {
        survivalFunctionEstimate: 0.6323033311585678,
        atRisk: 128,
    },
    {
        survivalFunctionEstimate: 0.6323033311585678,
        atRisk: 127,
    },
    {
        survivalFunctionEstimate: 0.6272850507525475,
        standardError: 0.03255531817285409,
        low95ConfidenceInterval: 0.5666157347189651,
        high95ConfidenceInterval: 0.6944504199001655,
        atRisk: 126,
    },
    {
        survivalFunctionEstimate: 0.6222667703465271,
        standardError: 0.0326793614521676,
        low95ConfidenceInterval: 0.5614025265382906,
        high95ConfidenceInterval: 0.6897295882602112,
        atRisk: 125,
    },
    {
        survivalFunctionEstimate: 0.6172484899405067,
        standardError: 0.03279886154936215,
        low95ConfidenceInterval: 0.5561981787765651,
        high95ConfidenceInterval: 0.6849999026819695,
        atRisk: 124,
    },
    {
        survivalFunctionEstimate: 0.6172484899405067,
        atRisk: 123,
    },
    {
        survivalFunctionEstimate: 0.6172484899405067,
        atRisk: 122,
    },
    {
        survivalFunctionEstimate: 0.6121472627509158,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 121,
    },
    {
        survivalFunctionEstimate: 0.6070460355613249,
        standardError: 0.033040604156857036,
        low95ConfidenceInterval: 0.54562216612668,
        high95ConfidenceInterval: 0.675384748216339,
        atRisk: 120,
    },
    {
        survivalFunctionEstimate: 0.6070460355613249,
        atRisk: 119,
    },
    {
        survivalFunctionEstimate: 0.6070460355613249,
        atRisk: 118,
    },
    {
        survivalFunctionEstimate: 0.6018576079069545,
        standardError: 0.03316307807140852,
        low95ConfidenceInterval: 0.5402459575191962,
        high95ConfidenceInterval: 0.6704956791511215,
        atRisk: 117,
    },
    {
        survivalFunctionEstimate: 0.5966691802525843,
        standardError: 0.033280584455718375,
        low95ConfidenceInterval: 0.5348794322420494,
        high95ConfidenceInterval: 0.665596934941001,
        atRisk: 116,
    },
    {
        survivalFunctionEstimate: 0.5966691802525843,
        atRisk: 115,
    },
    {
        survivalFunctionEstimate: 0.5966691802525843,
        atRisk: 114,
    },
    {
        survivalFunctionEstimate: 0.5966691802525843,
        atRisk: 113,
    },
    {
        survivalFunctionEstimate: 0.5913417768574719,
        standardError: 0.03340710850358679,
        low95ConfidenceInterval: 0.5293598558707564,
        high95ConfidenceInterval: 0.6605810644287464,
        atRisk: 112,
    },
    {
        survivalFunctionEstimate: 0.5860143734623595,
        standardError: 0.03352823111588733,
        low95ConfidenceInterval: 0.523850805819741,
        high95ConfidenceInterval: 0.6555546771892367,
        atRisk: 111,
    },
    {
        survivalFunctionEstimate: 0.5860143734623595,
        atRisk: 110,
    },
    {
        survivalFunctionEstimate: 0.5806380948067416,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 109,
    },
    {
        survivalFunctionEstimate: 0.5752618161511236,
        standardError: 0.03334523294750955,
        low95ConfidenceInterval: 0.5134821862490782,
        high95ConfidenceInterval: 0.6444744647109619,
        atRisk: 108,
    },
    {
        survivalFunctionEstimate: 0.5752618161511236,
        atRisk: 107,
    },
    {
        survivalFunctionEstimate: 0.5752618161511236,
        atRisk: 106,
    },
    {
        survivalFunctionEstimate: 0.5752618161511236,
        atRisk: 105,
    },
    {
        survivalFunctionEstimate: 0.5697304525342859,
        standardError: 0.03348023855138149,
        low95ConfidenceInterval: 0.507748363323936,
        high95ConfidenceInterval: 0.6392788475377846,
        atRisk: 104,
    },
    {
        survivalFunctionEstimate: 0.5641990889174482,
        standardError: 0.03360900780962913,
        low95ConfidenceInterval: 0.5020266883028248,
        high95ConfidenceInterval: 0.6340710949280571,
        atRisk: 103,
    },
    {
        survivalFunctionEstimate: 0.5641990889174482,
        atRisk: 102,
    },
    {
        survivalFunctionEstimate: 0.5586129593242061,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 101,
    },
    {
        survivalFunctionEstimate: 0.553026829730964,
        standardError: 0.03385922205269199,
        low95ConfidenceInterval: 0.4904911200029596,
        high95ConfidenceInterval: 0.623535599177484,
        atRisk: 100,
    },
    {
        survivalFunctionEstimate: 0.547440700137722,
        standardError: 0.03397488763849586,
        low95ConfidenceInterval: 0.4847417189527875,
        high95ConfidenceInterval: 0.618249489263515,
        atRisk: 99,
    },
    {
        survivalFunctionEstimate: 0.5418545705444798,
        standardError: 0.03408434546950667,
        low95ConfidenceInterval: 0.47900440184531806,
        high95ConfidenceInterval: 0.6129513100273246,
        atRisk: 98,
    },
    {
        survivalFunctionEstimate: 0.5362684409512378,
        standardError: 0.03418765517145462,
        low95ConfidenceInterval: 0.47327905045942403,
        high95ConfidenceInterval: 0.6076411801475394,
        atRisk: 97,
    },
    {
        survivalFunctionEstimate: 0.5362684409512378,
        atRisk: 96,
    },
    {
        survivalFunctionEstimate: 0.5362684409512378,
        atRisk: 95,
    },
    {
        survivalFunctionEstimate: 0.530563457536863,
        standardError: 0.03429665640397586,
        low95ConfidenceInterval: 0.4674272285217594,
        high95ConfidenceInterval: 0.6022276095547705,
        atRisk: 94,
    },
    {
        survivalFunctionEstimate: 0.530563457536863,
        atRisk: 93,
    },
    {
        survivalFunctionEstimate: 0.530563457536863,
        atRisk: 92,
    },
    {
        survivalFunctionEstimate: 0.5247330898716227,
        standardError: 0.03441177759427357,
        low95ConfidenceInterval: 0.46144188093134003,
        high95ConfidenceInterval: 0.5967052991602864,
        atRisk: 91,
    },
    {
        survivalFunctionEstimate: 0.5247330898716227,
        atRisk: 90,
    },
    {
        survivalFunctionEstimate: 0.5188372124573348,
        standardError: 0.03452651381846376,
        low95ConfidenceInterval: 0.4553937902443122,
        high95ConfidenceInterval: 0.5911192879597238,
        atRisk: 89,
    },
    {
        survivalFunctionEstimate: 0.5188372124573348,
        atRisk: 88,
    },
    {
        survivalFunctionEstimate: 0.5128735663371355,
        standardError: 0.03464086879793856,
        low95ConfidenceInterval: 0.44928080049300384,
        high95ConfidenceInterval: 0.5854674732566681,
        atRisk: 87,
    },
    {
        survivalFunctionEstimate: 0.5069099202169363,
        standardError: 0.03474761614695037,
        low95ConfidenceInterval: 0.4431826017473868,
        high95ConfidenceInterval: 0.5798008906514026,
        atRisk: 86,
    },
    {
        survivalFunctionEstimate: 0.500946274096737,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 85,
    },
    {
        survivalFunctionEstimate: 0.49498262797653775,
        standardError: 0.03493856190710485,
        low95ConfidenceInterval: 0.43103003103773135,
        high95ConfidenceInterval: 0.5684239713151498,
        atRisk: 84,
    },
    {
        survivalFunctionEstimate: 0.49498262797653775,
        atRisk: 83,
    },
    {
        survivalFunctionEstimate: 0.48894625446462875,
        standardError: 0.03503005634423714,
        low95ConfidenceInterval: 0.42489112140588586,
        high95ConfidenceInterval: 0.5626581204237838,
        atRisk: 82,
    },
    {
        survivalFunctionEstimate: 0.48290988095271975,
        standardError: 0.03511382844751059,
        low95ConfidenceInterval: 0.4187672113094657,
        high95ConfidenceInterval: 0.5568772979922622,
        atRisk: 81,
    },
    {
        survivalFunctionEstimate: 0.48290988095271975,
        atRisk: 80,
    },
    {
        survivalFunctionEstimate: 0.4767970976495208,
        standardError: 0.035197399520216575,
        low95ConfidenceInterval: 0.41256984384085826,
        high95ConfidenceInterval: 0.5510229982167515,
        atRisk: 79,
    },
    {
        survivalFunctionEstimate: 0.4706843143463218,
        standardError: 0.03527296504259942,
        low95ConfidenceInterval: 0.40638801774047384,
        high95ConfidenceInterval: 0.5451531888254358,
        atRisk: 78,
    },
    {
        survivalFunctionEstimate: 0.4645715310431228,
        standardError: 0.03534057636730476,
        low95ConfidenceInterval: 0.4002216290061309,
        high95ConfidenceInterval: 0.5392679750759923,
        atRisk: 77,
    },
    {
        survivalFunctionEstimate: 0.45845874773992384,
        standardError: 0.035400279069751975,
        low95ConfidenceInterval: 0.39407058479741336,
        high95ConfidenceInterval: 0.5333674511314065,
        atRisk: 76,
    },
    {
        survivalFunctionEstimate: 0.45234596443672487,
        standardError: 0.03545211310522265,
        low95ConfidenceInterval: 0.3879348031192501,
        high95ConfidenceInterval: 0.5274517003809326,
        atRisk: 75,
    },
    {
        survivalFunctionEstimate: 0.4462331811335259,
        standardError: 0.035496112944952724,
        low95ConfidenceInterval: 0.38181421254599174,
        high95ConfidenceInterval: 0.5215207957209306,
        atRisk: 74,
    },
    {
        survivalFunctionEstimate: 0.44012039783032686,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 73,
    },
    {
        survivalFunctionEstimate: 0.4340076145271279,
        standardError: 0.035560721180116724,
        low95ConfidenceInterval: 0.3696183704692744,
        high95ConfidenceInterval: 0.5096137652151308,
        atRisk: 72,
    },
    {
        survivalFunctionEstimate: 0.4340076145271279,
        atRisk: 71,
    },
    {
        survivalFunctionEstimate: 0.42780750574816895,
        standardError: 0.035589109663053696,
        low95ConfidenceInterval: 0.3634438588487101,
        high95ConfidenceInterval: 0.5035695541925627,
        atRisk: 70,
    },
    {
        survivalFunctionEstimate: 0.42160739696921,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 69,
    },
    {
        survivalFunctionEstimate: 0.4154072881902511,
        standardError: 0.03562131205011039,
        low95ConfidenceInterval: 0.3511424205785881,
        high95ConfidenceInterval: 0.49143368892098155,
        atRisk: 68,
    },
    {
        survivalFunctionEstimate: 0.4092071794112921,
        standardError: 0.03562514816776145,
        low95ConfidenceInterval: 0.34501543962757225,
        high95ConfidenceInterval: 0.4853420932770436,
        atRisk: 67,
    },
    {
        survivalFunctionEstimate: 0.4092071794112921,
        atRisk: 66,
    },
    {
        survivalFunctionEstimate: 0.40291168434342606,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 65,
    },
    {
        survivalFunctionEstimate: 0.39661618927556,
        standardError: 0.03562413059419805,
        low95ConfidenceInterval: 0.3325947399570127,
        high95ConfidenceInterval: 0.472961182776968,
        atRisk: 64,
    },
    {
        survivalFunctionEstimate: 0.39661618927556,
        atRisk: 63,
    },
    {
        survivalFunctionEstimate: 0.39661618927556,
        atRisk: 62,
    },
    {
        survivalFunctionEstimate: 0.39661618927556,
        atRisk: 61,
    },
    {
        survivalFunctionEstimate: 0.3900059194543007,
        standardError: 0.035638406049557494,
        low95ConfidenceInterval: 0.3260536997855951,
        high95ConfidenceInterval: 0.46650173670599265,
        atRisk: 60,
    },
    {
        survivalFunctionEstimate: 0.38339564963304135,
        standardError: 0.03564213535112956,
        low95ConfidenceInterval: 0.3195330352046811,
        high95ConfidenceInterval: 0.46002199448136555,
        atRisk: 59,
    },
    {
        survivalFunctionEstimate: 0.376785379811782,
        standardError: 0.03563532180993724,
        low95ConfidenceInterval: 0.3130327301333202,
        high95ConfidenceInterval: 0.4535219763743081,
        atRisk: 58,
    },
    {
        survivalFunctionEstimate: 0.376785379811782,
        atRisk: 57,
    },
    {
        survivalFunctionEstimate: 0.376785379811782,
        atRisk: 56,
    },
    {
        survivalFunctionEstimate: 0.3699347365424769,
        standardError: 0.03563981840501213,
        low95ConfidenceInterval: 0.3062807152300911,
        high95ConfidenceInterval: 0.44681791081081607,
        atRisk: 55,
    },
    {
        survivalFunctionEstimate: 0.36308409327317176,
        standardError: 0.035632150450449006,
        low95ConfidenceInterval: 0.2995521672284722,
        high95ConfidenceInterval: 0.4400904857665504,
        atRisk: 54,
    },
    {
        survivalFunctionEstimate: 0.35623345000386664,
        standardError: 0.035612310088526465,
        low95ConfidenceInterval: 0.29284708804998383,
        high95ConfidenceInterval: 0.43333970553259304,
        atRisk: 53,
    },
    {
        survivalFunctionEstimate: 0.3493828067345615,
        standardError: 0.03558027695642677,
        low95ConfidenceInterval: 0.28616550323445694,
        high95ConfidenceInterval: 0.4265655512701987,
        atRisk: 52,
    },
    {
        survivalFunctionEstimate: 0.3425321634652564,
        standardError: 0.03553601808148237,
        low95ConfidenceInterval: 0.2795074620717978,
        high95ConfidenceInterval: 0.4197679809280037,
        atRisk: 51,
    },
    {
        survivalFunctionEstimate: 0.33568152019595127,
        standardError: 0.035479487710726454,
        low95ConfidenceInterval: 0.27287303785454065,
        high95ConfidenceInterval: 0.4129469290444586,
        atRisk: 50,
    },
    {
        survivalFunctionEstimate: 0.33568152019595127,
        atRisk: 49,
    },
    {
        survivalFunctionEstimate: 0.32868815519186895,
        standardError: 0.035422858479053,
        low95ConfidenceInterval: 0.266102963619344,
        high95ConfidenceInterval: 0.40599286041014465,
        atRisk: 48,
    },
    {
        survivalFunctionEstimate: 0.32169479018778663,
        standardError: 0.03535277446705729,
        low95ConfidenceInterval: 0.2593587485381947,
        high95ConfidenceInterval: 0.3990130991040153,
        atRisk: 47,
    },
    {
        survivalFunctionEstimate: 0.3147014251837043,
        standardError: 0.03526915546607818,
        low95ConfidenceInterval: 0.25264052771810225,
        high95ConfidenceInterval: 0.39200752114942017,
        atRisk: 46,
    },
    {
        survivalFunctionEstimate: 0.3147014251837043,
        atRisk: 45,
    },
    {
        survivalFunctionEstimate: 0.30754912006589286,
        standardError: 0.035185326115332316,
        low95ConfidenceInterval: 0.24577194611898923,
        high95ConfidenceInterval: 0.3848545887638105,
        atRisk: 44,
    },
    {
        survivalFunctionEstimate: 0.3003968149480814,
        standardError: 0.03508647611623453,
        low95ConfidenceInterval: 0.23893216608647452,
        high95ConfidenceInterval: 0.37767307729630994,
        atRisk: 43,
    },
    {
        survivalFunctionEstimate: 0.29324450983026995,
        standardError: 0.034972478101055714,
        low95ConfidenceInterval: 0.23212140665713243,
        high95ConfidenceInterval: 0.37046278404048694,
        atRisk: 42,
    },
    {
        survivalFunctionEstimate: 0.29324450983026995,
        atRisk: 41,
    },
    {
        survivalFunctionEstimate: 0.29324450983026995,
        atRisk: 40,
    },
    {
        survivalFunctionEstimate: 0.285725419834622,
        standardError: 0.03487468493779172,
        low95ConfidenceInterval: 0.2249334676392631,
        high95ConfidenceInterval: 0.36294739238448726,
        atRisk: 39,
    },
    {
        survivalFunctionEstimate: 0.2782063298389741,
        standardError: 0.03475804951284322,
        low95ConfidenceInterval: 0.21778155615308648,
        high95ConfidenceInterval: 0.35539631238591046,
        atRisk: 38,
    },
    {
        survivalFunctionEstimate: 0.27068723984332616,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 37,
    },
    {
        survivalFunctionEstimate: 0.2631681498476782,
        standardError: 0.03446745585517061,
        low95ConfidenceInterval: 0.2035871747453536,
        high95ConfidenceInterval: 0.34018584510972816,
        atRisk: 36,
    },
    {
        survivalFunctionEstimate: 0.2631681498476782,
        atRisk: 35,
    },
    {
        survivalFunctionEstimate: 0.2554279101462759,
        standardError: 0.03431180169531887,
        low95ConfidenceInterval: 0.19630252167644624,
        high95ConfidenceInterval: 0.3323615851926286,
        atRisk: 34,
    },
    {
        survivalFunctionEstimate: 0.2554279101462759,
        atRisk: 33,
    },
    {
        survivalFunctionEstimate: 0.2474457879542048,
        standardError: 0.03415540092698885,
        low95ConfidenceInterval: 0.18879354367362663,
        high95ConfidenceInterval: 0.3243194485618984,
        atRisk: 32,
    },
    {
        survivalFunctionEstimate: 0.2474457879542048,
        atRisk: 31,
    },
    {
        survivalFunctionEstimate: 0.23919759502239799,
        standardError: 0.0339982322297885,
        low95ConfidenceInterval: 0.18103881453283843,
        high95ConfidenceInterval: 0.3160399034435837,
        atRisk: 30,
    },
    {
        survivalFunctionEstimate: 0.23919759502239799,
        atRisk: 29,
    },
    {
        survivalFunctionEstimate: 0.23065482377159807,
        standardError: 0.03384026948249029,
        low95ConfidenceInterval: 0.17301328048135686,
        high95ConfidenceInterval: 0.30750036980450035,
        atRisk: 28,
    },
    {
        survivalFunctionEstimate: 0.22211205252079813,
        standardError: 0.0336479386295172,
        low95ConfidenceInterval: 0.1650528011650719,
        high95ConfidenceInterval: 0.298896859227868,
        atRisk: 27,
    },
    {
        survivalFunctionEstimate: 0.21356928126999822,
        standardError: 0.033420646326114106,
        low95ConfidenceInterval: 0.15715835956266835,
        high95ConfidenceInterval: 0.2902285187317412,
        atRisk: 26,
    },
    {
        survivalFunctionEstimate: 0.21356928126999822,
        atRisk: 25,
    },
    {
        survivalFunctionEstimate: 0.20467056121708163,
        standardError: 0.03319168846525831,
        low95ConfidenceInterval: 0.14894122794986697,
        high95ConfidenceInterval: 0.2812521368698208,
        atRisk: 24,
    },
    {
        survivalFunctionEstimate: 0.19577184116416504,
        standardError: 0.03291984345521111,
        low95ConfidenceInterval: 0.14080470629668462,
        high95ConfidenceInterval: 0.2721969655762103,
        atRisk: 23,
    },
    {
        survivalFunctionEstimate: 0.18687312111124846,
        standardError: 0.03260403856392239,
        low95ConfidenceInterval: 0.13275057386617126,
        high95ConfidenceInterval: 0.26306148724497813,
        atRisk: 22,
    },
    {
        survivalFunctionEstimate: 0.17797440105833184,
        standardError: 0.0322429821179921,
        low95ConfidenceInterval: 0.1247809874285874,
        high95ConfidenceInterval: 0.2538438594277001,
        atRisk: 21,
    },
    {
        survivalFunctionEstimate: 0.16907568100541526,
        standardError: 0.031835134500514777,
        low95ConfidenceInterval: 0.11689852691761661,
        high95ConfidenceInterval: 0.24454188312904168,
        atRisk: 20,
    },
    {
        survivalFunctionEstimate: 0.16017696095249864,
        standardError: 0.0313786712448418,
        low95ConfidenceInterval: 0.10910625339694567,
        high95ConfidenceInterval: 0.23515296347529532,
        atRisk: 19,
    },
    {
        survivalFunctionEstimate: 0.15127824089958206,
        standardError: 0.030871435942293735,
        low95ConfidenceInterval: 0.10140778267344999,
        high95ConfidenceInterval: 0.2256740613623892,
        atRisk: 18,
    },
    {
        survivalFunctionEstimate: 0.14237952084666547,
        standardError: 0.0303108797762077,
        low95ConfidenceInterval: 0.09380737916875256,
        high95ConfidenceInterval: 0.21610163439337052,
        atRisk: 17,
    },
    {
        survivalFunctionEstimate: 0.13348080079374888,
        standardError: 0.02969398316321972,
        low95ConfidenceInterval: 0.08631007650527339,
        high95ConfidenceInterval: 0.20643156514236066,
        atRisk: 16,
    },
    {
        survivalFunctionEstimate: 0.1245820807408323,
        standardError: 0.02901715297903506,
        low95ConfidenceInterval: 0.07892183398935097,
        high95ConfidenceInterval: 0.196659074646054,
        atRisk: 15,
    },
    {
        survivalFunctionEstimate: 0.11568336068791571,
        standardError: 0.02827608575353311,
        low95ConfidenceInterval: 0.0716497423010419,
        high95ConfidenceInterval: 0.18677861929805994,
        atRisk: 14,
    },
    {
        survivalFunctionEstimate: 0.10678464063499912,
        standardError: 0.027465582311228733,
        low95ConfidenceInterval: 0.06450229811167155,
        high95ConfidenceInterval: 0.17678377064649992,
        atRisk: 13,
    },
    {
        survivalFunctionEstimate: 0.09788592058208252,
        standardError: 0.026579291289834855,
        low95ConfidenceInterval: 0.057489777604174515,
        high95ConfidenceInterval: 0.1666670814796474,
        atRisk: 12,
    },
    {
        survivalFunctionEstimate: 0.09788592058208252,
        atRisk: 11,
    },
    {
        survivalFunctionEstimate: 0.08809732852387427,
        standardError: 0.02566060109709879,
        low95ConfidenceInterval: 0.049776976047339196,
        high95ConfidenceInterval: 0.1559182559756621,
        atRisk: 10,
    },
    {
        survivalFunctionEstimate: 0.07830873646566601,
        standardError: 0.024605691200857545,
        low95ConfidenceInterval: 0.04230102440735633,
        high95ConfidenceInterval: 0.14496713242203899,
        atRisk: 9,
    },
    {
        survivalFunctionEstimate: 0.07830873646566601,
        atRisk: 8,
    },
    {
        survivalFunctionEstimate: 0.067121774113428,
        standardError: 0.023496444690306453,
        low95ConfidenceInterval: 0.033798264168238876,
        high95ConfidenceInterval: 0.1333007085129488,
        atRisk: 7,
    },
    {
        survivalFunctionEstimate: 0.067121774113428,
        atRisk: 6,
    },
    {
        survivalFunctionEstimate: 0.067121774113428,
        atRisk: 5,
    },
    {
        survivalFunctionEstimate: 0.050341330585070995,
        standardError: 0.02284149956474811,
        low95ConfidenceInterval: 0.020687444972787645,
        high95ConfidenceInterval: 0.12250181539619648,
        atRisk: 4,
    },
    {
        survivalFunctionEstimate: 0.050341330585070995,
        atRisk: 3,
    },
    {
        survivalFunctionEstimate: 0.050341330585070995,
        atRisk: 2,
    },
    {
        survivalFunctionEstimate: 0.050341330585070995,
        atRisk: 1,
    },
];

// data from dataset and validation code can be found at: https://github.com/slb2240/delayed_entry_clin_genom_studies
const examplePatientSurvivalsFromDelayedEntryRepo = [
    {
        patientId: '1',
        uniquePatientKey: '1',
        studyId: '1',
        status: true,
        entryMonths: 0.888157895,
        months: 1.217105263,
    },
    {
        patientId: '2',
        uniquePatientKey: '2',
        studyId: '1',
        status: true,
        entryMonths: 1.611842105,
        months: 1.907894737,
    },
    {
        patientId: '3',
        uniquePatientKey: '3',
        studyId: '1',
        status: false,
        entryMonths: 1.677631579,
        months: 2.138157895,
    },
    {
        patientId: '4',
        uniquePatientKey: '4',
        studyId: '1',
        status: true,
        entryMonths: 0.888157895,
        months: 2.730263158,
    },
    {
        patientId: '5',
        uniquePatientKey: '5',
        studyId: '1',
        status: true,
        entryMonths: 1.25,
        months: 2.730263158,
    },
    {
        patientId: '6',
        uniquePatientKey: '6',
        studyId: '1',
        status: false,
        entryMonths: 0.888157895,
        months: 2.993421053,
    },
    {
        patientId: '7',
        uniquePatientKey: '7',
        studyId: '1',
        status: true,
        entryMonths: 1.743421053,
        months: 3.092105263,
    },
    {
        patientId: '8',
        uniquePatientKey: '8',
        studyId: '1',
        status: false,
        entryMonths: 2.434210526,
        months: 3.190789474,
    },
    {
        patientId: '9',
        uniquePatientKey: '9',
        studyId: '1',
        status: true,
        entryMonths: 1.184210526,
        months: 3.223684211,
    },
    {
        patientId: '10',
        uniquePatientKey: '10',
        studyId: '1',
        status: true,
        entryMonths: 2.730263158,
        months: 3.388157895,
    },
    {
        patientId: '11',
        uniquePatientKey: '11',
        studyId: '1',
        status: true,
        entryMonths: 1.25,
        months: 3.519736842,
    },
    {
        patientId: '12',
        uniquePatientKey: '12',
        studyId: '1',
        status: true,
        entryMonths: 0.953947368,
        months: 3.881578947,
    },
    {
        patientId: '13',
        uniquePatientKey: '13',
        studyId: '1',
        status: true,
        entryMonths: 1.677631579,
        months: 3.881578947,
    },
    {
        patientId: '14',
        uniquePatientKey: '14',
        studyId: '1',
        status: true,
        entryMonths: 2.368421053,
        months: 3.914473684,
    },
    {
        patientId: '15',
        uniquePatientKey: '15',
        studyId: '1',
        status: true,
        entryMonths: 1.151315789,
        months: 4.078947368,
    },
    {
        patientId: '16',
        uniquePatientKey: '16',
        studyId: '1',
        status: true,
        entryMonths: 3.322368421,
        months: 4.078947368,
    },
    {
        patientId: '17',
        uniquePatientKey: '17',
        studyId: '1',
        status: true,
        entryMonths: 4.046052632,
        months: 4.111842105,
    },
    {
        patientId: '18',
        uniquePatientKey: '18',
        studyId: '1',
        status: true,
        entryMonths: 2.138157895,
        months: 4.210526316,
    },
    {
        patientId: '19',
        uniquePatientKey: '19',
        studyId: '1',
        status: true,
        entryMonths: 1.644736842,
        months: 4.309210526,
    },
    {
        patientId: '20',
        uniquePatientKey: '20',
        studyId: '1',
        status: true,
        entryMonths: 3.815789474,
        months: 4.309210526,
    },
    {
        patientId: '21',
        uniquePatientKey: '21',
        studyId: '1',
        status: true,
        entryMonths: 2.368421053,
        months: 4.506578947,
    },
    {
        patientId: '22',
        uniquePatientKey: '22',
        studyId: '1',
        status: true,
        entryMonths: 1.315789474,
        months: 5.197368421,
    },
    {
        patientId: '23',
        uniquePatientKey: '23',
        studyId: '1',
        status: true,
        entryMonths: 2.302631579,
        months: 5.230263158,
    },
    {
        patientId: '24',
        uniquePatientKey: '24',
        studyId: '1',
        status: true,
        entryMonths: 1.611842105,
        months: 5.296052632,
    },
    {
        patientId: '25',
        uniquePatientKey: '25',
        studyId: '1',
        status: true,
        entryMonths: 2.236842105,
        months: 5.657894737,
    },
    {
        patientId: '26',
        uniquePatientKey: '26',
        studyId: '1',
        status: true,
        entryMonths: 1.776315789,
        months: 5.822368421,
    },
    {
        patientId: '27',
        uniquePatientKey: '27',
        studyId: '1',
        status: true,
        entryMonths: 4.013157895,
        months: 5.986842105,
    },
    {
        patientId: '28',
        uniquePatientKey: '28',
        studyId: '1',
        status: true,
        entryMonths: 1.973684211,
        months: 6.085526316,
    },
    {
        patientId: '29',
        uniquePatientKey: '29',
        studyId: '1',
        status: true,
        entryMonths: 3.717105263,
        months: 6.118421053,
    },
    {
        patientId: '30',
        uniquePatientKey: '30',
        studyId: '1',
        status: true,
        entryMonths: 3.256578947,
        months: 6.381578947,
    },
    {
        patientId: '31',
        uniquePatientKey: '31',
        studyId: '1',
        status: true,
        entryMonths: 3.223684211,
        months: 6.414473684,
    },
    {
        patientId: '32',
        uniquePatientKey: '32',
        studyId: '1',
        status: true,
        entryMonths: 2.171052632,
        months: 6.776315789,
    },
    {
        patientId: '33',
        uniquePatientKey: '33',
        studyId: '1',
        status: false,
        entryMonths: 2.072368421,
        months: 7.368421053,
    },
    {
        patientId: '34',
        uniquePatientKey: '34',
        studyId: '1',
        status: true,
        entryMonths: 1.348684211,
        months: 7.401315789,
    },
    {
        patientId: '35',
        uniquePatientKey: '35',
        studyId: '1',
        status: true,
        entryMonths: 1.611842105,
        months: 7.697368421,
    },
    {
        patientId: '36',
        uniquePatientKey: '36',
        studyId: '1',
        status: true,
        entryMonths: 5.098684211,
        months: 7.730263158,
    },
    {
        patientId: '37',
        uniquePatientKey: '37',
        studyId: '1',
        status: true,
        entryMonths: 1.480263158,
        months: 7.927631579,
    },
    {
        patientId: '38',
        uniquePatientKey: '38',
        studyId: '1',
        status: true,
        entryMonths: 8.552631579,
        months: 8.585526316,
    },
    {
        patientId: '39',
        uniquePatientKey: '39',
        studyId: '1',
        status: true,
        entryMonths: 6.315789474,
        months: 8.618421053,
    },
    {
        patientId: '40',
        uniquePatientKey: '40',
        studyId: '1',
        status: true,
        entryMonths: 2.828947368,
        months: 8.684210526,
    },
    {
        patientId: '41',
        uniquePatientKey: '41',
        studyId: '1',
        status: true,
        entryMonths: 0.756578947,
        months: 8.815789474,
    },
    {
        patientId: '42',
        uniquePatientKey: '42',
        studyId: '1',
        status: true,
        entryMonths: 3.256578947,
        months: 8.947368421,
    },
    {
        patientId: '43',
        uniquePatientKey: '43',
        studyId: '1',
        status: true,
        entryMonths: 0.526315789,
        months: 9.111842105,
    },
    {
        patientId: '44',
        uniquePatientKey: '44',
        studyId: '1',
        status: true,
        entryMonths: 2.730263158,
        months: 9.111842105,
    },
    {
        patientId: '45',
        uniquePatientKey: '45',
        studyId: '1',
        status: true,
        entryMonths: 2.796052632,
        months: 9.177631579,
    },
    {
        patientId: '46',
        uniquePatientKey: '46',
        studyId: '1',
        status: true,
        entryMonths: 2.269736842,
        months: 9.309210526,
    },
    {
        patientId: '47',
        uniquePatientKey: '47',
        studyId: '1',
        status: true,
        entryMonths: 7.763157895,
        months: 9.407894737,
    },
    {
        patientId: '48',
        uniquePatientKey: '48',
        studyId: '1',
        status: false,
        entryMonths: 3.421052632,
        months: 9.473684211,
    },
    {
        patientId: '49',
        uniquePatientKey: '49',
        studyId: '1',
        status: true,
        entryMonths: 4.572368421,
        months: 9.506578947,
    },
    {
        patientId: '50',
        uniquePatientKey: '50',
        studyId: '1',
        status: true,
        entryMonths: 2.598684211,
        months: 9.638157895,
    },
    {
        patientId: '51',
        uniquePatientKey: '51',
        studyId: '1',
        status: true,
        entryMonths: 4.605263158,
        months: 9.671052632,
    },
    {
        patientId: '52',
        uniquePatientKey: '52',
        studyId: '1',
        status: true,
        entryMonths: 6.315789474,
        months: 9.802631579,
    },
    {
        patientId: '53',
        uniquePatientKey: '53',
        studyId: '1',
        status: true,
        entryMonths: 7.861842105,
        months: 9.802631579,
    },
    {
        patientId: '54',
        uniquePatientKey: '54',
        studyId: '1',
        status: true,
        entryMonths: 1.809210526,
        months: 9.835526316,
    },
    {
        patientId: '55',
        uniquePatientKey: '55',
        studyId: '1',
        status: true,
        entryMonths: 3.486842105,
        months: 10.03289474,
    },
    {
        patientId: '56',
        uniquePatientKey: '56',
        studyId: '1',
        status: true,
        entryMonths: 1.611842105,
        months: 10.09868421,
    },
    {
        patientId: '57',
        uniquePatientKey: '57',
        studyId: '1',
        status: false,
        entryMonths: 4.605263158,
        months: 10.36184211,
    },
    {
        patientId: '58',
        uniquePatientKey: '58',
        studyId: '1',
        status: false,
        entryMonths: 6.677631579,
        months: 10.36184211,
    },
    {
        patientId: '60',
        uniquePatientKey: '60',
        studyId: '1',
        status: true,
        entryMonths: 5.065789474,
        months: 10.42763158,
    },
    {
        patientId: '59',
        uniquePatientKey: '59',
        studyId: '1',
        status: false,
        entryMonths: 2.171052632,
        months: 10.42763158,
    },
    {
        patientId: '61',
        uniquePatientKey: '61',
        studyId: '1',
        status: true,
        entryMonths: 2.171052632,
        months: 10.59210526,
    },
    {
        patientId: '62',
        uniquePatientKey: '62',
        studyId: '1',
        status: true,
        entryMonths: 2.368421053,
        months: 10.625,
    },
    {
        patientId: '63',
        uniquePatientKey: '63',
        studyId: '1',
        status: true,
        entryMonths: 6.217105263,
        months: 10.625,
    },
    {
        patientId: '64',
        uniquePatientKey: '64',
        studyId: '1',
        status: false,
        entryMonths: 10.06578947,
        months: 10.625,
    },
    {
        patientId: '65',
        uniquePatientKey: '65',
        studyId: '1',
        status: false,
        entryMonths: 9.769736842,
        months: 10.65789474,
    },
    {
        patientId: '66',
        uniquePatientKey: '66',
        studyId: '1',
        status: true,
        entryMonths: 1.677631579,
        months: 10.85526316,
    },
    {
        patientId: '67',
        uniquePatientKey: '67',
        studyId: '1',
        status: true,
        entryMonths: 0.723684211,
        months: 11.11842105,
    },
    {
        patientId: '68',
        uniquePatientKey: '68',
        studyId: '1',
        status: false,
        entryMonths: 4.342105263,
        months: 11.18421053,
    },
    {
        patientId: '69',
        uniquePatientKey: '69',
        studyId: '1',
        status: true,
        entryMonths: 3.322368421,
        months: 11.31578947,
    },
    {
        patientId: '70',
        uniquePatientKey: '70',
        studyId: '1',
        status: true,
        entryMonths: 5.690789474,
        months: 11.44736842,
    },
    {
        patientId: '71',
        uniquePatientKey: '71',
        studyId: '1',
        status: true,
        entryMonths: 4.078947368,
        months: 11.61184211,
    },
    {
        patientId: '72',
        uniquePatientKey: '72',
        studyId: '1',
        status: false,
        entryMonths: 4.671052632,
        months: 11.71052632,
    },
    {
        patientId: '73',
        uniquePatientKey: '73',
        studyId: '1',
        status: false,
        entryMonths: 8.947368421,
        months: 11.71052632,
    },
    {
        patientId: '74',
        uniquePatientKey: '74',
        studyId: '1',
        status: true,
        entryMonths: 3.223684211,
        months: 11.74342105,
    },
    {
        patientId: '75',
        uniquePatientKey: '75',
        studyId: '1',
        status: false,
        entryMonths: 2.269736842,
        months: 11.77631579,
    },
    {
        patientId: '76',
        uniquePatientKey: '76',
        studyId: '1',
        status: true,
        entryMonths: 6.217105263,
        months: 11.84210526,
    },
    {
        patientId: '77',
        uniquePatientKey: '77',
        studyId: '1',
        status: true,
        entryMonths: 6.414473684,
        months: 11.875,
    },
    {
        patientId: '78',
        uniquePatientKey: '78',
        studyId: '1',
        status: true,
        entryMonths: 1.677631579,
        months: 11.97368421,
    },
    {
        patientId: '79',
        uniquePatientKey: '79',
        studyId: '1',
        status: true,
        entryMonths: 3.453947368,
        months: 11.97368421,
    },
    {
        patientId: '80',
        uniquePatientKey: '80',
        studyId: '1',
        status: true,
        entryMonths: 10.13157895,
        months: 12.03947368,
    },
    {
        patientId: '81',
        uniquePatientKey: '81',
        studyId: '1',
        status: true,
        entryMonths: 2.105263158,
        months: 12.17105263,
    },
    {
        patientId: '82',
        uniquePatientKey: '82',
        studyId: '1',
        status: true,
        entryMonths: 7.434210526,
        months: 12.26973684,
    },
    {
        patientId: '83',
        uniquePatientKey: '83',
        studyId: '1',
        status: true,
        entryMonths: 5.460526316,
        months: 12.33552632,
    },
    {
        patientId: '84',
        uniquePatientKey: '84',
        studyId: '1',
        status: false,
        entryMonths: 6.546052632,
        months: 12.33552632,
    },
    {
        patientId: '85',
        uniquePatientKey: '85',
        studyId: '1',
        status: true,
        entryMonths: 2.105263158,
        months: 12.36842105,
    },
    {
        patientId: '86',
        uniquePatientKey: '86',
        studyId: '1',
        status: false,
        entryMonths: 6.480263158,
        months: 12.5,
    },
    {
        patientId: '87',
        uniquePatientKey: '87',
        studyId: '1',
        status: false,
        entryMonths: 2.072368421,
        months: 12.56578947,
    },
    {
        patientId: '88',
        uniquePatientKey: '88',
        studyId: '1',
        status: true,
        entryMonths: 0.592105263,
        months: 12.59868421,
    },
    {
        patientId: '89',
        uniquePatientKey: '89',
        studyId: '1',
        status: true,
        entryMonths: 1.118421053,
        months: 12.59868421,
    },
    {
        patientId: '90',
        uniquePatientKey: '90',
        studyId: '1',
        status: true,
        entryMonths: 2.5,
        months: 12.63157895,
    },
    {
        patientId: '91',
        uniquePatientKey: '91',
        studyId: '1',
        status: false,
        entryMonths: 4.572368421,
        months: 12.63157895,
    },
    {
        patientId: '92',
        uniquePatientKey: '92',
        studyId: '1',
        status: true,
        entryMonths: 3.782894737,
        months: 12.86184211,
    },
    {
        patientId: '93',
        uniquePatientKey: '93',
        studyId: '1',
        status: true,
        entryMonths: 7.072368421,
        months: 12.89473684,
    },
    {
        patientId: '94',
        uniquePatientKey: '94',
        studyId: '1',
        status: true,
        entryMonths: 10.82236842,
        months: 12.89473684,
    },
    {
        patientId: '95',
        uniquePatientKey: '95',
        studyId: '1',
        status: true,
        entryMonths: 1.052631579,
        months: 12.92763158,
    },
    {
        patientId: '96',
        uniquePatientKey: '96',
        studyId: '1',
        status: true,
        entryMonths: 1.809210526,
        months: 13.15789474,
    },
    {
        patientId: '97',
        uniquePatientKey: '97',
        studyId: '1',
        status: false,
        entryMonths: 4.835526316,
        months: 13.25657895,
    },
    {
        patientId: '98',
        uniquePatientKey: '98',
        studyId: '1',
        status: true,
        entryMonths: 2.072368421,
        months: 13.28947368,
    },
    {
        patientId: '99',
        uniquePatientKey: '99',
        studyId: '1',
        status: true,
        entryMonths: 10.42763158,
        months: 13.28947368,
    },
    {
        patientId: '100',
        uniquePatientKey: '100',
        studyId: '1',
        status: true,
        entryMonths: 7.401315789,
        months: 13.45394737,
    },
    {
        patientId: '101',
        uniquePatientKey: '101',
        studyId: '1',
        status: true,
        entryMonths: 3.125,
        months: 13.48684211,
    },
    {
        patientId: '102',
        uniquePatientKey: '102',
        studyId: '1',
        status: true,
        entryMonths: 11.57894737,
        months: 13.65131579,
    },
    {
        patientId: '103',
        uniquePatientKey: '103',
        studyId: '1',
        status: false,
        entryMonths: 1.447368421,
        months: 13.81578947,
    },
    {
        patientId: '104',
        uniquePatientKey: '104',
        studyId: '1',
        status: true,
        entryMonths: 1.776315789,
        months: 14.14473684,
    },
    {
        patientId: '105',
        uniquePatientKey: '105',
        studyId: '1',
        status: true,
        entryMonths: 3.552631579,
        months: 14.17763158,
    },
    {
        patientId: '106',
        uniquePatientKey: '106',
        studyId: '1',
        status: true,
        entryMonths: 8.75,
        months: 14.17763158,
    },
    {
        patientId: '107',
        uniquePatientKey: '107',
        studyId: '1',
        status: true,
        entryMonths: 10.06578947,
        months: 14.24342105,
    },
    {
        patientId: '108',
        uniquePatientKey: '108',
        studyId: '1',
        status: true,
        entryMonths: 3.486842105,
        months: 14.30921053,
    },
    {
        patientId: '109',
        uniquePatientKey: '109',
        studyId: '1',
        status: true,
        entryMonths: 2.006578947,
        months: 14.40789474,
    },
    {
        patientId: '110',
        uniquePatientKey: '110',
        studyId: '1',
        status: true,
        entryMonths: 3.651315789,
        months: 14.44078947,
    },
    {
        patientId: '111',
        uniquePatientKey: '111',
        studyId: '1',
        status: true,
        entryMonths: 10.69078947,
        months: 14.44078947,
    },
    {
        patientId: '112',
        uniquePatientKey: '112',
        studyId: '1',
        status: true,
        entryMonths: 2.730263158,
        months: 15.16447368,
    },
    {
        patientId: '113',
        uniquePatientKey: '113',
        studyId: '1',
        status: true,
        entryMonths: 4.868421053,
        months: 15.19736842,
    },
    {
        patientId: '114',
        uniquePatientKey: '114',
        studyId: '1',
        status: true,
        entryMonths: 1.480263158,
        months: 15.23026316,
    },
    {
        patientId: '115',
        uniquePatientKey: '115',
        studyId: '1',
        status: true,
        entryMonths: 3.289473684,
        months: 15.23026316,
    },
    {
        patientId: '116',
        uniquePatientKey: '116',
        studyId: '1',
        status: true,
        entryMonths: 2.302631579,
        months: 15.26315789,
    },
    {
        patientId: '117',
        uniquePatientKey: '117',
        studyId: '1',
        status: true,
        entryMonths: 14.07894737,
        months: 15.29605263,
    },
    {
        patientId: '118',
        uniquePatientKey: '118',
        studyId: '1',
        status: true,
        entryMonths: 2.927631579,
        months: 15.55921053,
    },
    {
        patientId: '119',
        uniquePatientKey: '119',
        studyId: '1',
        status: false,
        entryMonths: 1.875,
        months: 15.59210526,
    },
    {
        patientId: '120',
        uniquePatientKey: '120',
        studyId: '1',
        status: true,
        entryMonths: 2.434210526,
        months: 15.69078947,
    },
    {
        patientId: '121',
        uniquePatientKey: '121',
        studyId: '1',
        status: true,
        entryMonths: 1.973684211,
        months: 15.75657895,
    },
    {
        patientId: '122',
        uniquePatientKey: '122',
        studyId: '1',
        status: true,
        entryMonths: 4.276315789,
        months: 15.78947368,
    },
    {
        patientId: '123',
        uniquePatientKey: '123',
        studyId: '1',
        status: true,
        entryMonths: 2.269736842,
        months: 15.82236842,
    },
    {
        patientId: '124',
        uniquePatientKey: '124',
        studyId: '1',
        status: true,
        entryMonths: 3.059210526,
        months: 15.82236842,
    },
    {
        patientId: '125',
        uniquePatientKey: '125',
        studyId: '1',
        status: true,
        entryMonths: 2.796052632,
        months: 15.85526316,
    },
    {
        patientId: '126',
        uniquePatientKey: '126',
        studyId: '1',
        status: true,
        entryMonths: 5.493421053,
        months: 15.85526316,
    },
    {
        patientId: '127',
        uniquePatientKey: '127',
        studyId: '1',
        status: true,
        entryMonths: 2.401315789,
        months: 15.88815789,
    },
    {
        patientId: '128',
        uniquePatientKey: '128',
        studyId: '1',
        status: true,
        entryMonths: 5.723684211,
        months: 15.98684211,
    },
    {
        patientId: '129',
        uniquePatientKey: '129',
        studyId: '1',
        status: true,
        entryMonths: 9.901315789,
        months: 16.15131579,
    },
    {
        patientId: '130',
        uniquePatientKey: '130',
        studyId: '1',
        status: false,
        entryMonths: 12.30263158,
        months: 16.15131579,
    },
    {
        patientId: '131',
        uniquePatientKey: '131',
        studyId: '1',
        status: true,
        entryMonths: 6.052631579,
        months: 16.18421053,
    },
    {
        patientId: '132',
        uniquePatientKey: '132',
        studyId: '1',
        status: false,
        entryMonths: 2.697368421,
        months: 16.25,
    },
    {
        patientId: '133',
        uniquePatientKey: '133',
        studyId: '1',
        status: true,
        entryMonths: 4.736842105,
        months: 16.38157895,
    },
    {
        patientId: '134',
        uniquePatientKey: '134',
        studyId: '1',
        status: true,
        entryMonths: 10.16447368,
        months: 16.44736842,
    },
    {
        patientId: '135',
        uniquePatientKey: '135',
        studyId: '1',
        status: true,
        entryMonths: 13.81578947,
        months: 16.48026316,
    },
    {
        patientId: '136',
        uniquePatientKey: '136',
        studyId: '1',
        status: true,
        entryMonths: 1.381578947,
        months: 16.57894737,
    },
    {
        patientId: '137',
        uniquePatientKey: '137',
        studyId: '1',
        status: true,
        entryMonths: 9.506578947,
        months: 16.57894737,
    },
    {
        patientId: '138',
        uniquePatientKey: '138',
        studyId: '1',
        status: true,
        entryMonths: 14.44078947,
        months: 16.71052632,
    },
    {
        patientId: '139',
        uniquePatientKey: '139',
        studyId: '1',
        status: false,
        entryMonths: 3.125,
        months: 16.74342105,
    },
    {
        patientId: '140',
        uniquePatientKey: '140',
        studyId: '1',
        status: true,
        entryMonths: 2.598684211,
        months: 16.77631579,
    },
    {
        patientId: '141',
        uniquePatientKey: '141',
        studyId: '1',
        status: true,
        entryMonths: 1.381578947,
        months: 16.84210526,
    },
    {
        patientId: '142',
        uniquePatientKey: '142',
        studyId: '1',
        status: true,
        entryMonths: 12.10526316,
        months: 16.84210526,
    },
    {
        patientId: '143',
        uniquePatientKey: '143',
        studyId: '1',
        status: true,
        entryMonths: 4.901315789,
        months: 17.00657895,
    },
    {
        patientId: '144',
        uniquePatientKey: '144',
        studyId: '1',
        status: true,
        entryMonths: 1.118421053,
        months: 17.10526316,
    },
    {
        patientId: '145',
        uniquePatientKey: '145',
        studyId: '1',
        status: true,
        entryMonths: 10.46052632,
        months: 17.36842105,
    },
    {
        patientId: '146',
        uniquePatientKey: '146',
        studyId: '1',
        status: true,
        entryMonths: 10.42763158,
        months: 17.5,
    },
    {
        patientId: '147',
        uniquePatientKey: '147',
        studyId: '1',
        status: true,
        entryMonths: 7.434210526,
        months: 17.59868421,
    },
    {
        patientId: '148',
        uniquePatientKey: '148',
        studyId: '1',
        status: true,
        entryMonths: 13.65131579,
        months: 17.63157895,
    },
    {
        patientId: '149',
        uniquePatientKey: '149',
        studyId: '1',
        status: true,
        entryMonths: 6.052631579,
        months: 17.69736842,
    },
    {
        patientId: '150',
        uniquePatientKey: '150',
        studyId: '1',
        status: true,
        entryMonths: 15.88815789,
        months: 17.89473684,
    },
    {
        patientId: '152',
        uniquePatientKey: '152',
        studyId: '1',
        status: true,
        entryMonths: 3.355263158,
        months: 17.99342105,
    },
    {
        patientId: '151',
        uniquePatientKey: '151',
        studyId: '1',
        status: false,
        entryMonths: 2.828947368,
        months: 17.99342105,
    },
    {
        patientId: '153',
        uniquePatientKey: '153',
        studyId: '1',
        status: false,
        entryMonths: 3.585526316,
        months: 17.99342105,
    },
    {
        patientId: '154',
        uniquePatientKey: '154',
        studyId: '1',
        status: true,
        entryMonths: 2.5,
        months: 18.09210526,
    },
    {
        patientId: '155',
        uniquePatientKey: '155',
        studyId: '1',
        status: true,
        entryMonths: 1.907894737,
        months: 18.51973684,
    },
    {
        patientId: '156',
        uniquePatientKey: '156',
        studyId: '1',
        status: false,
        entryMonths: 4.276315789,
        months: 18.58552632,
    },
    {
        patientId: '157',
        uniquePatientKey: '157',
        studyId: '1',
        status: true,
        entryMonths: 15.09868421,
        months: 18.61842105,
    },
    {
        patientId: '158',
        uniquePatientKey: '158',
        studyId: '1',
        status: true,
        entryMonths: 5.789473684,
        months: 18.65131579,
    },
    {
        patientId: '159',
        uniquePatientKey: '159',
        studyId: '1',
        status: true,
        entryMonths: 4.144736842,
        months: 18.75,
    },
    {
        patientId: '160',
        uniquePatientKey: '160',
        studyId: '1',
        status: true,
        entryMonths: 6.447368421,
        months: 18.91447368,
    },
    {
        patientId: '162',
        uniquePatientKey: '162',
        studyId: '1',
        status: true,
        entryMonths: 1.513157895,
        months: 19.01315789,
    },
    {
        patientId: '163',
        uniquePatientKey: '163',
        studyId: '1',
        status: true,
        entryMonths: 5.953947368,
        months: 19.01315789,
    },
    {
        patientId: '161',
        uniquePatientKey: '161',
        studyId: '1',
        status: false,
        entryMonths: 0.855263158,
        months: 19.01315789,
    },
    {
        patientId: '164',
        uniquePatientKey: '164',
        studyId: '1',
        status: true,
        entryMonths: 1.776315789,
        months: 19.11184211,
    },
    {
        patientId: '165',
        uniquePatientKey: '165',
        studyId: '1',
        status: true,
        entryMonths: 1.184210526,
        months: 19.47368421,
    },
    {
        patientId: '166',
        uniquePatientKey: '166',
        studyId: '1',
        status: true,
        entryMonths: 4.572368421,
        months: 19.57236842,
    },
    {
        patientId: '167',
        uniquePatientKey: '167',
        studyId: '1',
        status: true,
        entryMonths: 3.552631579,
        months: 19.60526316,
    },
    {
        patientId: '168',
        uniquePatientKey: '168',
        studyId: '1',
        status: true,
        entryMonths: 14.60526316,
        months: 19.60526316,
    },
    {
        patientId: '169',
        uniquePatientKey: '169',
        studyId: '1',
        status: true,
        entryMonths: 13.38815789,
        months: 19.76973684,
    },
    {
        patientId: '170',
        uniquePatientKey: '170',
        studyId: '1',
        status: true,
        entryMonths: 6.875,
        months: 19.80263158,
    },
    {
        patientId: '171',
        uniquePatientKey: '171',
        studyId: '1',
        status: false,
        entryMonths: 12.00657895,
        months: 19.80263158,
    },
    {
        patientId: '172',
        uniquePatientKey: '172',
        studyId: '1',
        status: true,
        entryMonths: 2.730263158,
        months: 19.83552632,
    },
    {
        patientId: '173',
        uniquePatientKey: '173',
        studyId: '1',
        status: true,
        entryMonths: 15.49342105,
        months: 19.86842105,
    },
    {
        patientId: '174',
        uniquePatientKey: '174',
        studyId: '1',
        status: true,
        entryMonths: 3.684210526,
        months: 19.90131579,
    },
    {
        patientId: '175',
        uniquePatientKey: '175',
        studyId: '1',
        status: true,
        entryMonths: 6.282894737,
        months: 19.93421053,
    },
    {
        patientId: '176',
        uniquePatientKey: '176',
        studyId: '1',
        status: true,
        entryMonths: 15.82236842,
        months: 19.93421053,
    },
    {
        patientId: '177',
        uniquePatientKey: '177',
        studyId: '1',
        status: false,
        entryMonths: 1.644736842,
        months: 19.96710526,
    },
    {
        patientId: '178',
        uniquePatientKey: '178',
        studyId: '1',
        status: true,
        entryMonths: 2.368421053,
        months: 20.06578947,
    },
    {
        patientId: '179',
        uniquePatientKey: '179',
        studyId: '1',
        status: true,
        entryMonths: 5.921052632,
        months: 20.49342105,
    },
    {
        patientId: '180',
        uniquePatientKey: '180',
        studyId: '1',
        status: true,
        entryMonths: 10.29605263,
        months: 20.55921053,
    },
    {
        patientId: '181',
        uniquePatientKey: '181',
        studyId: '1',
        status: true,
        entryMonths: 17.30263158,
        months: 20.59210526,
    },
    {
        patientId: '182',
        uniquePatientKey: '182',
        studyId: '1',
        status: true,
        entryMonths: 4.440789474,
        months: 20.75657895,
    },
    {
        patientId: '183',
        uniquePatientKey: '183',
        studyId: '1',
        status: true,
        entryMonths: 7.697368421,
        months: 20.78947368,
    },
    {
        patientId: '184',
        uniquePatientKey: '184',
        studyId: '1',
        status: true,
        entryMonths: 10.13157895,
        months: 20.85526316,
    },
    {
        patientId: '185',
        uniquePatientKey: '185',
        studyId: '1',
        status: true,
        entryMonths: 2.236842105,
        months: 20.98684211,
    },
    {
        patientId: '186',
        uniquePatientKey: '186',
        studyId: '1',
        status: true,
        entryMonths: 2.5,
        months: 21.05263158,
    },
    {
        patientId: '187',
        uniquePatientKey: '187',
        studyId: '1',
        status: false,
        entryMonths: 20.26315789,
        months: 21.21710526,
    },
    {
        patientId: '188',
        uniquePatientKey: '188',
        studyId: '1',
        status: true,
        entryMonths: 19.83552632,
        months: 21.31578947,
    },
    {
        patientId: '189',
        uniquePatientKey: '189',
        studyId: '1',
        status: true,
        entryMonths: 1.348684211,
        months: 21.38157895,
    },
    {
        patientId: '190',
        uniquePatientKey: '190',
        studyId: '1',
        status: true,
        entryMonths: 2.434210526,
        months: 21.44736842,
    },
    {
        patientId: '191',
        uniquePatientKey: '191',
        studyId: '1',
        status: false,
        entryMonths: 12.43421053,
        months: 21.44736842,
    },
    {
        patientId: '192',
        uniquePatientKey: '192',
        studyId: '1',
        status: true,
        entryMonths: 2.236842105,
        months: 21.48026316,
    },
    {
        patientId: '193',
        uniquePatientKey: '193',
        studyId: '1',
        status: true,
        entryMonths: 8.585526316,
        months: 21.51315789,
    },
    {
        patientId: '194',
        uniquePatientKey: '194',
        studyId: '1',
        status: true,
        entryMonths: 2.368421053,
        months: 21.54605263,
    },
    {
        patientId: '195',
        uniquePatientKey: '195',
        studyId: '1',
        status: false,
        entryMonths: 2.401315789,
        months: 21.57894737,
    },
    {
        patientId: '196',
        uniquePatientKey: '196',
        studyId: '1',
        status: true,
        entryMonths: 1.776315789,
        months: 21.67763158,
    },
    {
        patientId: '197',
        uniquePatientKey: '197',
        studyId: '1',
        status: true,
        entryMonths: 5.098684211,
        months: 21.67763158,
    },
    {
        patientId: '198',
        uniquePatientKey: '198',
        studyId: '1',
        status: true,
        entryMonths: 1.052631579,
        months: 21.71052632,
    },
    {
        patientId: '199',
        uniquePatientKey: '199',
        studyId: '1',
        status: true,
        entryMonths: 9.802631579,
        months: 21.74342105,
    },
    {
        patientId: '200',
        uniquePatientKey: '200',
        studyId: '1',
        status: true,
        entryMonths: 2.006578947,
        months: 21.97368421,
    },
    {
        patientId: '201',
        uniquePatientKey: '201',
        studyId: '1',
        status: true,
        entryMonths: 4.835526316,
        months: 22.00657895,
    },
    {
        patientId: '202',
        uniquePatientKey: '202',
        studyId: '1',
        status: true,
        entryMonths: 5.789473684,
        months: 22.07236842,
    },
    {
        patientId: '203',
        uniquePatientKey: '203',
        studyId: '1',
        status: true,
        entryMonths: 4.210526316,
        months: 22.17105263,
    },
    {
        patientId: '204',
        uniquePatientKey: '204',
        studyId: '1',
        status: true,
        entryMonths: 4.342105263,
        months: 22.23684211,
    },
    {
        patientId: '205',
        uniquePatientKey: '205',
        studyId: '1',
        status: true,
        entryMonths: 2.203947368,
        months: 22.30263158,
    },
    {
        patientId: '206',
        uniquePatientKey: '206',
        studyId: '1',
        status: true,
        entryMonths: 21.57894737,
        months: 22.30263158,
    },
    {
        patientId: '207',
        uniquePatientKey: '207',
        studyId: '1',
        status: true,
        entryMonths: 2.039473684,
        months: 22.33552632,
    },
    {
        patientId: '208',
        uniquePatientKey: '208',
        studyId: '1',
        status: true,
        entryMonths: 2.138157895,
        months: 22.46710526,
    },
    {
        patientId: '209',
        uniquePatientKey: '209',
        studyId: '1',
        status: true,
        entryMonths: 1.480263158,
        months: 22.66447368,
    },
    {
        patientId: '211',
        uniquePatientKey: '211',
        studyId: '1',
        status: true,
        entryMonths: 11.74342105,
        months: 22.66447368,
    },
    {
        patientId: '210',
        uniquePatientKey: '210',
        studyId: '1',
        status: false,
        entryMonths: 3.585526316,
        months: 22.66447368,
    },
    {
        patientId: '212',
        uniquePatientKey: '212',
        studyId: '1',
        status: true,
        entryMonths: 12.03947368,
        months: 23.02631579,
    },
    {
        patientId: '213',
        uniquePatientKey: '213',
        studyId: '1',
        status: true,
        entryMonths: 12.46710526,
        months: 23.05921053,
    },
    {
        patientId: '214',
        uniquePatientKey: '214',
        studyId: '1',
        status: true,
        entryMonths: 9.177631579,
        months: 23.09210526,
    },
    {
        patientId: '215',
        uniquePatientKey: '215',
        studyId: '1',
        status: true,
        entryMonths: 1.480263158,
        months: 23.125,
    },
    {
        patientId: '216',
        uniquePatientKey: '216',
        studyId: '1',
        status: true,
        entryMonths: 2.006578947,
        months: 23.19078947,
    },
    {
        patientId: '217',
        uniquePatientKey: '217',
        studyId: '1',
        status: true,
        entryMonths: 19.11184211,
        months: 23.19078947,
    },
    {
        patientId: '218',
        uniquePatientKey: '218',
        studyId: '1',
        status: true,
        entryMonths: 18.22368421,
        months: 23.22368421,
    },
    {
        patientId: '219',
        uniquePatientKey: '219',
        studyId: '1',
        status: true,
        entryMonths: 22.20394737,
        months: 23.22368421,
    },
    {
        patientId: '220',
        uniquePatientKey: '220',
        studyId: '1',
        status: true,
        entryMonths: 1.940789474,
        months: 23.28947368,
    },
    {
        patientId: '221',
        uniquePatientKey: '221',
        studyId: '1',
        status: true,
        entryMonths: 5.296052632,
        months: 23.28947368,
    },
    {
        patientId: '222',
        uniquePatientKey: '222',
        studyId: '1',
        status: true,
        entryMonths: 7.993421053,
        months: 23.42105263,
    },
    {
        patientId: '223',
        uniquePatientKey: '223',
        studyId: '1',
        status: true,
        entryMonths: 22.56578947,
        months: 23.48684211,
    },
    {
        patientId: '224',
        uniquePatientKey: '224',
        studyId: '1',
        status: true,
        entryMonths: 21.64473684,
        months: 23.61842105,
    },
    {
        patientId: '225',
        uniquePatientKey: '225',
        studyId: '1',
        status: true,
        entryMonths: 10.32894737,
        months: 23.71710526,
    },
    {
        patientId: '226',
        uniquePatientKey: '226',
        studyId: '1',
        status: true,
        entryMonths: 2.631578947,
        months: 23.75,
    },
    {
        patientId: '227',
        uniquePatientKey: '227',
        studyId: '1',
        status: true,
        entryMonths: 4.407894737,
        months: 23.75,
    },
    {
        patientId: '228',
        uniquePatientKey: '228',
        studyId: '1',
        status: true,
        entryMonths: 1.710526316,
        months: 24.04605263,
    },
    {
        patientId: '229',
        uniquePatientKey: '229',
        studyId: '1',
        status: true,
        entryMonths: 2.697368421,
        months: 24.07894737,
    },
    {
        patientId: '230',
        uniquePatientKey: '230',
        studyId: '1',
        status: true,
        entryMonths: 10.29605263,
        months: 24.27631579,
    },
    {
        patientId: '231',
        uniquePatientKey: '231',
        studyId: '1',
        status: true,
        entryMonths: 3.453947368,
        months: 24.375,
    },
    {
        patientId: '232',
        uniquePatientKey: '232',
        studyId: '1',
        status: true,
        entryMonths: 12.23684211,
        months: 24.44078947,
    },
    {
        patientId: '233',
        uniquePatientKey: '233',
        studyId: '1',
        status: true,
        entryMonths: 16.94078947,
        months: 24.47368421,
    },
    {
        patientId: '234',
        uniquePatientKey: '234',
        studyId: '1',
        status: true,
        entryMonths: 20.78947368,
        months: 24.53947368,
    },
    {
        patientId: '235',
        uniquePatientKey: '235',
        studyId: '1',
        status: true,
        entryMonths: 5.855263158,
        months: 24.73684211,
    },
    {
        patientId: '236',
        uniquePatientKey: '236',
        studyId: '1',
        status: true,
        entryMonths: 8.092105263,
        months: 24.76973684,
    },
    {
        patientId: '237',
        uniquePatientKey: '237',
        studyId: '1',
        status: true,
        entryMonths: 2.302631579,
        months: 24.80263158,
    },
    {
        patientId: '238',
        uniquePatientKey: '238',
        studyId: '1',
        status: true,
        entryMonths: 4.440789474,
        months: 24.83552632,
    },
    {
        patientId: '239',
        uniquePatientKey: '239',
        studyId: '1',
        status: true,
        entryMonths: 19.76973684,
        months: 24.86842105,
    },
    {
        patientId: '240',
        uniquePatientKey: '240',
        studyId: '1',
        status: true,
        entryMonths: 20.16447368,
        months: 25.06578947,
    },
    {
        patientId: '241',
        uniquePatientKey: '241',
        studyId: '1',
        status: true,
        entryMonths: 2.434210526,
        months: 25.09868421,
    },
    {
        patientId: '242',
        uniquePatientKey: '242',
        studyId: '1',
        status: true,
        entryMonths: 13.45394737,
        months: 25.13157895,
    },
    {
        patientId: '243',
        uniquePatientKey: '243',
        studyId: '1',
        status: false,
        entryMonths: 2.171052632,
        months: 25.36184211,
    },
    {
        patientId: '244',
        uniquePatientKey: '244',
        studyId: '1',
        status: true,
        entryMonths: 2.730263158,
        months: 25.42763158,
    },
    {
        patientId: '245',
        uniquePatientKey: '245',
        studyId: '1',
        status: false,
        entryMonths: 4.506578947,
        months: 25.69078947,
    },
    {
        patientId: '246',
        uniquePatientKey: '246',
        studyId: '1',
        status: true,
        entryMonths: 2.302631579,
        months: 25.72368421,
    },
    {
        patientId: '247',
        uniquePatientKey: '247',
        studyId: '1',
        status: true,
        entryMonths: 12.89473684,
        months: 25.75657895,
    },
    {
        patientId: '248',
        uniquePatientKey: '248',
        studyId: '1',
        status: false,
        entryMonths: 18.88157895,
        months: 26.01973684,
    },
    {
        patientId: '250',
        uniquePatientKey: '250',
        studyId: '1',
        status: true,
        entryMonths: 12.00657895,
        months: 26.25,
    },
    {
        patientId: '251',
        uniquePatientKey: '251',
        studyId: '1',
        status: true,
        entryMonths: 12.36842105,
        months: 26.25,
    },
    {
        patientId: '249',
        uniquePatientKey: '249',
        studyId: '1',
        status: false,
        entryMonths: 1.743421053,
        months: 26.25,
    },
    {
        patientId: '252',
        uniquePatientKey: '252',
        studyId: '1',
        status: false,
        entryMonths: 5.822368421,
        months: 26.34868421,
    },
    {
        patientId: '253',
        uniquePatientKey: '253',
        studyId: '1',
        status: true,
        entryMonths: 23.19078947,
        months: 26.38157895,
    },
    {
        patientId: '254',
        uniquePatientKey: '254',
        studyId: '1',
        status: true,
        entryMonths: 0.953947368,
        months: 26.57894737,
    },
    {
        patientId: '255',
        uniquePatientKey: '255',
        studyId: '1',
        status: true,
        entryMonths: 1.842105263,
        months: 26.57894737,
    },
    {
        patientId: '256',
        uniquePatientKey: '256',
        studyId: '1',
        status: true,
        entryMonths: 20.88815789,
        months: 26.61184211,
    },
    {
        patientId: '258',
        uniquePatientKey: '258',
        studyId: '1',
        status: true,
        entryMonths: 21.77631579,
        months: 26.71052632,
    },
    {
        patientId: '257',
        uniquePatientKey: '257',
        studyId: '1',
        status: false,
        entryMonths: 9.111842105,
        months: 26.71052632,
    },
    {
        patientId: '259',
        uniquePatientKey: '259',
        studyId: '1',
        status: true,
        entryMonths: 1.677631579,
        months: 26.77631579,
    },
    {
        patientId: '260',
        uniquePatientKey: '260',
        studyId: '1',
        status: true,
        entryMonths: 2.565789474,
        months: 26.77631579,
    },
    {
        patientId: '261',
        uniquePatientKey: '261',
        studyId: '1',
        status: true,
        entryMonths: 17.76315789,
        months: 26.77631579,
    },
    {
        patientId: '262',
        uniquePatientKey: '262',
        studyId: '1',
        status: false,
        entryMonths: 2.138157895,
        months: 26.80921053,
    },
    {
        patientId: '263',
        uniquePatientKey: '263',
        studyId: '1',
        status: true,
        entryMonths: 11.05263158,
        months: 26.84210526,
    },
    {
        patientId: '264',
        uniquePatientKey: '264',
        studyId: '1',
        status: true,
        entryMonths: 24.17763158,
        months: 27.03947368,
    },
    {
        patientId: '265',
        uniquePatientKey: '265',
        studyId: '1',
        status: true,
        entryMonths: 21.18421053,
        months: 27.20394737,
    },
    {
        patientId: '266',
        uniquePatientKey: '266',
        studyId: '1',
        status: true,
        entryMonths: 8.059210526,
        months: 27.23684211,
    },
    {
        patientId: '267',
        uniquePatientKey: '267',
        studyId: '1',
        status: true,
        entryMonths: 6.052631579,
        months: 27.30263158,
    },
    {
        patientId: '268',
        uniquePatientKey: '268',
        studyId: '1',
        status: false,
        entryMonths: 4.144736842,
        months: 27.33552632,
    },
    {
        patientId: '269',
        uniquePatientKey: '269',
        studyId: '1',
        status: true,
        entryMonths: 1.842105263,
        months: 27.43421053,
    },
    {
        patientId: '270',
        uniquePatientKey: '270',
        studyId: '1',
        status: true,
        entryMonths: 2.105263158,
        months: 27.46710526,
    },
    {
        patientId: '271',
        uniquePatientKey: '271',
        studyId: '1',
        status: true,
        entryMonths: 14.93421053,
        months: 27.53289474,
    },
    {
        patientId: '272',
        uniquePatientKey: '272',
        studyId: '1',
        status: true,
        entryMonths: 27.23684211,
        months: 27.53289474,
    },
    {
        patientId: '273',
        uniquePatientKey: '273',
        studyId: '1',
        status: true,
        entryMonths: 9.506578947,
        months: 27.63157895,
    },
    {
        patientId: '274',
        uniquePatientKey: '274',
        studyId: '1',
        status: true,
        entryMonths: 24.21052632,
        months: 27.69736842,
    },
    {
        patientId: '275',
        uniquePatientKey: '275',
        studyId: '1',
        status: true,
        entryMonths: 5.986842105,
        months: 27.79605263,
    },
    {
        patientId: '276',
        uniquePatientKey: '276',
        studyId: '1',
        status: false,
        entryMonths: 4.013157895,
        months: 27.86184211,
    },
    {
        patientId: '277',
        uniquePatientKey: '277',
        studyId: '1',
        status: true,
        entryMonths: 12.23684211,
        months: 28.05921053,
    },
    {
        patientId: '278',
        uniquePatientKey: '278',
        studyId: '1',
        status: true,
        entryMonths: 5.032894737,
        months: 28.09210526,
    },
    {
        patientId: '279',
        uniquePatientKey: '279',
        studyId: '1',
        status: true,
        entryMonths: 2.171052632,
        months: 28.28947368,
    },
    {
        patientId: '280',
        uniquePatientKey: '280',
        studyId: '1',
        status: true,
        entryMonths: 1.875,
        months: 28.48684211,
    },
    {
        patientId: '281',
        uniquePatientKey: '281',
        studyId: '1',
        status: true,
        entryMonths: 22.23684211,
        months: 28.48684211,
    },
    {
        patientId: '282',
        uniquePatientKey: '282',
        studyId: '1',
        status: true,
        entryMonths: 2.203947368,
        months: 28.71710526,
    },
    {
        patientId: '283',
        uniquePatientKey: '283',
        studyId: '1',
        status: false,
        entryMonths: 2.401315789,
        months: 28.71710526,
    },
    {
        patientId: '284',
        uniquePatientKey: '284',
        studyId: '1',
        status: true,
        entryMonths: 15.09868421,
        months: 28.81578947,
    },
    {
        patientId: '285',
        uniquePatientKey: '285',
        studyId: '1',
        status: true,
        entryMonths: 1.776315789,
        months: 28.88157895,
    },
    {
        patientId: '286',
        uniquePatientKey: '286',
        studyId: '1',
        status: true,
        entryMonths: 20.46052632,
        months: 28.94736842,
    },
    {
        patientId: '287',
        uniquePatientKey: '287',
        studyId: '1',
        status: true,
        entryMonths: 4.342105263,
        months: 29.11184211,
    },
    {
        patientId: '288',
        uniquePatientKey: '288',
        studyId: '1',
        status: true,
        entryMonths: 1.381578947,
        months: 29.14473684,
    },
    {
        patientId: '289',
        uniquePatientKey: '289',
        studyId: '1',
        status: false,
        entryMonths: 2.368421053,
        months: 29.27631579,
    },
    {
        patientId: '290',
        uniquePatientKey: '290',
        studyId: '1',
        status: true,
        entryMonths: 21.64473684,
        months: 29.40789474,
    },
    {
        patientId: '291',
        uniquePatientKey: '291',
        studyId: '1',
        status: true,
        entryMonths: 1.546052632,
        months: 29.63815789,
    },
    {
        patientId: '292',
        uniquePatientKey: '292',
        studyId: '1',
        status: true,
        entryMonths: 5.625,
        months: 29.76973684,
    },
    {
        patientId: '293',
        uniquePatientKey: '293',
        studyId: '1',
        status: true,
        entryMonths: 3.914473684,
        months: 30.32894737,
    },
    {
        patientId: '294',
        uniquePatientKey: '294',
        studyId: '1',
        status: true,
        entryMonths: 12.59868421,
        months: 30.32894737,
    },
    {
        patientId: '295',
        uniquePatientKey: '295',
        studyId: '1',
        status: true,
        entryMonths: 1.381578947,
        months: 30.39473684,
    },
    {
        patientId: '296',
        uniquePatientKey: '296',
        studyId: '1',
        status: true,
        entryMonths: 0.789473684,
        months: 30.42763158,
    },
    {
        patientId: '297',
        uniquePatientKey: '297',
        studyId: '1',
        status: true,
        entryMonths: 8.717105263,
        months: 30.52631579,
    },
    {
        patientId: '299',
        uniquePatientKey: '299',
        studyId: '1',
        status: true,
        entryMonths: 8.947368421,
        months: 30.75657895,
    },
    {
        patientId: '298',
        uniquePatientKey: '298',
        studyId: '1',
        status: false,
        entryMonths: 2.5,
        months: 30.75657895,
    },
    {
        patientId: '301',
        uniquePatientKey: '301',
        studyId: '1',
        status: true,
        entryMonths: 22.59868421,
        months: 30.78947368,
    },
    {
        patientId: '300',
        uniquePatientKey: '300',
        studyId: '1',
        status: false,
        entryMonths: 2.861842105,
        months: 30.78947368,
    },
    {
        patientId: '302',
        uniquePatientKey: '302',
        studyId: '1',
        status: false,
        entryMonths: 1.513157895,
        months: 30.88815789,
    },
    {
        patientId: '303',
        uniquePatientKey: '303',
        studyId: '1',
        status: false,
        entryMonths: 12.86184211,
        months: 31.01973684,
    },
    {
        patientId: '305',
        uniquePatientKey: '305',
        studyId: '1',
        status: true,
        entryMonths: 8.881578947,
        months: 31.11842105,
    },
    {
        patientId: '306',
        uniquePatientKey: '306',
        studyId: '1',
        status: true,
        entryMonths: 14.17763158,
        months: 31.11842105,
    },
    {
        patientId: '304',
        uniquePatientKey: '304',
        studyId: '1',
        status: false,
        entryMonths: 1.940789474,
        months: 31.11842105,
    },
    {
        patientId: '307',
        uniquePatientKey: '307',
        studyId: '1',
        status: true,
        entryMonths: 1.25,
        months: 31.15131579,
    },
    {
        patientId: '308',
        uniquePatientKey: '308',
        studyId: '1',
        status: true,
        entryMonths: 1.546052632,
        months: 31.18421053,
    },
    {
        patientId: '310',
        uniquePatientKey: '310',
        studyId: '1',
        status: true,
        entryMonths: 4.177631579,
        months: 31.18421053,
    },
    {
        patientId: '309',
        uniquePatientKey: '309',
        studyId: '1',
        status: false,
        entryMonths: 2.072368421,
        months: 31.18421053,
    },
    {
        patientId: '311',
        uniquePatientKey: '311',
        studyId: '1',
        status: true,
        entryMonths: 23.88157895,
        months: 31.28289474,
    },
    {
        patientId: '312',
        uniquePatientKey: '312',
        studyId: '1',
        status: true,
        entryMonths: 30.26315789,
        months: 31.28289474,
    },
    {
        patientId: '313',
        uniquePatientKey: '313',
        studyId: '1',
        status: true,
        entryMonths: 18.65131579,
        months: 31.31578947,
    },
    {
        patientId: '314',
        uniquePatientKey: '314',
        studyId: '1',
        status: false,
        entryMonths: 0.986842105,
        months: 31.41447368,
    },
    {
        patientId: '315',
        uniquePatientKey: '315',
        studyId: '1',
        status: true,
        entryMonths: 10.29605263,
        months: 31.44736842,
    },
    {
        patientId: '316',
        uniquePatientKey: '316',
        studyId: '1',
        status: true,
        entryMonths: 2.368421053,
        months: 31.51315789,
    },
    {
        patientId: '317',
        uniquePatientKey: '317',
        studyId: '1',
        status: true,
        entryMonths: 12.13815789,
        months: 31.61184211,
    },
    {
        patientId: '318',
        uniquePatientKey: '318',
        studyId: '1',
        status: true,
        entryMonths: 30.36184211,
        months: 31.71052632,
    },
    {
        patientId: '319',
        uniquePatientKey: '319',
        studyId: '1',
        status: true,
        entryMonths: 3.256578947,
        months: 31.74342105,
    },
    {
        patientId: '320',
        uniquePatientKey: '320',
        studyId: '1',
        status: false,
        entryMonths: 5.263157895,
        months: 31.74342105,
    },
    {
        patientId: '321',
        uniquePatientKey: '321',
        studyId: '1',
        status: true,
        entryMonths: 1.776315789,
        months: 31.80921053,
    },
    {
        patientId: '322',
        uniquePatientKey: '322',
        studyId: '1',
        status: true,
        entryMonths: 15.59210526,
        months: 31.80921053,
    },
    {
        patientId: '323',
        uniquePatientKey: '323',
        studyId: '1',
        status: false,
        entryMonths: 2.171052632,
        months: 31.84210526,
    },
    {
        patientId: '324',
        uniquePatientKey: '324',
        studyId: '1',
        status: false,
        entryMonths: 15.26315789,
        months: 31.97368421,
    },
    {
        patientId: '325',
        uniquePatientKey: '325',
        studyId: '1',
        status: false,
        entryMonths: 29.07894737,
        months: 31.97368421,
    },
    {
        patientId: '326',
        uniquePatientKey: '326',
        studyId: '1',
        status: true,
        entryMonths: 3.585526316,
        months: 32.00657895,
    },
    {
        patientId: '327',
        uniquePatientKey: '327',
        studyId: '1',
        status: false,
        entryMonths: 2.565789474,
        months: 32.10526316,
    },
    {
        patientId: '328',
        uniquePatientKey: '328',
        studyId: '1',
        status: false,
        entryMonths: 1.644736842,
        months: 32.17105263,
    },
    {
        patientId: '329',
        uniquePatientKey: '329',
        studyId: '1',
        status: false,
        entryMonths: 3.388157895,
        months: 32.17105263,
    },
    {
        patientId: '330',
        uniquePatientKey: '330',
        studyId: '1',
        status: false,
        entryMonths: 3.848684211,
        months: 32.20394737,
    },
    {
        patientId: '331',
        uniquePatientKey: '331',
        studyId: '1',
        status: true,
        entryMonths: 2.236842105,
        months: 32.26973684,
    },
    {
        patientId: '332',
        uniquePatientKey: '332',
        studyId: '1',
        status: true,
        entryMonths: 25.16447368,
        months: 32.33552632,
    },
    {
        patientId: '333',
        uniquePatientKey: '333',
        studyId: '1',
        status: true,
        entryMonths: 11.28289474,
        months: 32.66447368,
    },
    {
        patientId: '334',
        uniquePatientKey: '334',
        studyId: '1',
        status: false,
        entryMonths: 19.21052632,
        months: 32.66447368,
    },
    {
        patientId: '335',
        uniquePatientKey: '335',
        studyId: '1',
        status: true,
        entryMonths: 5.328947368,
        months: 32.69736842,
    },
    {
        patientId: '336',
        uniquePatientKey: '336',
        studyId: '1',
        status: false,
        entryMonths: 0.921052632,
        months: 32.73026316,
    },
    {
        patientId: '337',
        uniquePatientKey: '337',
        studyId: '1',
        status: true,
        entryMonths: 31.77631579,
        months: 32.92763158,
    },
    {
        patientId: '338',
        uniquePatientKey: '338',
        studyId: '1',
        status: false,
        entryMonths: 8.289473684,
        months: 32.96052632,
    },
    {
        patientId: '339',
        uniquePatientKey: '339',
        studyId: '1',
        status: false,
        entryMonths: 3.421052632,
        months: 32.99342105,
    },
    {
        patientId: '340',
        uniquePatientKey: '340',
        studyId: '1',
        status: true,
        entryMonths: 19.60526316,
        months: 33.05921053,
    },
    {
        patientId: '341',
        uniquePatientKey: '341',
        studyId: '1',
        status: false,
        entryMonths: 4.013157895,
        months: 33.15789474,
    },
    {
        patientId: '342',
        uniquePatientKey: '342',
        studyId: '1',
        status: true,
        entryMonths: 23.61842105,
        months: 33.22368421,
    },
    {
        patientId: '343',
        uniquePatientKey: '343',
        studyId: '1',
        status: true,
        entryMonths: 24.11184211,
        months: 33.28947368,
    },
    {
        patientId: '344',
        uniquePatientKey: '344',
        studyId: '1',
        status: true,
        entryMonths: 1.776315789,
        months: 33.32236842,
    },
    {
        patientId: '345',
        uniquePatientKey: '345',
        studyId: '1',
        status: true,
        entryMonths: 32.59868421,
        months: 33.55263158,
    },
    {
        patientId: '346',
        uniquePatientKey: '346',
        studyId: '1',
        status: true,
        entryMonths: 2.828947368,
        months: 33.58552632,
    },
    {
        patientId: '347',
        uniquePatientKey: '347',
        studyId: '1',
        status: true,
        entryMonths: 17.5,
        months: 33.68421053,
    },
    {
        patientId: '348',
        uniquePatientKey: '348',
        studyId: '1',
        status: false,
        entryMonths: 5.361842105,
        months: 33.84868421,
    },
    {
        patientId: '349',
        uniquePatientKey: '349',
        studyId: '1',
        status: false,
        entryMonths: 7.598684211,
        months: 33.98026316,
    },
    {
        patientId: '350',
        uniquePatientKey: '350',
        studyId: '1',
        status: true,
        entryMonths: 3.618421053,
        months: 34.07894737,
    },
    {
        patientId: '351',
        uniquePatientKey: '351',
        studyId: '1',
        status: true,
        entryMonths: 13.125,
        months: 34.14473684,
    },
    {
        patientId: '352',
        uniquePatientKey: '352',
        studyId: '1',
        status: true,
        entryMonths: 2.039473684,
        months: 34.21052632,
    },
    {
        patientId: '353',
        uniquePatientKey: '353',
        studyId: '1',
        status: true,
        entryMonths: 4.013157895,
        months: 34.21052632,
    },
    {
        patientId: '354',
        uniquePatientKey: '354',
        studyId: '1',
        status: true,
        entryMonths: 16.97368421,
        months: 34.27631579,
    },
    {
        patientId: '355',
        uniquePatientKey: '355',
        studyId: '1',
        status: true,
        entryMonths: 28.28947368,
        months: 34.27631579,
    },
    {
        patientId: '356',
        uniquePatientKey: '356',
        studyId: '1',
        status: false,
        entryMonths: 3.421052632,
        months: 34.30921053,
    },
    {
        patientId: '357',
        uniquePatientKey: '357',
        studyId: '1',
        status: true,
        entryMonths: 19.07894737,
        months: 34.375,
    },
    {
        patientId: '358',
        uniquePatientKey: '358',
        studyId: '1',
        status: false,
        entryMonths: 3.059210526,
        months: 34.40789474,
    },
    {
        patientId: '359',
        uniquePatientKey: '359',
        studyId: '1',
        status: true,
        entryMonths: 25.95394737,
        months: 34.57236842,
    },
    {
        patientId: '360',
        uniquePatientKey: '360',
        studyId: '1',
        status: true,
        entryMonths: 22.33552632,
        months: 34.80263158,
    },
    {
        patientId: '361',
        uniquePatientKey: '361',
        studyId: '1',
        status: true,
        entryMonths: 1.776315789,
        months: 34.86842105,
    },
    {
        patientId: '362',
        uniquePatientKey: '362',
        studyId: '1',
        status: true,
        entryMonths: 4.769736842,
        months: 34.90131579,
    },
    {
        patientId: '363',
        uniquePatientKey: '363',
        studyId: '1',
        status: false,
        entryMonths: 3.289473684,
        months: 35.03289474,
    },
    {
        patientId: '364',
        uniquePatientKey: '364',
        studyId: '1',
        status: true,
        entryMonths: 13.125,
        months: 35.06578947,
    },
    {
        patientId: '365',
        uniquePatientKey: '365',
        studyId: '1',
        status: true,
        entryMonths: 20.98684211,
        months: 35.09868421,
    },
    {
        patientId: '366',
        uniquePatientKey: '366',
        studyId: '1',
        status: false,
        entryMonths: 17.56578947,
        months: 35.16447368,
    },
    {
        patientId: '367',
        uniquePatientKey: '367',
        studyId: '1',
        status: true,
        entryMonths: 27.17105263,
        months: 35.19736842,
    },
    {
        patientId: '368',
        uniquePatientKey: '368',
        studyId: '1',
        status: false,
        entryMonths: 1.710526316,
        months: 35.49342105,
    },
    {
        patientId: '369',
        uniquePatientKey: '369',
        studyId: '1',
        status: false,
        entryMonths: 1.381578947,
        months: 35.65789474,
    },
    {
        patientId: '370',
        uniquePatientKey: '370',
        studyId: '1',
        status: false,
        entryMonths: 2.335526316,
        months: 35.69078947,
    },
    {
        patientId: '371',
        uniquePatientKey: '371',
        studyId: '1',
        status: true,
        entryMonths: 2.105263158,
        months: 35.82236842,
    },
    {
        patientId: '372',
        uniquePatientKey: '372',
        studyId: '1',
        status: false,
        entryMonths: 2.927631579,
        months: 35.92105263,
    },
    {
        patientId: '373',
        uniquePatientKey: '373',
        studyId: '1',
        status: true,
        entryMonths: 17.46710526,
        months: 35.95394737,
    },
    {
        patientId: '376',
        uniquePatientKey: '376',
        studyId: '1',
        status: true,
        entryMonths: 21.41447368,
        months: 36.18421053,
    },
    {
        patientId: '374',
        uniquePatientKey: '374',
        studyId: '1',
        status: false,
        entryMonths: 2.730263158,
        months: 36.18421053,
    },
    {
        patientId: '375',
        uniquePatientKey: '375',
        studyId: '1',
        status: false,
        entryMonths: 3.453947368,
        months: 36.18421053,
    },
    {
        patientId: '377',
        uniquePatientKey: '377',
        studyId: '1',
        status: false,
        entryMonths: 3.815789474,
        months: 36.21710526,
    },
    {
        patientId: '378',
        uniquePatientKey: '378',
        studyId: '1',
        status: true,
        entryMonths: 2.302631579,
        months: 36.34868421,
    },
    {
        patientId: '379',
        uniquePatientKey: '379',
        studyId: '1',
        status: true,
        entryMonths: 20.46052632,
        months: 36.38157895,
    },
    {
        patientId: '380',
        uniquePatientKey: '380',
        studyId: '1',
        status: true,
        entryMonths: 23.05921053,
        months: 36.51315789,
    },
    {
        patientId: '381',
        uniquePatientKey: '381',
        studyId: '1',
        status: true,
        entryMonths: 33.81578947,
        months: 36.54605263,
    },
    {
        patientId: '382',
        uniquePatientKey: '382',
        studyId: '1',
        status: false,
        entryMonths: 10.06578947,
        months: 36.57894737,
    },
    {
        patientId: '383',
        uniquePatientKey: '383',
        studyId: '1',
        status: false,
        entryMonths: 3.223684211,
        months: 36.61184211,
    },
    {
        patientId: '384',
        uniquePatientKey: '384',
        studyId: '1',
        status: false,
        entryMonths: 0.888157895,
        months: 36.64473684,
    },
    {
        patientId: '385',
        uniquePatientKey: '385',
        studyId: '1',
        status: false,
        entryMonths: 2.894736842,
        months: 36.80921053,
    },
    {
        patientId: '386',
        uniquePatientKey: '386',
        studyId: '1',
        status: true,
        entryMonths: 24.67105263,
        months: 37.17105263,
    },
    {
        patientId: '387',
        uniquePatientKey: '387',
        studyId: '1',
        status: false,
        entryMonths: 1.842105263,
        months: 37.20394737,
    },
    {
        patientId: '388',
        uniquePatientKey: '388',
        studyId: '1',
        status: true,
        entryMonths: 6.052631579,
        months: 37.23684211,
    },
    {
        patientId: '389',
        uniquePatientKey: '389',
        studyId: '1',
        status: false,
        entryMonths: 1.644736842,
        months: 37.33552632,
    },
    {
        patientId: '390',
        uniquePatientKey: '390',
        studyId: '1',
        status: false,
        entryMonths: 6.315789474,
        months: 37.33552632,
    },
    {
        patientId: '391',
        uniquePatientKey: '391',
        studyId: '1',
        status: false,
        entryMonths: 2.236842105,
        months: 37.43421053,
    },
    {
        patientId: '392',
        uniquePatientKey: '392',
        studyId: '1',
        status: false,
        entryMonths: 1.019736842,
        months: 37.59868421,
    },
    {
        patientId: '394',
        uniquePatientKey: '394',
        studyId: '1',
        status: true,
        entryMonths: 32.69736842,
        months: 37.66447368,
    },
    {
        patientId: '393',
        uniquePatientKey: '393',
        studyId: '1',
        status: false,
        entryMonths: 2.697368421,
        months: 37.66447368,
    },
    {
        patientId: '395',
        uniquePatientKey: '395',
        studyId: '1',
        status: true,
        entryMonths: 14.04605263,
        months: 37.73026316,
    },
    {
        patientId: '396',
        uniquePatientKey: '396',
        studyId: '1',
        status: false,
        entryMonths: 1.447368421,
        months: 37.76315789,
    },
    {
        patientId: '397',
        uniquePatientKey: '397',
        studyId: '1',
        status: false,
        entryMonths: 1.414473684,
        months: 37.79605263,
    },
    {
        patientId: '398',
        uniquePatientKey: '398',
        studyId: '1',
        status: false,
        entryMonths: 1.776315789,
        months: 37.86184211,
    },
    {
        patientId: '399',
        uniquePatientKey: '399',
        studyId: '1',
        status: true,
        entryMonths: 17.76315789,
        months: 37.96052632,
    },
    {
        patientId: '400',
        uniquePatientKey: '400',
        studyId: '1',
        status: true,
        entryMonths: 19.44078947,
        months: 37.99342105,
    },
    {
        patientId: '401',
        uniquePatientKey: '401',
        studyId: '1',
        status: true,
        entryMonths: 2.894736842,
        months: 38.09210526,
    },
    {
        patientId: '402',
        uniquePatientKey: '402',
        studyId: '1',
        status: false,
        entryMonths: 6.776315789,
        months: 38.125,
    },
    {
        patientId: '404',
        uniquePatientKey: '404',
        studyId: '1',
        status: true,
        entryMonths: 29.93421053,
        months: 38.19078947,
    },
    {
        patientId: '403',
        uniquePatientKey: '403',
        studyId: '1',
        status: false,
        entryMonths: 21.01973684,
        months: 38.19078947,
    },
    {
        patientId: '405',
        uniquePatientKey: '405',
        studyId: '1',
        status: true,
        entryMonths: 10.625,
        months: 38.25657895,
    },
    {
        patientId: '406',
        uniquePatientKey: '406',
        studyId: '1',
        status: true,
        entryMonths: 19.63815789,
        months: 38.25657895,
    },
    {
        patientId: '407',
        uniquePatientKey: '407',
        studyId: '1',
        status: true,
        entryMonths: 10.09868421,
        months: 38.42105263,
    },
    {
        patientId: '408',
        uniquePatientKey: '408',
        studyId: '1',
        status: true,
        entryMonths: 27.00657895,
        months: 38.48684211,
    },
    {
        patientId: '409',
        uniquePatientKey: '409',
        studyId: '1',
        status: false,
        entryMonths: 1.875,
        months: 38.51973684,
    },
    {
        patientId: '410',
        uniquePatientKey: '410',
        studyId: '1',
        status: false,
        entryMonths: 6.907894737,
        months: 38.58552632,
    },
    {
        patientId: '411',
        uniquePatientKey: '411',
        studyId: '1',
        status: false,
        entryMonths: 2.138157895,
        months: 38.61842105,
    },
    {
        patientId: '412',
        uniquePatientKey: '412',
        studyId: '1',
        status: false,
        entryMonths: 20.42763158,
        months: 38.68421053,
    },
    {
        patientId: '413',
        uniquePatientKey: '413',
        studyId: '1',
        status: false,
        entryMonths: 3.519736842,
        months: 38.71710526,
    },
    {
        patientId: '414',
        uniquePatientKey: '414',
        studyId: '1',
        status: true,
        entryMonths: 5.131578947,
        months: 38.94736842,
    },
    {
        patientId: '415',
        uniquePatientKey: '415',
        studyId: '1',
        status: true,
        entryMonths: 11.80921053,
        months: 38.98026316,
    },
    {
        patientId: '416',
        uniquePatientKey: '416',
        studyId: '1',
        status: true,
        entryMonths: 3.190789474,
        months: 39.01315789,
    },
    {
        patientId: '417',
        uniquePatientKey: '417',
        studyId: '1',
        status: false,
        entryMonths: 2.467105263,
        months: 39.11184211,
    },
    {
        patientId: '418',
        uniquePatientKey: '418',
        studyId: '1',
        status: true,
        entryMonths: 7.763157895,
        months: 39.34210526,
    },
    {
        patientId: '419',
        uniquePatientKey: '419',
        studyId: '1',
        status: false,
        entryMonths: 3.026315789,
        months: 39.375,
    },
    {
        patientId: '420',
        uniquePatientKey: '420',
        studyId: '1',
        status: false,
        entryMonths: 2.960526316,
        months: 39.50657895,
    },
    {
        patientId: '421',
        uniquePatientKey: '421',
        studyId: '1',
        status: false,
        entryMonths: 1.480263158,
        months: 39.67105263,
    },
    {
        patientId: '422',
        uniquePatientKey: '422',
        studyId: '1',
        status: false,
        entryMonths: 1.776315789,
        months: 39.73684211,
    },
    {
        patientId: '423',
        uniquePatientKey: '423',
        studyId: '1',
        status: true,
        entryMonths: 11.67763158,
        months: 39.76973684,
    },
    {
        patientId: '424',
        uniquePatientKey: '424',
        studyId: '1',
        status: false,
        entryMonths: 4.605263158,
        months: 39.80263158,
    },
    {
        patientId: '425',
        uniquePatientKey: '425',
        studyId: '1',
        status: true,
        entryMonths: 6.677631579,
        months: 39.83552632,
    },
    {
        patientId: '426',
        uniquePatientKey: '426',
        studyId: '1',
        status: false,
        entryMonths: 3.026315789,
        months: 39.86842105,
    },
    {
        patientId: '427',
        uniquePatientKey: '427',
        studyId: '1',
        status: false,
        entryMonths: 18.25657895,
        months: 39.93421053,
    },
    {
        patientId: '428',
        uniquePatientKey: '428',
        studyId: '1',
        status: false,
        entryMonths: 2.138157895,
        months: 39.96710526,
    },
    {
        patientId: '429',
        uniquePatientKey: '429',
        studyId: '1',
        status: true,
        entryMonths: 6.940789474,
        months: 40,
    },
    {
        patientId: '430',
        uniquePatientKey: '430',
        studyId: '1',
        status: false,
        entryMonths: 9.407894737,
        months: 40.03289474,
    },
    {
        patientId: '431',
        uniquePatientKey: '431',
        studyId: '1',
        status: false,
        entryMonths: 2.960526316,
        months: 40.13157895,
    },
    {
        patientId: '432',
        uniquePatientKey: '432',
        studyId: '1',
        status: true,
        entryMonths: 11.05263158,
        months: 40.19736842,
    },
    {
        patientId: '433',
        uniquePatientKey: '433',
        studyId: '1',
        status: true,
        entryMonths: 5.032894737,
        months: 40.23026316,
    },
    {
        patientId: '434',
        uniquePatientKey: '434',
        studyId: '1',
        status: true,
        entryMonths: 0.953947368,
        months: 40.26315789,
    },
    {
        patientId: '435',
        uniquePatientKey: '435',
        studyId: '1',
        status: true,
        entryMonths: 1.282894737,
        months: 40.32894737,
    },
    {
        patientId: '437',
        uniquePatientKey: '437',
        studyId: '1',
        status: true,
        entryMonths: 31.64473684,
        months: 40.46052632,
    },
    {
        patientId: '436',
        uniquePatientKey: '436',
        studyId: '1',
        status: false,
        entryMonths: 8.355263158,
        months: 40.46052632,
    },
    {
        patientId: '439',
        uniquePatientKey: '439',
        studyId: '1',
        status: true,
        entryMonths: 13.61842105,
        months: 40.52631579,
    },
    {
        patientId: '438',
        uniquePatientKey: '438',
        studyId: '1',
        status: false,
        entryMonths: 2.302631579,
        months: 40.52631579,
    },
    {
        patientId: '440',
        uniquePatientKey: '440',
        studyId: '1',
        status: true,
        entryMonths: 37.17105263,
        months: 40.55921053,
    },
    {
        patientId: '441',
        uniquePatientKey: '441',
        studyId: '1',
        status: false,
        entryMonths: 1.710526316,
        months: 40.59210526,
    },
    {
        patientId: '442',
        uniquePatientKey: '442',
        studyId: '1',
        status: false,
        entryMonths: 5.493421053,
        months: 40.59210526,
    },
    {
        patientId: '443',
        uniquePatientKey: '443',
        studyId: '1',
        status: true,
        entryMonths: 11.41447368,
        months: 40.625,
    },
    {
        patientId: '444',
        uniquePatientKey: '444',
        studyId: '1',
        status: false,
        entryMonths: 37.17105263,
        months: 40.65789474,
    },
    {
        patientId: '445',
        uniquePatientKey: '445',
        studyId: '1',
        status: false,
        entryMonths: 13.91447368,
        months: 40.85526316,
    },
    {
        patientId: '446',
        uniquePatientKey: '446',
        studyId: '1',
        status: false,
        entryMonths: 3.289473684,
        months: 40.98684211,
    },
    {
        patientId: '447',
        uniquePatientKey: '447',
        studyId: '1',
        status: true,
        entryMonths: 20.88815789,
        months: 41.15131579,
    },
    {
        patientId: '448',
        uniquePatientKey: '448',
        studyId: '1',
        status: false,
        entryMonths: 3.125,
        months: 41.34868421,
    },
    {
        patientId: '449',
        uniquePatientKey: '449',
        studyId: '1',
        status: true,
        entryMonths: 33.38815789,
        months: 41.71052632,
    },
    {
        patientId: '450',
        uniquePatientKey: '450',
        studyId: '1',
        status: true,
        entryMonths: 1.118421053,
        months: 41.80921053,
    },
    {
        patientId: '452',
        uniquePatientKey: '452',
        studyId: '1',
        status: true,
        entryMonths: 36.11842105,
        months: 41.80921053,
    },
    {
        patientId: '451',
        uniquePatientKey: '451',
        studyId: '1',
        status: false,
        entryMonths: 3.717105263,
        months: 41.80921053,
    },
    {
        patientId: '453',
        uniquePatientKey: '453',
        studyId: '1',
        status: false,
        entryMonths: 10.85526316,
        months: 41.875,
    },
    {
        patientId: '455',
        uniquePatientKey: '455',
        studyId: '1',
        status: true,
        entryMonths: 17.82894737,
        months: 41.90789474,
    },
    {
        patientId: '454',
        uniquePatientKey: '454',
        studyId: '1',
        status: false,
        entryMonths: 14.21052632,
        months: 41.90789474,
    },
    {
        patientId: '456',
        uniquePatientKey: '456',
        studyId: '1',
        status: false,
        entryMonths: 41.84210526,
        months: 41.90789474,
    },
    {
        patientId: '457',
        uniquePatientKey: '457',
        studyId: '1',
        status: true,
        entryMonths: 2.861842105,
        months: 41.94078947,
    },
    {
        patientId: '459',
        uniquePatientKey: '459',
        studyId: '1',
        status: true,
        entryMonths: 2.960526316,
        months: 42.00657895,
    },
    {
        patientId: '458',
        uniquePatientKey: '458',
        studyId: '1',
        status: false,
        entryMonths: 2.434210526,
        months: 42.00657895,
    },
    {
        patientId: '460',
        uniquePatientKey: '460',
        studyId: '1',
        status: false,
        entryMonths: 1.184210526,
        months: 42.10526316,
    },
    {
        patientId: '461',
        uniquePatientKey: '461',
        studyId: '1',
        status: false,
        entryMonths: 5.197368421,
        months: 42.10526316,
    },
    {
        patientId: '462',
        uniquePatientKey: '462',
        studyId: '1',
        status: false,
        entryMonths: 2.072368421,
        months: 42.13815789,
    },
    {
        patientId: '463',
        uniquePatientKey: '463',
        studyId: '1',
        status: true,
        entryMonths: 6.019736842,
        months: 42.43421053,
    },
    {
        patientId: '464',
        uniquePatientKey: '464',
        studyId: '1',
        status: true,
        entryMonths: 27.10526316,
        months: 42.5,
    },
    {
        patientId: '465',
        uniquePatientKey: '465',
        studyId: '1',
        status: false,
        entryMonths: 4.440789474,
        months: 42.53289474,
    },
    {
        patientId: '466',
        uniquePatientKey: '466',
        studyId: '1',
        status: false,
        entryMonths: 0.296052632,
        months: 42.63157895,
    },
    {
        patientId: '467',
        uniquePatientKey: '467',
        studyId: '1',
        status: false,
        entryMonths: 8.157894737,
        months: 43.05921053,
    },
    {
        patientId: '468',
        uniquePatientKey: '468',
        studyId: '1',
        status: false,
        entryMonths: 2.598684211,
        months: 43.19078947,
    },
    {
        patientId: '470',
        uniquePatientKey: '470',
        studyId: '1',
        status: true,
        entryMonths: 32.76315789,
        months: 43.22368421,
    },
    {
        patientId: '469',
        uniquePatientKey: '469',
        studyId: '1',
        status: false,
        entryMonths: 24.86842105,
        months: 43.22368421,
    },
    {
        patientId: '471',
        uniquePatientKey: '471',
        studyId: '1',
        status: false,
        entryMonths: 1.940789474,
        months: 43.28947368,
    },
    {
        patientId: '472',
        uniquePatientKey: '472',
        studyId: '1',
        status: true,
        entryMonths: 42.46710526,
        months: 43.32236842,
    },
    {
        patientId: '473',
        uniquePatientKey: '473',
        studyId: '1',
        status: false,
        entryMonths: 7.631578947,
        months: 43.45394737,
    },
    {
        patientId: '475',
        uniquePatientKey: '475',
        studyId: '1',
        status: true,
        entryMonths: 17.92763158,
        months: 43.55263158,
    },
    {
        patientId: '474',
        uniquePatientKey: '474',
        studyId: '1',
        status: false,
        entryMonths: 10.59210526,
        months: 43.55263158,
    },
    {
        patientId: '476',
        uniquePatientKey: '476',
        studyId: '1',
        status: false,
        entryMonths: 7.302631579,
        months: 43.65131579,
    },
    {
        patientId: '477',
        uniquePatientKey: '477',
        studyId: '1',
        status: false,
        entryMonths: 2.072368421,
        months: 43.75,
    },
    {
        patientId: '478',
        uniquePatientKey: '478',
        studyId: '1',
        status: false,
        entryMonths: 4.276315789,
        months: 43.78289474,
    },
    {
        patientId: '479',
        uniquePatientKey: '479',
        studyId: '1',
        status: true,
        entryMonths: 27.33552632,
        months: 43.94736842,
    },
    {
        patientId: '480',
        uniquePatientKey: '480',
        studyId: '1',
        status: true,
        entryMonths: 17.00657895,
        months: 43.98026316,
    },
    {
        patientId: '481',
        uniquePatientKey: '481',
        studyId: '1',
        status: true,
        entryMonths: 2.138157895,
        months: 44.17763158,
    },
    {
        patientId: '482',
        uniquePatientKey: '482',
        studyId: '1',
        status: true,
        entryMonths: 2.269736842,
        months: 44.27631579,
    },
    {
        patientId: '483',
        uniquePatientKey: '483',
        studyId: '1',
        status: false,
        entryMonths: 6.907894737,
        months: 44.60526316,
    },
    {
        patientId: '484',
        uniquePatientKey: '484',
        studyId: '1',
        status: false,
        entryMonths: 12.00657895,
        months: 44.90131579,
    },
    {
        patientId: '485',
        uniquePatientKey: '485',
        studyId: '1',
        status: true,
        entryMonths: 28.32236842,
        months: 45,
    },
    {
        patientId: '486',
        uniquePatientKey: '486',
        studyId: '1',
        status: true,
        entryMonths: 23.02631579,
        months: 45.03289474,
    },
    {
        patientId: '487',
        uniquePatientKey: '487',
        studyId: '1',
        status: true,
        entryMonths: 32.00657895,
        months: 45.46052632,
    },
    {
        patientId: '488',
        uniquePatientKey: '488',
        studyId: '1',
        status: true,
        entryMonths: 8.980263158,
        months: 45.49342105,
    },
    {
        patientId: '489',
        uniquePatientKey: '489',
        studyId: '1',
        status: true,
        entryMonths: 4.605263158,
        months: 45.625,
    },
    {
        patientId: '490',
        uniquePatientKey: '490',
        studyId: '1',
        status: false,
        entryMonths: 1.513157895,
        months: 45.72368421,
    },
    {
        patientId: '491',
        uniquePatientKey: '491',
        studyId: '1',
        status: true,
        entryMonths: 32.20394737,
        months: 45.75657895,
    },
    {
        patientId: '492',
        uniquePatientKey: '492',
        studyId: '1',
        status: false,
        entryMonths: 17.99342105,
        months: 45.82236842,
    },
    {
        patientId: '493',
        uniquePatientKey: '493',
        studyId: '1',
        status: true,
        entryMonths: 2.631578947,
        months: 46.05263158,
    },
    {
        patientId: '494',
        uniquePatientKey: '494',
        studyId: '1',
        status: true,
        entryMonths: 7.105263158,
        months: 46.15131579,
    },
    {
        patientId: '495',
        uniquePatientKey: '495',
        studyId: '1',
        status: false,
        entryMonths: 3.552631579,
        months: 46.38157895,
    },
    {
        patientId: '496',
        uniquePatientKey: '496',
        studyId: '1',
        status: false,
        entryMonths: 0.986842105,
        months: 46.44736842,
    },
    {
        patientId: '497',
        uniquePatientKey: '497',
        studyId: '1',
        status: false,
        entryMonths: 2.796052632,
        months: 46.80921053,
    },
    {
        patientId: '498',
        uniquePatientKey: '498',
        studyId: '1',
        status: false,
        entryMonths: 4.671052632,
        months: 47.23684211,
    },
    {
        patientId: '499',
        uniquePatientKey: '499',
        studyId: '1',
        status: true,
        entryMonths: 21.15131579,
        months: 47.43421053,
    },
    {
        patientId: '500',
        uniquePatientKey: '500',
        studyId: '1',
        status: true,
        entryMonths: 1.743421053,
        months: 47.63157895,
    },
    {
        patientId: '501',
        uniquePatientKey: '501',
        studyId: '1',
        status: true,
        entryMonths: 1.842105263,
        months: 47.63157895,
    },
    {
        patientId: '502',
        uniquePatientKey: '502',
        studyId: '1',
        status: false,
        entryMonths: 9.078947368,
        months: 47.79605263,
    },
    {
        patientId: '503',
        uniquePatientKey: '503',
        studyId: '1',
        status: false,
        entryMonths: 11.77631579,
        months: 48.09210526,
    },
    {
        patientId: '504',
        uniquePatientKey: '504',
        studyId: '1',
        status: true,
        entryMonths: 25.29605263,
        months: 48.19078947,
    },
    {
        patientId: '505',
        uniquePatientKey: '505',
        studyId: '1',
        status: false,
        entryMonths: 3.059210526,
        months: 48.32236842,
    },
    {
        patientId: '506',
        uniquePatientKey: '506',
        studyId: '1',
        status: false,
        entryMonths: 1.282894737,
        months: 48.48684211,
    },
    {
        patientId: '507',
        uniquePatientKey: '507',
        studyId: '1',
        status: true,
        entryMonths: 24.14473684,
        months: 48.51973684,
    },
    {
        patientId: '508',
        uniquePatientKey: '508',
        studyId: '1',
        status: false,
        entryMonths: 10.09868421,
        months: 48.75,
    },
    {
        patientId: '509',
        uniquePatientKey: '509',
        studyId: '1',
        status: true,
        entryMonths: 27.82894737,
        months: 48.81578947,
    },
    {
        patientId: '510',
        uniquePatientKey: '510',
        studyId: '1',
        status: false,
        entryMonths: 7.138157895,
        months: 49.04605263,
    },
    {
        patientId: '511',
        uniquePatientKey: '511',
        studyId: '1',
        status: false,
        entryMonths: 1.677631579,
        months: 49.07894737,
    },
    {
        patientId: '512',
        uniquePatientKey: '512',
        studyId: '1',
        status: true,
        entryMonths: 35.69078947,
        months: 49.11184211,
    },
    {
        patientId: '513',
        uniquePatientKey: '513',
        studyId: '1',
        status: true,
        entryMonths: 14.83552632,
        months: 49.63815789,
    },
    {
        patientId: '514',
        uniquePatientKey: '514',
        studyId: '1',
        status: false,
        entryMonths: 3.881578947,
        months: 50.09868421,
    },
    {
        patientId: '516',
        uniquePatientKey: '516',
        studyId: '1',
        status: true,
        entryMonths: 46.71052632,
        months: 50.23026316,
    },
    {
        patientId: '515',
        uniquePatientKey: '515',
        studyId: '1',
        status: false,
        entryMonths: 8.125,
        months: 50.23026316,
    },
    {
        patientId: '517',
        uniquePatientKey: '517',
        studyId: '1',
        status: false,
        entryMonths: 2.796052632,
        months: 50.36184211,
    },
    {
        patientId: '518',
        uniquePatientKey: '518',
        studyId: '1',
        status: false,
        entryMonths: 2.5,
        months: 50.39473684,
    },
    {
        patientId: '519',
        uniquePatientKey: '519',
        studyId: '1',
        status: true,
        entryMonths: 35.16447368,
        months: 50.49342105,
    },
    {
        patientId: '520',
        uniquePatientKey: '520',
        studyId: '1',
        status: false,
        entryMonths: 19.93421053,
        months: 50.55921053,
    },
    {
        patientId: '521',
        uniquePatientKey: '521',
        studyId: '1',
        status: false,
        entryMonths: 13.42105263,
        months: 50.72368421,
    },
    {
        patientId: '522',
        uniquePatientKey: '522',
        studyId: '1',
        status: false,
        entryMonths: 6.25,
        months: 50.92105263,
    },
    {
        patientId: '523',
        uniquePatientKey: '523',
        studyId: '1',
        status: true,
        entryMonths: 19.80263158,
        months: 50.98684211,
    },
    {
        patientId: '524',
        uniquePatientKey: '524',
        studyId: '1',
        status: false,
        entryMonths: 1.875,
        months: 51.38157895,
    },
    {
        patientId: '525',
        uniquePatientKey: '525',
        studyId: '1',
        status: true,
        entryMonths: 22.96052632,
        months: 51.51315789,
    },
    {
        patientId: '526',
        uniquePatientKey: '526',
        studyId: '1',
        status: false,
        entryMonths: 39.80263158,
        months: 51.51315789,
    },
    {
        patientId: '527',
        uniquePatientKey: '527',
        studyId: '1',
        status: false,
        entryMonths: 15.82236842,
        months: 51.54605263,
    },
    {
        patientId: '528',
        uniquePatientKey: '528',
        studyId: '1',
        status: true,
        entryMonths: 34.14473684,
        months: 51.77631579,
    },
    {
        patientId: '529',
        uniquePatientKey: '529',
        studyId: '1',
        status: false,
        entryMonths: 5.427631579,
        months: 51.875,
    },
    {
        patientId: '530',
        uniquePatientKey: '530',
        studyId: '1',
        status: false,
        entryMonths: 2.006578947,
        months: 52.23684211,
    },
    {
        patientId: '531',
        uniquePatientKey: '531',
        studyId: '1',
        status: true,
        entryMonths: 2.039473684,
        months: 52.33552632,
    },
    {
        patientId: '532',
        uniquePatientKey: '532',
        studyId: '1',
        status: false,
        entryMonths: 2.467105263,
        months: 52.40131579,
    },
    {
        patientId: '533',
        uniquePatientKey: '533',
        studyId: '1',
        status: false,
        entryMonths: 2.664473684,
        months: 52.99342105,
    },
    {
        patientId: '534',
        uniquePatientKey: '534',
        studyId: '1',
        status: false,
        entryMonths: 16.34868421,
        months: 53.02631579,
    },
    {
        patientId: '535',
        uniquePatientKey: '535',
        studyId: '1',
        status: true,
        entryMonths: 25.06578947,
        months: 53.09210526,
    },
    {
        patientId: '536',
        uniquePatientKey: '536',
        studyId: '1',
        status: true,
        entryMonths: 52.40131579,
        months: 53.125,
    },
    {
        patientId: '537',
        uniquePatientKey: '537',
        studyId: '1',
        status: false,
        entryMonths: 15.29605263,
        months: 53.32236842,
    },
    {
        patientId: '538',
        uniquePatientKey: '538',
        studyId: '1',
        status: false,
        entryMonths: 3.059210526,
        months: 53.61842105,
    },
    {
        patientId: '539',
        uniquePatientKey: '539',
        studyId: '1',
        status: false,
        entryMonths: 8.684210526,
        months: 53.78289474,
    },
    {
        patientId: '540',
        uniquePatientKey: '540',
        studyId: '1',
        status: false,
        entryMonths: 7.302631579,
        months: 53.84868421,
    },
    {
        patientId: '541',
        uniquePatientKey: '541',
        studyId: '1',
        status: false,
        entryMonths: 8.519736842,
        months: 53.91447368,
    },
    {
        patientId: '542',
        uniquePatientKey: '542',
        studyId: '1',
        status: false,
        entryMonths: 10.06578947,
        months: 54.40789474,
    },
    {
        patientId: '543',
        uniquePatientKey: '543',
        studyId: '1',
        status: true,
        entryMonths: 13.19078947,
        months: 54.47368421,
    },
    {
        patientId: '544',
        uniquePatientKey: '544',
        studyId: '1',
        status: false,
        entryMonths: 17.20394737,
        months: 54.93421053,
    },
    {
        patientId: '545',
        uniquePatientKey: '545',
        studyId: '1',
        status: false,
        entryMonths: 1.842105263,
        months: 55.06578947,
    },
    {
        patientId: '546',
        uniquePatientKey: '546',
        studyId: '1',
        status: false,
        entryMonths: 2.236842105,
        months: 55.06578947,
    },
    {
        patientId: '547',
        uniquePatientKey: '547',
        studyId: '1',
        status: true,
        entryMonths: 44.34210526,
        months: 55.42763158,
    },
    {
        patientId: '549',
        uniquePatientKey: '549',
        studyId: '1',
        status: true,
        entryMonths: 37.26973684,
        months: 55.82236842,
    },
    {
        patientId: '548',
        uniquePatientKey: '548',
        studyId: '1',
        status: false,
        entryMonths: 3.848684211,
        months: 55.82236842,
    },
    {
        patientId: '550',
        uniquePatientKey: '550',
        studyId: '1',
        status: false,
        entryMonths: 8.125,
        months: 55.88815789,
    },
    {
        patientId: '551',
        uniquePatientKey: '551',
        studyId: '1',
        status: true,
        entryMonths: 32.66447368,
        months: 56.05263158,
    },
    {
        patientId: '552',
        uniquePatientKey: '552',
        studyId: '1',
        status: false,
        entryMonths: 2.138157895,
        months: 57.30263158,
    },
    {
        patientId: '553',
        uniquePatientKey: '553',
        studyId: '1',
        status: false,
        entryMonths: 5.164473684,
        months: 57.56578947,
    },
    {
        patientId: '554',
        uniquePatientKey: '554',
        studyId: '1',
        status: true,
        entryMonths: 50.09868421,
        months: 57.82894737,
    },
    {
        patientId: '555',
        uniquePatientKey: '555',
        studyId: '1',
        status: false,
        entryMonths: 4.539473684,
        months: 57.86184211,
    },
    {
        patientId: '556',
        uniquePatientKey: '556',
        studyId: '1',
        status: true,
        entryMonths: 18.02631579,
        months: 58.61842105,
    },
    {
        patientId: '557',
        uniquePatientKey: '557',
        studyId: '1',
        status: true,
        entryMonths: 49.34210526,
        months: 58.75,
    },
    {
        patientId: '558',
        uniquePatientKey: '558',
        studyId: '1',
        status: false,
        entryMonths: 56.90789474,
        months: 58.75,
    },
    {
        patientId: '559',
        uniquePatientKey: '559',
        studyId: '1',
        status: true,
        entryMonths: 26.51315789,
        months: 58.88157895,
    },
    {
        patientId: '560',
        uniquePatientKey: '560',
        studyId: '1',
        status: true,
        entryMonths: 24.93421053,
        months: 58.94736842,
    },
    {
        patientId: '561',
        uniquePatientKey: '561',
        studyId: '1',
        status: true,
        entryMonths: 55.26315789,
        months: 58.94736842,
    },
    {
        patientId: '562',
        uniquePatientKey: '562',
        studyId: '1',
        status: true,
        entryMonths: 55.52631579,
        months: 59.11184211,
    },
    {
        patientId: '563',
        uniquePatientKey: '563',
        studyId: '1',
        status: false,
        entryMonths: 23.68421053,
        months: 59.34210526,
    },
    {
        patientId: '564',
        uniquePatientKey: '564',
        studyId: '1',
        status: false,
        entryMonths: 32.63157895,
        months: 59.34210526,
    },
    {
        patientId: '565',
        uniquePatientKey: '565',
        studyId: '1',
        status: true,
        entryMonths: 36.31578947,
        months: 60.39473684,
    },
    {
        patientId: '566',
        uniquePatientKey: '566',
        studyId: '1',
        status: false,
        entryMonths: 12.10526316,
        months: 60.78947368,
    },
    {
        patientId: '567',
        uniquePatientKey: '567',
        studyId: '1',
        status: false,
        entryMonths: 1.776315789,
        months: 60.95394737,
    },
    {
        patientId: '568',
        uniquePatientKey: '568',
        studyId: '1',
        status: true,
        entryMonths: 55.55921053,
        months: 60.98684211,
    },
    {
        patientId: '569',
        uniquePatientKey: '569',
        studyId: '1',
        status: false,
        entryMonths: 1.381578947,
        months: 61.08552632,
    },
    {
        patientId: '570',
        uniquePatientKey: '570',
        studyId: '1',
        status: false,
        entryMonths: 1.644736842,
        months: 61.875,
    },
    {
        patientId: '571',
        uniquePatientKey: '571',
        studyId: '1',
        status: false,
        entryMonths: 22.59868421,
        months: 62.20394737,
    },
    {
        patientId: '572',
        uniquePatientKey: '572',
        studyId: '1',
        status: false,
        entryMonths: 6.677631579,
        months: 62.40131579,
    },
    {
        patientId: '573',
        uniquePatientKey: '573',
        studyId: '1',
        status: false,
        entryMonths: 20.26315789,
        months: 62.5,
    },
    {
        patientId: '574',
        uniquePatientKey: '574',
        studyId: '1',
        status: false,
        entryMonths: 7.598684211,
        months: 63.45394737,
    },
    {
        patientId: '575',
        uniquePatientKey: '575',
        studyId: '1',
        status: true,
        entryMonths: 46.18421053,
        months: 63.55263158,
    },
    {
        patientId: '576',
        uniquePatientKey: '576',
        studyId: '1',
        status: true,
        entryMonths: 48.09210526,
        months: 63.55263158,
    },
    {
        patientId: '577',
        uniquePatientKey: '577',
        studyId: '1',
        status: true,
        entryMonths: 31.57894737,
        months: 63.78289474,
    },
    {
        patientId: '578',
        uniquePatientKey: '578',
        studyId: '1',
        status: false,
        entryMonths: 8.552631579,
        months: 63.84868421,
    },
    {
        patientId: '579',
        uniquePatientKey: '579',
        studyId: '1',
        status: true,
        entryMonths: 21.01973684,
        months: 63.91447368,
    },
    {
        patientId: '580',
        uniquePatientKey: '580',
        studyId: '1',
        status: false,
        entryMonths: 2.960526316,
        months: 64.07894737,
    },
    {
        patientId: '581',
        uniquePatientKey: '581',
        studyId: '1',
        status: false,
        entryMonths: 26.97368421,
        months: 64.07894737,
    },
    {
        patientId: '582',
        uniquePatientKey: '582',
        studyId: '1',
        status: true,
        entryMonths: 55.26315789,
        months: 64.14473684,
    },
    {
        patientId: '583',
        uniquePatientKey: '583',
        studyId: '1',
        status: true,
        entryMonths: 60.26315789,
        months: 64.17763158,
    },
    {
        patientId: '584',
        uniquePatientKey: '584',
        studyId: '1',
        status: false,
        entryMonths: 12.53289474,
        months: 64.27631579,
    },
    {
        patientId: '585',
        uniquePatientKey: '585',
        studyId: '1',
        status: true,
        entryMonths: 61.71052632,
        months: 64.34210526,
    },
    {
        patientId: '586',
        uniquePatientKey: '586',
        studyId: '1',
        status: false,
        entryMonths: 9.868421053,
        months: 64.40789474,
    },
    {
        patientId: '587',
        uniquePatientKey: '587',
        studyId: '1',
        status: true,
        entryMonths: 55.03289474,
        months: 64.73684211,
    },
    {
        patientId: '588',
        uniquePatientKey: '588',
        studyId: '1',
        status: true,
        entryMonths: 63.38815789,
        months: 65,
    },
    {
        patientId: '589',
        uniquePatientKey: '589',
        studyId: '1',
        status: false,
        entryMonths: 5.559210526,
        months: 65.23026316,
    },
    {
        patientId: '590',
        uniquePatientKey: '590',
        studyId: '1',
        status: false,
        entryMonths: 13.58552632,
        months: 65.36184211,
    },
    {
        patientId: '591',
        uniquePatientKey: '591',
        studyId: '1',
        status: true,
        entryMonths: 39.67105263,
        months: 65.75657895,
    },
    {
        patientId: '592',
        uniquePatientKey: '592',
        studyId: '1',
        status: false,
        entryMonths: 63.09210526,
        months: 65.88815789,
    },
    {
        patientId: '593',
        uniquePatientKey: '593',
        studyId: '1',
        status: true,
        entryMonths: 12.99342105,
        months: 66.41447368,
    },
    {
        patientId: '594',
        uniquePatientKey: '594',
        studyId: '1',
        status: false,
        entryMonths: 25.52631579,
        months: 66.54605263,
    },
    {
        patientId: '595',
        uniquePatientKey: '595',
        studyId: '1',
        status: false,
        entryMonths: 42.20394737,
        months: 66.64473684,
    },
    {
        patientId: '596',
        uniquePatientKey: '596',
        studyId: '1',
        status: false,
        entryMonths: 21.67763158,
        months: 66.67763158,
    },
    {
        patientId: '597',
        uniquePatientKey: '597',
        studyId: '1',
        status: false,
        entryMonths: 35.19736842,
        months: 66.67763158,
    },
    {
        patientId: '598',
        uniquePatientKey: '598',
        studyId: '1',
        status: false,
        entryMonths: 11.28289474,
        months: 67.36842105,
    },
    {
        patientId: '599',
        uniquePatientKey: '599',
        studyId: '1',
        status: false,
        entryMonths: 15.26315789,
        months: 67.89473684,
    },
    {
        patientId: '600',
        uniquePatientKey: '600',
        studyId: '1',
        status: false,
        entryMonths: 19.90131579,
        months: 68.05921053,
    },
    {
        patientId: '601',
        uniquePatientKey: '601',
        studyId: '1',
        status: true,
        entryMonths: 43.88157895,
        months: 68.09210526,
    },
    {
        patientId: '602',
        uniquePatientKey: '602',
        studyId: '1',
        status: false,
        entryMonths: 16.77631579,
        months: 68.88157895,
    },
    {
        patientId: '603',
        uniquePatientKey: '603',
        studyId: '1',
        status: false,
        entryMonths: 31.77631579,
        months: 69.07894737,
    },
    {
        patientId: '604',
        uniquePatientKey: '604',
        studyId: '1',
        status: true,
        entryMonths: 54.24342105,
        months: 69.90131579,
    },
    {
        patientId: '605',
        uniquePatientKey: '605',
        studyId: '1',
        status: false,
        entryMonths: 9.440789474,
        months: 70.13157895,
    },
    {
        patientId: '606',
        uniquePatientKey: '606',
        studyId: '1',
        status: true,
        entryMonths: 64.93421053,
        months: 70.39473684,
    },
    {
        patientId: '607',
        uniquePatientKey: '607',
        studyId: '1',
        status: false,
        entryMonths: 43.32236842,
        months: 71.61184211,
    },
    {
        patientId: '608',
        uniquePatientKey: '608',
        studyId: '1',
        status: true,
        entryMonths: 31.77631579,
        months: 71.77631579,
    },
    {
        patientId: '609',
        uniquePatientKey: '609',
        studyId: '1',
        status: true,
        entryMonths: 67.36842105,
        months: 72.00657895,
    },
    {
        patientId: '610',
        uniquePatientKey: '610',
        studyId: '1',
        status: true,
        entryMonths: 55.59210526,
        months: 72.82894737,
    },
    {
        patientId: '611',
        uniquePatientKey: '611',
        studyId: '1',
        status: false,
        entryMonths: 42.13815789,
        months: 72.86184211,
    },
    {
        patientId: '612',
        uniquePatientKey: '612',
        studyId: '1',
        status: true,
        entryMonths: 47.07236842,
        months: 72.89473684,
    },
    {
        patientId: '613',
        uniquePatientKey: '613',
        studyId: '1',
        status: true,
        entryMonths: 63.09210526,
        months: 73.84868421,
    },
    {
        patientId: '614',
        uniquePatientKey: '614',
        studyId: '1',
        status: false,
        entryMonths: 73.84868421,
        months: 74.40789474,
    },
    {
        patientId: '615',
        uniquePatientKey: '615',
        studyId: '1',
        status: true,
        entryMonths: 54.11184211,
        months: 75.16447368,
    },
    {
        patientId: '616',
        uniquePatientKey: '616',
        studyId: '1',
        status: true,
        entryMonths: 53.02631579,
        months: 75.26315789,
    },
    {
        patientId: '617',
        uniquePatientKey: '617',
        studyId: '1',
        status: true,
        entryMonths: 54.27631579,
        months: 75.26315789,
    },
    {
        patientId: '618',
        uniquePatientKey: '618',
        studyId: '1',
        status: false,
        entryMonths: 46.54605263,
        months: 75.95394737,
    },
    {
        patientId: '619',
        uniquePatientKey: '619',
        studyId: '1',
        status: false,
        entryMonths: 18.61842105,
        months: 76.21710526,
    },
    {
        patientId: '620',
        uniquePatientKey: '620',
        studyId: '1',
        status: false,
        entryMonths: 40.625,
        months: 77.07236842,
    },
    {
        patientId: '621',
        uniquePatientKey: '621',
        studyId: '1',
        status: false,
        entryMonths: 22.00657895,
        months: 77.96052632,
    },
    {
        patientId: '622',
        uniquePatientKey: '622',
        studyId: '1',
        status: false,
        entryMonths: 27.96052632,
        months: 77.96052632,
    },
    {
        patientId: '623',
        uniquePatientKey: '623',
        studyId: '1',
        status: true,
        entryMonths: 55.03289474,
        months: 78.58552632,
    },
    {
        patientId: '624',
        uniquePatientKey: '624',
        studyId: '1',
        status: true,
        entryMonths: 65.32894737,
        months: 78.81578947,
    },
    {
        patientId: '625',
        uniquePatientKey: '625',
        studyId: '1',
        status: true,
        entryMonths: 71.64473684,
        months: 79.93421053,
    },
    {
        patientId: '626',
        uniquePatientKey: '626',
        studyId: '1',
        status: true,
        entryMonths: 66.51315789,
        months: 80.69078947,
    },
    {
        patientId: '627',
        uniquePatientKey: '627',
        studyId: '1',
        status: false,
        entryMonths: 18.84868421,
        months: 81.05263158,
    },
    {
        patientId: '628',
        uniquePatientKey: '628',
        studyId: '1',
        status: true,
        entryMonths: 65.19736842,
        months: 82.92763158,
    },
    {
        patientId: '629',
        uniquePatientKey: '629',
        studyId: '1',
        status: true,
        entryMonths: 45.39473684,
        months: 84.27631579,
    },
    {
        patientId: '630',
        uniquePatientKey: '630',
        studyId: '1',
        status: true,
        entryMonths: 47.99342105,
        months: 85.55921053,
    },
    {
        patientId: '631',
        uniquePatientKey: '631',
        studyId: '1',
        status: false,
        entryMonths: 29.96710526,
        months: 86.48026316,
    },
    {
        patientId: '632',
        uniquePatientKey: '632',
        studyId: '1',
        status: true,
        entryMonths: 59.21052632,
        months: 86.67763158,
    },
    {
        patientId: '633',
        uniquePatientKey: '633',
        studyId: '1',
        status: false,
        entryMonths: 43.98026316,
        months: 88.19078947,
    },
    {
        patientId: '634',
        uniquePatientKey: '634',
        studyId: '1',
        status: true,
        entryMonths: 56.28289474,
        months: 89.30921053,
    },
    {
        patientId: '635',
        uniquePatientKey: '635',
        studyId: '1',
        status: true,
        entryMonths: 73.88157895,
        months: 90.42763158,
    },
    {
        patientId: '636',
        uniquePatientKey: '636',
        studyId: '1',
        status: true,
        entryMonths: 73.68421053,
        months: 92.86184211,
    },
    {
        patientId: '637',
        uniquePatientKey: '637',
        studyId: '1',
        status: true,
        entryMonths: 59.86842105,
        months: 93.38815789,
    },
    {
        patientId: '638',
        uniquePatientKey: '638',
        studyId: '1',
        status: true,
        entryMonths: 82.03947368,
        months: 95.42763158,
    },
    {
        patientId: '639',
        uniquePatientKey: '639',
        studyId: '1',
        status: false,
        entryMonths: 90.625,
        months: 95.52631579,
    },
    {
        patientId: '640',
        uniquePatientKey: '640',
        studyId: '1',
        status: false,
        entryMonths: 42.10526316,
        months: 97.43421053,
    },
    {
        patientId: '641',
        uniquePatientKey: '641',
        studyId: '1',
        status: false,
        entryMonths: 39.17763158,
        months: 98.05921053,
    },
    {
        patientId: '642',
        uniquePatientKey: '642',
        studyId: '1',
        status: true,
        entryMonths: 94.73684211,
        months: 98.48684211,
    },
    {
        patientId: '643',
        uniquePatientKey: '643',
        studyId: '1',
        status: false,
        entryMonths: 40.09868421,
        months: 99.63815789,
    },
    {
        patientId: '644',
        uniquePatientKey: '644',
        studyId: '1',
        status: true,
        entryMonths: 59.34210526,
        months: 107.1710526,
    },
    {
        patientId: '645',
        uniquePatientKey: '645',
        studyId: '1',
        status: true,
        entryMonths: 88.84868421,
        months: 110.0986842,
    },
    {
        patientId: '646',
        uniquePatientKey: '646',
        studyId: '1',
        status: true,
        entryMonths: 83.61842105,
        months: 111.4802632,
    },
    {
        patientId: '647',
        uniquePatientKey: '647',
        studyId: '1',
        status: false,
        entryMonths: 74.86842105,
        months: 115.8881579,
    },
    {
        patientId: '648',
        uniquePatientKey: '648',
        studyId: '1',
        status: false,
        entryMonths: 59.07894737,
        months: 117.3684211,
    },
    {
        patientId: '649',
        uniquePatientKey: '649',
        studyId: '1',
        status: false,
        entryMonths: 73.05921053,
        months: 125.2631579,
    },
    {
        patientId: '650',
        uniquePatientKey: '650',
        studyId: '1',
        status: false,
        entryMonths: 78.09210526,
        months: 125.9868421,
    },
    {
        patientId: '651',
        uniquePatientKey: '651',
        studyId: '1',
        status: true,
        entryMonths: 130.5263158,
        months: 143.5197368,
    },
    {
        patientId: '652',
        uniquePatientKey: '652',
        studyId: '1',
        status: false,
        entryMonths: 131.3815789,
        months: 145.1644737,
    },
    {
        patientId: '653',
        uniquePatientKey: '653',
        studyId: '1',
        status: false,
        entryMonths: 88.68421053,
        months: 151.4144737,
    },
    {
        patientId: '654',
        uniquePatientKey: '654',
        studyId: '1',
        status: true,
        entryMonths: 103.4868421,
        months: 154.1447368,
    },
    {
        patientId: '655',
        uniquePatientKey: '655',
        studyId: '1',
        status: false,
        entryMonths: 114.7039474,
        months: 162.7960526,
    },
    {
        patientId: '656',
        uniquePatientKey: '656',
        studyId: '1',
        status: true,
        entryMonths: 130.4276316,
        months: 169.2434211,
    },
    {
        patientId: '657',
        uniquePatientKey: '657',
        studyId: '1',
        status: false,
        entryMonths: 174.7039474,
        months: 197.1052632,
    },
    {
        patientId: '658',
        uniquePatientKey: '658',
        studyId: '1',
        status: false,
        entryMonths: 159.5394737,
        months: 205.5592105,
    },
    {
        patientId: '659',
        uniquePatientKey: '659',
        studyId: '1',
        status: false,
        entryMonths: 163.8157895,
        months: 215.3289474,
    },
] as PatientSurvival[];

const numberOfRiskFoPatientSurvivalsFromDelayedEntryRepo = [
    27,
    96,
    118,
    182,
    181,
    203,
    208,
    210,
    210,
    221,
    229,
    245,
    244,
    244,
    249,
    248,
    248,
    250,
    253,
    252,
    258,
    281,
    281,
    281,
    289,
    292,
    295,
    300,
    299,
    305,
    304,
    310,
    319,
    318,
    323,
    323,
    325,
    335,
    335,
    334,
    336,
    336,
    339,
    338,
    338,
    338,
    337,
    338,
    337,
    338,
    337,
    337,
    336,
    336,
    337,
    340,
    348,
    347,
    346,
    345,
    347,
    347,
    346,
    345,
    345,
    346,
    348,
    347,
    348,
    348,
    348,
    348,
    347,
    346,
    346,
    347,
    346,
    345,
    344,
    346,
    349,
    350,
    350,
    349,
    348,
    350,
    350,
    349,
    348,
    348,
    347,
    346,
    346,
    345,
    345,
    347,
    347,
    346,
    345,
    346,
    346,
    347,
    347,
    350,
    349,
    348,
    349,
    348,
    347,
    346,
    345,
    350,
    349,
    348,
    347,
    346,
    347,
    348,
    347,
    347,
    346,
    345,
    344,
    343,
    344,
    343,
    342,
    342,
    341,
    340,
    339,
    338,
    338,
    337,
    336,
    335,
    334,
    333,
    332,
    331,
    331,
    330,
    331,
    331,
    332,
    332,
    333,
    332,
    331,
    333,
    333,
    332,
    331,
    332,
    333,
    332,
    331,
    331,
    331,
    332,
    331,
    330,
    329,
    329,
    331,
    330,
    329,
    328,
    329,
    329,
    328,
    328,
    328,
    327,
    327,
    326,
    326,
    325,
    330,
    329,
    328,
    327,
    326,
    326,
    327,
    329,
    330,
    329,
    328,
    328,
    327,
    326,
    325,
    324,
    323,
    325,
    324,
    324,
    323,
    323,
    322,
    322,
    321,
    321,
    321,
    320,
    319,
    319,
    321,
    320,
    319,
    319,
    319,
    319,
    318,
    317,
    316,
    316,
    315,
    314,
    313,
    312,
    311,
    310,
    311,
    310,
    309,
    309,
    308,
    311,
    310,
    309,
    308,
    307,
    307,
    306,
    305,
    304,
    303,
    304,
    304,
    303,
    304,
    303,
    303,
    302,
    301,
    301,
    300,
    299,
    298,
    297,
    296,
    296,
    295,
    294,
    293,
    292,
    291,
    290,
    289,
    288,
    287,
    288,
    289,
    288,
    288,
    287,
    287,
    286,
    285,
    284,
    283,
    282,
    281,
    281,
    281,
    280,
    279,
    280,
    279,
    278,
    277,
    276,
    275,
    274,
    274,
    273,
    272,
    271,
    270,
    269,
    271,
    270,
    270,
    269,
    268,
    267,
    266,
    265,
    264,
    263,
    262,
    261,
    260,
    259,
    258,
    257,
    256,
    255,
    254,
    253,
    252,
    251,
    250,
    249,
    249,
    249,
    248,
    247,
    249,
    248,
    247,
    246,
    245,
    244,
    244,
    243,
    242,
    241,
    241,
    240,
    241,
    240,
    240,
    240,
    240,
    239,
    238,
    237,
    236,
    235,
    234,
    233,
    233,
    232,
    231,
    231,
    230,
    229,
    228,
    228,
    227,
    226,
    225,
    224,
    223,
    222,
    221,
    220,
    219,
    218,
    217,
    216,
    215,
    214,
    214,
    214,
    213,
    212,
    212,
    211,
    210,
    210,
    209,
    208,
    207,
    207,
    206,
    205,
    204,
    203,
    202,
    201,
    200,
    199,
    200,
    199,
    199,
    198,
    197,
    196,
    195,
    194,
    193,
    192,
    191,
    190,
    189,
    188,
    187,
    186,
    185,
    184,
    183,
    182,
    181,
    180,
    179,
    178,
    177,
    176,
    175,
    174,
    173,
    172,
    171,
    171,
    170,
    169,
    168,
    168,
    167,
    166,
    166,
    165,
    164,
    163,
    162,
    161,
    161,
    160,
    159,
    158,
    157,
    156,
    155,
    154,
    153,
    152,
    151,
    150,
    149,
    149,
    148,
    147,
    146,
    145,
    144,
    143,
    142,
    141,
    141,
    140,
    139,
    138,
    137,
    136,
    135,
    134,
    133,
    133,
    134,
    134,
    133,
    132,
    131,
    130,
    129,
    128,
    127,
    126,
    126,
    125,
    124,
    123,
    122,
    121,
    121,
    120,
    120,
    119,
    119,
    118,
    117,
    116,
    116,
    115,
    114,
    113,
    112,
    111,
    110,
    109,
    109,
    108,
    109,
    109,
    108,
    107,
    106,
    105,
    105,
    105,
    104,
    103,
    102,
    101,
    100,
    99,
    98,
    97,
    97,
    96,
    96,
    95,
    94,
    93,
    92,
    91,
    90,
    89,
    88,
    87,
    86,
    85,
    84,
    83,
    82,
    81,
    80,
    79,
    79,
    78,
    78,
    77,
    76,
    75,
    74,
    73,
    72,
    74,
    73,
    72,
    73,
    72,
    73,
    75,
    74,
    73,
    72,
    73,
    72,
    71,
    70,
    69,
    68,
    67,
    66,
    65,
    64,
    64,
    64,
    63,
    65,
    64,
    63,
    62,
    61,
    61,
    60,
    59,
    58,
    60,
    59,
    58,
    57,
    56,
    55,
    54,
    53,
    52,
    51,
    50,
    49,
    48,
    47,
    47,
    47,
    47,
    46,
    45,
    44,
    44,
    43,
    42,
    41,
    40,
    40,
    39,
    38,
    37,
    36,
    35,
    34,
    33,
    32,
    32,
    31,
    30,
    29,
    28,
    29,
    30,
    30,
    29,
    28,
    27,
    26,
    25,
    24,
    23,
    23,
    22,
    21,
    20,
    19,
    19,
    19,
    18,
    17,
    16,
    15,
    16,
    15,
    15,
    14,
    14,
    13,
    12,
    11,
    10,
    9,
    9,
    8,
    7,
    7,
    6,
    5,
    4,
    6,
    5,
    4,
    3,
    3,
    3,
    3,
    2,
    1,
];

const delayedEntrySurvivalSummaries = [
    {
        survivalFunctionEstimate: 0.9629629629629629,
        standardError: 0.036344694923159865,
        low95ConfidenceInterval: 0.8942996316071836,
        high95ConfidenceInterval: 1,
        atRisk: 27,
    },
    {
        survivalFunctionEstimate: 0.9529320987654321,
        standardError: 0.03732466734089861,
        low95ConfidenceInterval: 0.8825145918441618,
        high95ConfidenceInterval: 1,
        atRisk: 96,
    },
    {
        survivalFunctionEstimate: 0.9529320987654321,
        atRisk: 118,
    },
    {
        survivalFunctionEstimate: 0.9476962081128748,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 182,
    },
    {
        survivalFunctionEstimate: 0.9424603174603176,
        standardError: 0.03764182978261099,
        low95ConfidenceInterval: 0.8714974394032606,
        high95ConfidenceInterval: 1,
        atRisk: 181,
    },
    {
        survivalFunctionEstimate: 0.9424603174603176,
        atRisk: 203,
    },
    {
        survivalFunctionEstimate: 0.9379292582417583,
        standardError: 0.037732582522843686,
        low95ConfidenceInterval: 0.8668152200646726,
        high95ConfidenceInterval: 1,
        atRisk: 208,
    },
    {
        survivalFunctionEstimate: 0.9379292582417583,
        atRisk: 210,
    },
    {
        survivalFunctionEstimate: 0.9334629284406071,
        standardError: 0.03781631495269527,
        low95ConfidenceInterval: 0.862210527859478,
        high95ConfidenceInterval: 1,
        atRisk: 210,
    },
    {
        survivalFunctionEstimate: 0.9292391142847672,
        standardError: 0.03788035101291387,
        low95ConfidenceInterval: 0.8578835216522631,
        high95ConfidenceInterval: 1,
        atRisk: 221,
    },
    {
        survivalFunctionEstimate: 0.9251813015586329,
        standardError: 0.03793165215539977,
        low95ConfidenceInterval: 0.8537452537635137,
        high95ConfidenceInterval: 1,
        atRisk: 229,
    },
    {
        survivalFunctionEstimate: 0.9214050513481894,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 245,
    },
    {
        survivalFunctionEstimate: 0.917628801137746,
        standardError: 0.03799608730546883,
        low95ConfidenceInterval: 0.846099600615412,
        high95ConfidenceInterval: 0.9952050752240468,
        atRisk: 244,
    },
    {
        survivalFunctionEstimate: 0.9138680273625914,
        standardError: 0.038026026955684425,
        low95ConfidenceInterval: 0.8422965217941104,
        high95ConfidenceInterval: 0.9915210971745387,
        atRisk: 244,
    },
    {
        survivalFunctionEstimate: 0.9101978746422597,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 249,
    },
    {
        survivalFunctionEstimate: 0.906527721921928,
        standardError: 0.03807318079264483,
        low95ConfidenceInterval: 0.8348944002530576,
        high95ConfidenceInterval: 0.9843071295769549,
        atRisk: 248,
    },
    {
        survivalFunctionEstimate: 0.9028723682045009,
        standardError: 0.038094728512573,
        low95ConfidenceInterval: 0.8312119362445197,
        high95ConfidenceInterval: 0.9807107883342531,
        atRisk: 248,
    },
    {
        survivalFunctionEstimate: 0.8992608787316829,
        standardError: 0.03811315491853098,
        low95ConfidenceInterval: 0.8275789442570616,
        high95ConfidenceInterval: 0.9771516465335425,
        atRisk: 250,
    },
    {
        survivalFunctionEstimate: 0.8957064879066565,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 253,
    },
    {
        survivalFunctionEstimate: 0.8921520970816301,
        standardError: 0.03814190336984624,
        low95ConfidenceInterval: 0.8204417337465904,
        high95ConfidenceInterval: 0.970130274446754,
        atRisk: 252,
    },
    {
        survivalFunctionEstimate: 0.8886941432169726,
        standardError: 0.03815049396949355,
        low95ConfidenceInterval: 0.8169798271118248,
        high95ConfidenceInterval: 0.966703526793505,
        atRisk: 258,
    },
    {
        survivalFunctionEstimate: 0.8855315306076595,
        standardError: 0.03814558951295535,
        low95ConfidenceInterval: 0.8138366715322336,
        high95ConfidenceInterval: 0.9635423410252236,
        atRisk: 281,
    },
    {
        survivalFunctionEstimate: 0.8823801728474899,
        standardError: 0.03813979099464029,
        low95ConfidenceInterval: 0.8107064113747753,
        high95ConfidenceInterval: 0.9603905415205056,
        atRisk: 281,
    },
    {
        survivalFunctionEstimate: 0.8792400298836198,
        standardError: 0.038133110951984274,
        low95ConfidenceInterval: 0.8075889827054565,
        high95ConfidenceInterval: 0.9572481134647918,
        atRisk: 281,
    },
    {
        survivalFunctionEstimate: 0.8761976768390398,
        standardError: 0.038122332481521574,
        low95ConfidenceInterval: 0.804576451309974,
        high95ConfidenceInterval: 0.9541944306823305,
        atRisk: 289,
    },
    {
        survivalFunctionEstimate: 0.8731969998635637,
        standardError: 0.03810968789930282,
        low95ConfidenceInterval: 0.8016088690784048,
        high95ConfidenceInterval: 0.951178348921376,
        atRisk: 292,
    },
    {
        survivalFunctionEstimate: 0.8702370100335177,
        standardError: 0.03809528069291566,
        low95ConfidenceInterval: 0.7986850596737869,
        high95ConfidenceInterval: 0.9481990985800984,
        atRisk: 295,
    },
    {
        survivalFunctionEstimate: 0.8673362200000727,
        standardError: 0.0380785775409857,
        low95ConfidenceInterval: 0.7958244257121352,
        high95ConfidenceInterval: 0.9452739752877674,
        atRisk: 300,
    },
    {
        survivalFunctionEstimate: 0.8644354299666277,
        standardError: 0.03806155378867283,
        low95ConfidenceInterval: 0.7929644179186229,
        high95ConfidenceInterval: 0.9423482260943947,
        atRisk: 299,
    },
    {
        survivalFunctionEstimate: 0.861601215442147,
        standardError: 0.03804213889319538,
        low95ConfidenceInterval: 0.790175098213409,
        high95ConfidenceInterval: 0.9394837373765107,
        atRisk: 305,
    },
    {
        survivalFunctionEstimate: 0.8587670009176662,
        standardError: 0.03802243099135169,
        low95ConfidenceInterval: 0.7873863507598877,
        high95ConfidenceInterval: 0.9366186766552378,
        atRisk: 304,
    },
    {
        survivalFunctionEstimate: 0.8559967847856738,
        standardError: 0.03800055938820291,
        low95ConfidenceInterval: 0.784665306275646,
        high95ConfidenceInterval: 0.9338127857866693,
        atRisk: 310,
    },
    {
        survivalFunctionEstimate: 0.8559967847856738,
        atRisk: 319,
    },
    {
        survivalFunctionEstimate: 0.8533049709970396,
        standardError: 0.037976279979629185,
        low95ConfidenceInterval: 0.782026749643399,
        high95ConfidenceInterval: 0.9310798816796007,
        atRisk: 318,
    },
    {
        survivalFunctionEstimate: 0.8506631599413212,
        standardError: 0.03795048359884288,
        low95ConfidenceInterval: 0.7794407772695371,
        high95ConfidenceInterval: 0.9283935775291333,
        atRisk: 323,
    },
    {
        survivalFunctionEstimate: 0.8480295278671994,
        standardError: 0.037924262214506115,
        low95ConfidenceInterval: 0.776863752392422,
        high95ConfidenceInterval: 0.9257145515155847,
        atRisk: 323,
    },
    {
        survivalFunctionEstimate: 0.8454202062429926,
        standardError: 0.03789723108405741,
        low95ConfidenceInterval: 0.774312437819973,
        high95ConfidenceInterval: 0.9230580450654204,
        atRisk: 325,
    },
    {
        survivalFunctionEstimate: 0.8428965638362971,
        standardError: 0.0378680386604216,
        low95ConfidenceInterval: 0.771850401774576,
        high95ConfidenceInterval: 0.9204822795888571,
        atRisk: 335,
    },
    {
        survivalFunctionEstimate: 0.840380454690517,
        standardError: 0.03783849775827341,
        low95ConfidenceInterval: 0.7693965262167507,
        high95ConfidenceInterval: 0.9179133055078049,
        atRisk: 335,
    },
    {
        survivalFunctionEstimate: 0.8378643455447371,
        standardError: 0.037808771904967904,
        low95ConfidenceInterval: 0.7669430118253654,
        high95ConfidenceInterval: 0.9153439704265294,
        atRisk: 334,
    },
    {
        survivalFunctionEstimate: 0.8353707016591873,
        standardError: 0.03777838933006048,
        low95ConfidenceInterval: 0.7645130869391956,
        high95ConfidenceInterval: 0.9127956356959849,
        atRisk: 336,
    },
    {
        survivalFunctionEstimate: 0.8328844793328206,
        standardError: 0.03774767501321795,
        low95ConfidenceInterval: 0.7620911795120664,
        high95ConfidenceInterval: 0.9102540149561201,
        atRisk: 336,
    },
    {
        survivalFunctionEstimate: 0.8304275929631072,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 339,
    },
    {
        survivalFunctionEstimate: 0.8279707065933939,
        standardError: 0.037684547168124326,
        low95ConfidenceInterval: 0.7573089479858927,
        high95ConfidenceInterval: 0.9052256582996749,
        atRisk: 338,
    },
    {
        survivalFunctionEstimate: 0.8255210891182655,
        standardError: 0.03765258678058559,
        low95ConfidenceInterval: 0.7549258252379543,
        high95ConfidenceInterval: 0.9027179171731232,
        atRisk: 338,
    },
    {
        survivalFunctionEstimate: 0.8230787190321167,
        standardError: 0.03762031850363015,
        low95ConfidenceInterval: 0.7525505012978057,
        high95ConfidenceInterval: 0.9002167649283918,
        atRisk: 338,
    },
    {
        survivalFunctionEstimate: 0.820636348945968,
        standardError: 0.037587883148663785,
        low95ConfidenceInterval: 0.7501755035864365,
        high95ConfidenceInterval: 0.8977152866119591,
        atRisk: 337,
    },
    {
        survivalFunctionEstimate: 0.820636348945968,
        atRisk: 338,
    },
    {
        survivalFunctionEstimate: 0.818201226248799,
        standardError: 0.03755514300846759,
        low95ConfidenceInterval: 0.7478082986116383,
        high95ConfidenceInterval: 0.8952204032476346,
        atRisk: 337,
    },
    {
        survivalFunctionEstimate: 0.8157805125616724,
        standardError: 0.03752196868558198,
        low95ConfidenceInterval: 0.7454562519227395,
        high95ConfidenceInterval: 0.89273896752342,
        atRisk: 338,
    },
    {
        survivalFunctionEstimate: 0.8133597988745458,
        standardError: 0.037488631861084525,
        low95ConfidenceInterval: 0.7431045225121696,
        high95ConfidenceInterval: 0.8902572146765632,
        atRisk: 337,
    },
    {
        survivalFunctionEstimate: 0.8109462683140872,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 337,
    },
    {
        survivalFunctionEstimate: 0.8085327377536287,
        standardError: 0.037421208698951346,
        low95ConfidenceInterval: 0.7384167953813807,
        high95ConfidenceInterval: 0.8853064991320236,
        atRisk: 336,
    },
    {
        survivalFunctionEstimate: 0.8061263903198381,
        standardError: 0.037387125333392966,
        low95ConfidenceInterval: 0.7360807918486372,
        high95ConfidenceInterval: 0.8828375422459344,
        atRisk: 336,
    },
    {
        survivalFunctionEstimate: 0.8037343238797199,
        standardError: 0.03735262934912141,
        low95ConfidenceInterval: 0.7337597779988563,
        high95ConfidenceInterval: 0.8803819489045329,
        atRisk: 337,
    },
    {
        survivalFunctionEstimate: 0.8013703993977207,
        standardError: 0.03731749618879802,
        low95ConfidenceInterval: 0.7314679613374273,
        high95ConfidenceInterval: 0.8779530355050194,
        atRisk: 340,
    },
    {
        survivalFunctionEstimate: 0.8013703993977207,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.8013703993977207,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7990542999774961,
        standardError: 0.037281446886033485,
        low95ConfidenceInterval: 0.729225444800741,
        high95ConfidenceInterval: 0.8755697965078433,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7990542999774961,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.7967515498334687,
        standardError: 0.03724505624508753,
        low95ConfidenceInterval: 0.726996858008345,
        high95ConfidenceInterval: 0.8731991413293667,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7944554358570034,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7921593218805381,
        standardError: 0.037171670541927666,
        low95ConfidenceInterval: 0.722554051699607,
        high95ConfidenceInterval: 0.868469825566927,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7921593218805381,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.7921593218805381,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.7898698440716347,
        standardError: 0.03713467780050072,
        low95ConfidenceInterval: 0.7203398276448971,
        high95ConfidenceInterval: 0.8661111695205435,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7876001031403944,
        standardError: 0.03709726951024023,
        low95ConfidenceInterval: 0.7181460261339148,
        high95ConfidenceInterval: 0.8637712942675091,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7876001031403944,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7853368844532094,
        standardError: 0.03705964072925468,
        low95ConfidenceInterval: 0.7159591360902685,
        high95ConfidenceInterval: 0.8614374633874535,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.783080169267999,
        standardError: 0.03702179411987329,
        low95ConfidenceInterval: 0.7137791337837989,
        high95ConfidenceInterval: 0.8591096635875299,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7808299388965392,
        standardError: 0.036983732319159524,
        low95ConfidenceInterval: 0.7116059955867244,
        high95ConfidenceInterval: 0.856787881578309,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7808299388965392,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7808299388965392,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7785732049690925,
        standardError: 0.036945631304755894,
        low95ConfidenceInterval: 0.709426471358844,
        high95ConfidenceInterval: 0.8544595669439391,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7785732049690925,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7763294781536196,
        standardError: 0.03690722818943878,
        low95ConfidenceInterval: 0.7072604619890879,
        high95ConfidenceInterval: 0.8521435751622294,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7740857513381467,
        standardError: 0.036868699040483024,
        low95ConfidenceInterval: 0.7050946986327248,
        high95ConfidenceInterval: 0.8498273375004677,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7718420245226739,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.769598297707201,
        standardError: 0.03679126105651621,
        low95ConfidenceInterval: 0.7007639130397437,
        high95ConfidenceInterval: 0.8451941214618888,
        atRisk: 344,
    },
    {
        survivalFunctionEstimate: 0.7673740251704749,
        standardError: 0.03675210234651936,
        low95ConfidenceInterval: 0.6986187105405293,
        high95ConfidenceInterval: 0.842895968318293,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7651752457287258,
        standardError: 0.036712510073948865,
        low95ConfidenceInterval: 0.6964996894188555,
        high95ConfidenceInterval: 0.8406222796230374,
        atRisk: 349,
    },
    {
        survivalFunctionEstimate: 0.7629890307409294,
        standardError: 0.03667265352194546,
        low95ConfidenceInterval: 0.6943936716365242,
        high95ConfidenceInterval: 0.8383605508082836,
        atRisk: 350,
    },
    {
        survivalFunctionEstimate: 0.7608090620816695,
        standardError: 0.03663261022336326,
        low95ConfidenceInterval: 0.6922942266042612,
        high95ConfidenceInterval: 0.8361046599865244,
        atRisk: 350,
    },
    {
        survivalFunctionEstimate: 0.7608090620816695,
        atRisk: 349,
    },
    {
        survivalFunctionEstimate: 0.7586228291446532,
        standardError: 0.03659252310348158,
        low95ConfidenceInterval: 0.6901886410901262,
        high95ConfidenceInterval: 0.8338424636928884,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7586228291446532,
        atRisk: 350,
    },
    {
        survivalFunctionEstimate: 0.7586228291446532,
        atRisk: 350,
    },
    {
        survivalFunctionEstimate: 0.7564491247631498,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 349,
    },
    {
        survivalFunctionEstimate: 0.7542754203816465,
        standardError: 0.03651171999165683,
        low95ConfidenceInterval: 0.6860036015035169,
        high95ConfidenceInterval: 0.8293417243655576,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7521079622771015,
        standardError: 0.036471078304495266,
        low95ConfidenceInterval: 0.6839177585878733,
        high95ConfidenceInterval: 0.827097088527983,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7521079622771015,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7499342398427746,
        standardError: 0.03643039127571743,
        low95ConfidenceInterval: 0.6818257780933705,
        high95ConfidenceInterval: 0.8248461442177153,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.747766799843229,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7455993598436834,
        standardError: 0.03634853490719075,
        low95ConfidenceInterval: 0.6776552463409553,
        high95ConfidenceInterval: 0.8203557906487539,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.7434382022789192,
        standardError: 0.03630736744981189,
        low95ConfidenceInterval: 0.6755766914028338,
        high95ConfidenceInterval: 0.8181163850695172,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.7412957290735044,
        standardError: 0.03626589312326489,
        low95ConfidenceInterval: 0.6735173061461899,
        high95ConfidenceInterval: 0.8158949338465,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7412957290735044,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7391532558680897,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.737010782662675,
        standardError: 0.036182612783914034,
        low95ConfidenceInterval: 0.6693991829186601,
        high95ConfidenceInterval: 0.8114513844978088,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.7348806936954417,
        standardError: 0.03614068408521011,
        low95ConfidenceInterval: 0.6673529937102899,
        high95ConfidenceInterval: 0.8092413446199944,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7327567610547034,
        standardError: 0.0360985874251007,
        low95ConfidenceInterval: 0.6653132515483737,
        high95ConfidenceInterval: 0.8070370906062441,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7306450701006553,
        standardError: 0.03605626873753675,
        low95ConfidenceInterval: 0.6632861110356798,
        high95ConfidenceInterval: 0.8048445604097306,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7306450701006553,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7285575127575105,
        standardError: 0.03601363199608488,
        low95ConfidenceInterval: 0.6612835820153499,
        high95ConfidenceInterval: 0.8026753783569499,
        atRisk: 350,
    },
    {
        survivalFunctionEstimate: 0.7264699554143658,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 349,
    },
    {
        survivalFunctionEstimate: 0.7243823980712211,
        standardError: 0.03592805320996548,
        low95ConfidenceInterval: 0.6572791197217307,
        high95ConfidenceInterval: 0.7983364188680838,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7223068038074066,
        standardError: 0.03588501183981949,
        low95ConfidenceInterval: 0.6552892730953189,
        high95ConfidenceInterval: 0.7961783295521465,
        atRisk: 349,
    },
    {
        survivalFunctionEstimate: 0.7202312095435922,
        standardError: 0.03584187006093651,
        low95ConfidenceInterval: 0.6532996223918497,
        high95ConfidenceInterval: 0.7940200444345111,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7181556152797778,
        standardError: 0.03579862751030165,
        low95ConfidenceInterval: 0.6513101683168072,
        high95ConfidenceInterval: 0.7918615628107448,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7160800210159634,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.714004426752149,
        standardError: 0.035711838628660036,
        low95ConfidenceInterval: 0.6473318528992328,
        high95ConfidenceInterval: 0.7875440072018571,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.7119644141042857,
        standardError: 0.035668024326415235,
        low95ConfidenceInterval: 0.6453788886776171,
        high95ConfidenceInterval: 0.7854197524023205,
        atRisk: 350,
    },
    {
        survivalFunctionEstimate: 0.7099244014564224,
        standardError: 0.03562411460508858,
        low95ConfidenceInterval: 0.6434261106293084,
        high95ConfidenceInterval: 0.7832953115476233,
        atRisk: 349,
    },
    {
        survivalFunctionEstimate: 0.7078843888085591,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.7058443761606957,
        standardError: 0.035536007489577116,
        low95ConfidenceInterval: 0.6395211158038323,
        high95ConfidenceInterval: 0.779045868925625,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.7038043635128324,
        standardError: 0.035491809381255006,
        low95ConfidenceInterval: 0.6375689004145393,
        high95ConfidenceInterval: 0.7769208657725288,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.7017761088629395,
        standardError: 0.035447434588617224,
        low95ConfidenceInterval: 0.63562871974513,
        high95ConfidenceInterval: 0.7748071974599944,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.6997595108489655,
        standardError: 0.0354028896182323,
        low95ConfidenceInterval: 0.6337004608295072,
        high95ConfidenceInterval: 0.7727047766110495,
        atRisk: 348,
    },
    {
        survivalFunctionEstimate: 0.6997595108489655,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.6977429128349915,
        standardError: 0.0353582514722489,
        low95ConfidenceInterval: 0.6317723836940674,
        high95ConfidenceInterval: 0.7706021741007452,
        atRisk: 347,
    },
    {
        survivalFunctionEstimate: 0.6957263148210174,
        standardError: 0.03531351979733046,
        low95ConfidenceInterval: 0.6298444890254943,
        high95ConfidenceInterval: 0.7684993892434313,
        atRisk: 346,
    },
    {
        survivalFunctionEstimate: 0.6937097168070435,
        standardError: 0.03526869423760563,
        low95ConfidenceInterval: 0.6279167775153814,
        high95ConfidenceInterval: 0.7663964213485603,
        atRisk: 345,
    },
    {
        survivalFunctionEstimate: 0.6916931187930696,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 344,
    },
    {
        survivalFunctionEstimate: 0.6896765207790957,
        standardError: 0.035178760027423824,
        low95ConfidenceInterval: 0.6240619067617569,
        high95ConfidenceInterval: 0.7621899336591698,
        atRisk: 343,
    },
    {
        survivalFunctionEstimate: 0.6876716471721798,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 344,
    },
    {
        survivalFunctionEstimate: 0.6856667735652638,
        standardError: 0.03508830612108222,
        low95ConfidenceInterval: 0.6202313634196391,
        high95ConfidenceInterval: 0.7580057251205304,
        atRisk: 343,
    },
    {
        survivalFunctionEstimate: 0.6836618999583479,
        standardError: 0.03504293910156889,
        low95ConfidenceInterval: 0.6183163649830142,
        high95ConfidenceInterval: 0.755913347801328,
        atRisk: 342,
    },
    {
        survivalFunctionEstimate: 0.6816628885549609,
        standardError: 0.03499744435326414,
        low95ConfidenceInterval: 0.616407444108708,
        high95ConfidenceInterval: 0.7538265445592934,
        atRisk: 342,
    },
    {
        survivalFunctionEstimate: 0.679663877151574,
        standardError: 0.0349518564742043,
        low95ConfidenceInterval: 0.6144987049028962,
        high95ConfidenceInterval: 0.7517395597728829,
        atRisk: 341,
    },
    {
        survivalFunctionEstimate: 0.679663877151574,
        atRisk: 340,
    },
    {
        survivalFunctionEstimate: 0.6776589689593864,
        standardError: 0.034906209055324905,
        low95ConfidenceInterval: 0.612584219227599,
        high95ConfidenceInterval: 0.7496466017197216,
        atRisk: 339,
    },
    {
        survivalFunctionEstimate: 0.6776589689593864,
        atRisk: 338,
    },
    {
        survivalFunctionEstimate: 0.6756540607671988,
        standardError: 0.03486046666108432,
        low95ConfidenceInterval: 0.610669918813427,
        high95ConfidenceInterval: 0.747553458533265,
        atRisk: 338,
    },
    {
        survivalFunctionEstimate: 0.6736491525750112,
        standardError: 0.034814628917122656,
        low95ConfidenceInterval: 0.6087558043877668,
        high95ConfidenceInterval: 0.7454601294872684,
        atRisk: 337,
    },
    {
        survivalFunctionEstimate: 0.6716442443828237,
        standardError: 0.034768695446323404,
        low95ConfidenceInterval: 0.6068418766833427,
        high95ConfidenceInterval: 0.7433666138501622,
        atRisk: 336,
    },
    {
        survivalFunctionEstimate: 0.6696393361906361,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 335,
    },
    {
        survivalFunctionEstimate: 0.6676344279984486,
        standardError: 0.034676539801791535,
        low95ConfidenceInterval: 0.6030145843961232,
        high95ConfidenceInterval: 0.7391790198493934,
        atRisk: 334,
    },
    {
        survivalFunctionEstimate: 0.6656295198062611,
        standardError: 0.03463031685978396,
        low95ConfidenceInterval: 0.6011012213059668,
        high95ConfidenceInterval: 0.7370849399954724,
        atRisk: 333,
    },
    {
        survivalFunctionEstimate: 0.6656295198062611,
        atRisk: 332,
    },
    {
        survivalFunctionEstimate: 0.6636185544896258,
        standardError: 0.03458403213725662,
        low95ConfidenceInterval: 0.5991819575308693,
        high95ConfidenceInterval: 0.7349847243025704,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.661613664596908,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.65960877470419,
        standardError: 0.034491095920116066,
        low95ConfidenceInterval: 0.5953562238942511,
        high95ConfidenceInterval: 0.7307956450356077,
        atRisk: 330,
    },
    {
        survivalFunctionEstimate: 0.6576159989497967,
        standardError: 0.034444412937803426,
        low95ConfidenceInterval: 0.5934558184222092,
        high95ConfidenceInterval: 0.7287127173586321,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6556292436659605,
        standardError: 0.03439760161194428,
        low95ConfidenceInterval: 0.5915616482050623,
        high95ConfidenceInterval: 0.7266355188073211,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6536544567874486,
        standardError: 0.03435063445252129,
        low95ConfidenceInterval: 0.5896796800694172,
        high95ConfidenceInterval: 0.7245699034903101,
        atRisk: 332,
    },
    {
        survivalFunctionEstimate: 0.6516856180621852,
        standardError: 0.03430354519958032,
        low95ConfidenceInterval: 0.5878038634792347,
        high95ConfidenceInterval: 0.7225099581266964,
        atRisk: 332,
    },
    {
        survivalFunctionEstimate: 0.6497286041941305,
        standardError: 0.03425630978949695,
        low95ConfidenceInterval: 0.5859400881334775,
        high95ConfidenceInterval: 0.7204614732076289,
        atRisk: 333,
    },
    {
        survivalFunctionEstimate: 0.6477715903260759,
        standardError: 0.03420898315837643,
        low95ConfidenceInterval: 0.5840764906863383,
        high95ConfidenceInterval: 0.718412810521614,
        atRisk: 332,
    },
    {
        survivalFunctionEstimate: 0.6458145764580213,
        standardError: 0.034161564927092276,
        low95ConfidenceInterval: 0.5822130718743569,
        high95ConfidenceInterval: 0.716363969333309,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6438751933455348,
        standardError: 0.03411398349894208,
        low95ConfidenceInterval: 0.5803675002750543,
        high95ConfidenceInterval: 0.7143323228975943,
        atRisk: 333,
    },
    {
        survivalFunctionEstimate: 0.641941634206359,
        standardError: 0.03406629149160535,
        low95ConfidenceInterval: 0.5785279349385396,
        high95ConfidenceInterval: 0.7123062463203431,
        atRisk: 333,
    },
    {
        survivalFunctionEstimate: 0.641941634206359,
        atRisk: 332,
    },
    {
        survivalFunctionEstimate: 0.641941634206359,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6400080750671832,
        standardError: 0.03401851109563754,
        low95ConfidenceInterval: 0.5766885419660017,
        high95ConfidenceInterval: 0.7102799975092091,
        atRisk: 332,
    },
    {
        survivalFunctionEstimate: 0.6380861288958102,
        standardError: 0.03397060218234941,
        low95ConfidenceInterval: 0.5748609462577111,
        high95ConfidenceInterval: 0.708265034422277,
        atRisk: 333,
    },
    {
        survivalFunctionEstimate: 0.6380861288958102,
        atRisk: 332,
    },
    {
        survivalFunctionEstimate: 0.6361583762405358,
        standardError: 0.03392262560579619,
        low95ConfidenceInterval: 0.5730277091674585,
        high95ConfidenceInterval: 0.706244171418818,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6342364476114104,
        standardError: 0.033874541210030834,
        low95ConfidenceInterval: 0.5712004729990434,
        high95ConfidenceInterval: 0.7042288837170045,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6323203254131886,
        standardError: 0.033826350769541214,
        low95ConfidenceInterval: 0.5693792168870334,
        high95ConfidenceInterval: 0.7022191574125685,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6304157461197754,
        standardError: 0.03377803990648778,
        low95ConfidenceInterval: 0.5675696729466835,
        high95ConfidenceInterval: 0.7002206634692519,
        atRisk: 332,
    },
    {
        survivalFunctionEstimate: 0.6285111668263621,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6266065875329488,
        standardError: 0.03368116174442781,
        low95ConfidenceInterval: 0.5639510850836905,
        high95ConfidenceInterval: 0.6962231759540276,
        atRisk: 330,
    },
    {
        survivalFunctionEstimate: 0.6266065875329488,
        atRisk: 329,
    },
    {
        survivalFunctionEstimate: 0.6247020082395356,
        standardError: 0.03363259370676151,
        low95ConfidenceInterval: 0.5621420425959901,
        high95ConfidenceInterval: 0.6942241809495509,
        atRisk: 329,
    },
    {
        survivalFunctionEstimate: 0.6228146909940989,
        standardError: 0.0335838969981537,
        low95ConfidenceInterval: 0.5603504152934856,
        high95ConfidenceInterval: 0.6922420841160827,
        atRisk: 331,
    },
    {
        survivalFunctionEstimate: 0.6209273737486622,
        standardError: 0.03353511655058653,
        low95ConfidenceInterval: 0.5585589512618023,
        high95ConfidenceInterval: 0.6902598241411035,
        atRisk: 330,
    },
    {
        survivalFunctionEstimate: 0.6190400565032256,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 329,
    },
    {
        survivalFunctionEstimate: 0.6171527392577889,
        standardError: 0.03343730297198089,
        low95ConfidenceInterval: 0.5549765158598703,
        high95ConfidenceInterval: 0.686294811922389,
        atRisk: 328,
    },
    {
        survivalFunctionEstimate: 0.615276895065516,
        standardError: 0.03338824612320433,
        low95ConfidenceInterval: 0.5531969990014581,
        high95ConfidenceInterval: 0.6843234115239012,
        atRisk: 329,
    },
    {
        survivalFunctionEstimate: 0.6134067525273229,
        standardError: 0.03333909616674878,
        low95ConfidenceInterval: 0.5514233333144747,
        high95ConfidenceInterval: 0.6823574943491413,
        atRisk: 329,
    },
    {
        survivalFunctionEstimate: 0.6134067525273229,
        atRisk: 328,
    },
    {
        survivalFunctionEstimate: 0.6115366099891298,
        standardError: 0.033289863686612034,
        low95ConfidenceInterval: 0.5496498285158671,
        high95ConfidenceInterval: 0.6803914164164999,
        atRisk: 328,
    },
    {
        survivalFunctionEstimate: 0.6096721691050166,
        standardError: 0.03324053938099707,
        low95ConfidenceInterval: 0.5478821723837145,
        high95ConfidenceInterval: 0.6784308242117654,
        atRisk: 328,
    },
    {
        survivalFunctionEstimate: 0.6078077282209035,
        standardError: 0.03319113282621302,
        low95ConfidenceInterval: 0.5461146765995588,
        high95ConfidenceInterval: 0.6764700717903285,
        atRisk: 327,
    },
    {
        survivalFunctionEstimate: 0.6059489889908701,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 327,
    },
    {
        survivalFunctionEstimate: 0.6040902497608368,
        standardError: 0.033092056647099105,
        low95ConfidenceInterval: 0.5425915371782166,
        high95ConfidenceInterval: 0.6725593837197076,
        atRisk: 326,
    },
    {
        survivalFunctionEstimate: 0.6040902497608368,
        atRisk: 326,
    },
    {
        survivalFunctionEstimate: 0.6022315105308035,
        standardError: 0.03304239522267011,
        low95ConfidenceInterval: 0.5408302079032844,
        high95ConfidenceInterval: 0.6706037994480352,
        atRisk: 325,
    },
    {
        survivalFunctionEstimate: 0.600406566559498,
        standardError: 0.032992624434783284,
        low95ConfidenceInterval: 0.5391026995370464,
        high95ConfidenceInterval: 0.668681580480553,
        atRisk: 330,
    },
    {
        survivalFunctionEstimate: 0.5985816225881925,
        standardError: 0.032942776434184876,
        low95ConfidenceInterval: 0.537375341687785,
        high95ConfidenceInterval: 0.6667592111222802,
        atRisk: 329,
    },
    {
        survivalFunctionEstimate: 0.596756678616887,
        standardError: 0.03289285086983514,
        low95ConfidenceInterval: 0.5356481350373814,
        high95ConfidenceInterval: 0.6648366906924936,
        atRisk: 328,
    },
    {
        survivalFunctionEstimate: 0.5949317346455815,
        standardError: 0.032842847388011964,
        low95ConfidenceInterval: 0.5339210802729082,
        high95ConfidenceInterval: 0.6629140185052926,
        atRisk: 327,
    },
    {
        survivalFunctionEstimate: 0.593106790674276,
        standardError: 0.032792765632281985,
        low95ConfidenceInterval: 0.5321941780866866,
        high95ConfidenceInterval: 0.6609911938695434,
        atRisk: 326,
    },
    {
        survivalFunctionEstimate: 0.5912874446906126,
        standardError: 0.03274260418137357,
        low95ConfidenceInterval: 0.5304729983519031,
        high95ConfidenceInterval: 0.6590737762995886,
        atRisk: 326,
    },
    {
        survivalFunctionEstimate: 0.5894792262053201,
        standardError: 0.03269236542416107,
        low95ConfidenceInterval: 0.528763035776617,
        high95ConfidenceInterval: 0.6571672651384485,
        atRisk: 327,
    },
    {
        survivalFunctionEstimate: 0.5876874960344833,
        standardError: 0.03264205817986329,
        low95ConfidenceInterval: 0.5270696056467303,
        high95ConfidenceInterval: 0.6552770057220304,
        atRisk: 329,
    },
    {
        survivalFunctionEstimate: 0.5876874960344833,
        atRisk: 330,
    },
    {
        survivalFunctionEstimate: 0.5859012118520077,
        standardError: 0.03259168131534819,
        low95ConfidenceInterval: 0.5253817279522479,
        high95ConfidenceInterval: 0.6533920229537409,
        atRisk: 329,
    },
    {
        survivalFunctionEstimate: 0.5841149276695321,
        standardError: 0.032541230927835084,
        low95ConfidenceInterval: 0.5236939935635179,
        high95ConfidenceInterval: 0.6515068970043484,
        atRisk: 328,
    },
    {
        survivalFunctionEstimate: 0.5823340894754176,
        standardError: 0.03249071203098561,
        low95ConfidenceInterval: 0.5220118094403285,
        high95ConfidenceInterval: 0.6496270498721887,
        atRisk: 328,
    },
    {
        survivalFunctionEstimate: 0.5823340894754176,
        atRisk: 327,
    },
    {
        survivalFunctionEstimate: 0.5805477885874563,
        standardError: 0.0324401144599133,
        low95ConfidenceInterval: 0.5203243453967664,
        high95ConfidenceInterval: 0.6477416208091967,
        atRisk: 326,
    },
    {
        survivalFunctionEstimate: 0.5787614876994949,
        standardError: 0.03238944232768389,
        low95ConfidenceInterval: 0.5186370266753946,
        high95ConfidenceInterval: 0.6458560465521505,
        atRisk: 325,
    },
    {
        survivalFunctionEstimate: 0.5769751868115335,
        standardError: 0.03233869528380339,
        low95ConfidenceInterval: 0.5169498539569374,
        high95ConfidenceInterval: 0.643970326421516,
        atRisk: 324,
    },
    {
        survivalFunctionEstimate: 0.5769751868115335,
        atRisk: 323,
    },
    {
        survivalFunctionEstimate: 0.5751998785444211,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 325,
    },
    {
        survivalFunctionEstimate: 0.5734245702773086,
        standardError: 0.032237000304249516,
        low95ConfidenceInterval: 0.5135977661235069,
        high95ConfidenceInterval: 0.6402203426224491,
        atRisk: 324,
    },
    {
        survivalFunctionEstimate: 0.5716547413566997,
        standardError: 0.0321860490113001,
        low95ConfidenceInterval: 0.5119273742467064,
        high95ConfidenceInterval: 0.6383505937662752,
        atRisk: 324,
    },
    {
        survivalFunctionEstimate: 0.5698849124360907,
        standardError: 0.03213502415466293,
        low95ConfidenceInterval: 0.5102571257361703,
        high95ConfidenceInterval: 0.6364807016731665,
        atRisk: 323,
    },
    {
        survivalFunctionEstimate: 0.5681205628619852,
        standardError: 0.032083933505858045,
        low95ConfidenceInterval: 0.5085924551798072,
        high95ConfidenceInterval: 0.6346161266440933,
        atRisk: 323,
    },
    {
        survivalFunctionEstimate: 0.5663562132878797,
        standardError: 0.032032769504187535,
        low95ConfidenceInterval: 0.5069279275740886,
        high95ConfidenceInterval: 0.6327514087945108,
        atRisk: 322,
    },
    {
        survivalFunctionEstimate: 0.5645973430602775,
        standardError: 0.03198154083143259,
        low95ConfidenceInterval: 0.5052689757329984,
        high95ConfidenceInterval: 0.6308920101977009,
        atRisk: 322,
    },
    {
        survivalFunctionEstimate: 0.5628384728326754,
        standardError: 0.03193023901472722,
        low95ConfidenceInterval: 0.5036101664306483,
        high95ConfidenceInterval: 0.6290324691930992,
        atRisk: 321,
    },
    {
        survivalFunctionEstimate: 0.5610850819515767,
        standardError: 0.031878873646357275,
        low95ConfidenceInterval: 0.5019569307066823,
        high95ConfidenceInterval: 0.6271782496266995,
        atRisk: 321,
    },
    {
        survivalFunctionEstimate: 0.5593371533473662,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 321,
    },
    {
        survivalFunctionEstimate: 0.5575892247431556,
        standardError: 0.031775946321799614,
        low95ConfidenceInterval: 0.4986617080391915,
        high95ConfidenceInterval: 0.6234802844040276,
        atRisk: 320,
    },
    {
        survivalFunctionEstimate: 0.5558412961389451,
        standardError: 0.031724373733967796,
        low95ConfidenceInterval: 0.4970143089602091,
        high95ConfidenceInterval: 0.6216310897362066,
        atRisk: 319,
    },
    {
        survivalFunctionEstimate: 0.5540988469347478,
        standardError: 0.03167273982009161,
        low95ConfidenceInterval: 0.4953724791657194,
        high95ConfidenceInterval: 0.6197872209039417,
        atRisk: 319,
    },
    {
        survivalFunctionEstimate: 0.5523726823025523,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 321,
    },
    {
        survivalFunctionEstimate: 0.5506465176703568,
        standardError: 0.03156933769478181,
        low95ConfidenceInterval: 0.4921214760205731,
        high95ConfidenceInterval: 0.6161315898552958,
        atRisk: 320,
    },
    {
        survivalFunctionEstimate: 0.5506465176703568,
        atRisk: 319,
    },
    {
        survivalFunctionEstimate: 0.5489203530381613,
        standardError: 0.031517530985620705,
        low95ConfidenceInterval: 0.4904961802991836,
        high95ConfidenceInterval: 0.6143035686756013,
        atRisk: 319,
    },
    {
        survivalFunctionEstimate: 0.5471995995803615,
        standardError: 0.031465668623165785,
        low95ConfidenceInterval: 0.4888763753071578,
        high95ConfidenceInterval: 0.6124808170425082,
        atRisk: 319,
    },
    {
        survivalFunctionEstimate: 0.545484240334028,
        standardError: 0.03141375199844324,
        low95ConfidenceInterval: 0.4872620415507213,
        high95ConfidenceInterval: 0.6106633209223994,
        atRisk: 319,
    },
    {
        survivalFunctionEstimate: 0.5437688810876946,
        standardError: 0.03136176555010138,
        low95ConfidenceInterval: 0.48564784383721354,
        high95ConfidenceInterval: 0.6088456888907247,
        atRisk: 318,
    },
    {
        survivalFunctionEstimate: 0.5420535218413612,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 317,
    },
    {
        survivalFunctionEstimate: 0.5403381625950278,
        standardError: 0.031257581788555214,
        low95ConfidenceInterval: 0.4824198592456338,
        high95ConfidenceInterval: 0.6052100143910341,
        atRisk: 316,
    },
    {
        survivalFunctionEstimate: 0.5386282317007397,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 316,
    },
    {
        survivalFunctionEstimate: 0.5369183008064516,
        standardError: 0.03115314950155905,
        low95ConfidenceInterval: 0.47920315741442476,
        high95ConfidenceInterval: 0.6015846458448431,
        atRisk: 315,
    },
    {
        survivalFunctionEstimate: 0.5352083699121636,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 314,
    },
    {
        survivalFunctionEstimate: 0.5334984390178755,
        standardError: 0.031048434456995025,
        low95ConfidenceInterval: 0.4759870064546844,
        high95ConfidenceInterval: 0.5979587269712723,
        atRisk: 313,
    },
    {
        survivalFunctionEstimate: 0.5317885081235875,
        standardError: 0.030995970005378334,
        low95ConfidenceInterval: 0.47437913928964814,
        high95ConfidenceInterval: 0.596145559426968,
        atRisk: 312,
    },
    {
        survivalFunctionEstimate: 0.5300785772292994,
        standardError: 0.030943433784240887,
        low95ConfidenceInterval: 0.47277141193940186,
        high95ConfidenceInterval: 0.5943322522078678,
        atRisk: 311,
    },
    {
        survivalFunctionEstimate: 0.5283686463350113,
        standardError: 0.030890825427406052,
        low95ConfidenceInterval: 0.4711638251147849,
        high95ConfidenceInterval: 0.592518804604492,
        atRisk: 310,
    },
    {
        survivalFunctionEstimate: 0.5266697117808795,
        standardError: 0.030838181652511433,
        low95ConfidenceInterval: 0.46956724395971705,
        high95ConfidenceInterval: 0.59071621556965,
        atRisk: 311,
    },
    {
        survivalFunctionEstimate: 0.5249707772267477,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 310,
    },
    {
        survivalFunctionEstimate: 0.5232718426726158,
        standardError: 0.03073268067669273,
        low95ConfidenceInterval: 0.46637449740172826,
        high95ConfidenceInterval: 0.5871106221705256,
        atRisk: 309,
    },
    {
        survivalFunctionEstimate: 0.5215784062885621,
        standardError: 0.030679842669350782,
        low95ConfidenceInterval: 0.46478376294093227,
        high95ConfidenceInterval: 0.5853131189117928,
        atRisk: 309,
    },
    {
        survivalFunctionEstimate: 0.5198849699045083,
        standardError: 0.030626933465661667,
        low95ConfidenceInterval: 0.4631931671633536,
        high95ConfidenceInterval: 0.5835154771125803,
        atRisk: 308,
    },
    {
        survivalFunctionEstimate: 0.5182133140527253,
        standardError: 0.030574041064422665,
        low95ConfidenceInterval: 0.4616242021615605,
        high95ConfidenceInterval: 0.5817395136651055,
        atRisk: 311,
    },
    {
        survivalFunctionEstimate: 0.5165416582009423,
        standardError: 0.03052108035535021,
        low95ConfidenceInterval: 0.4600553702116556,
        high95ConfidenceInterval: 0.579963417303936,
        atRisk: 310,
    },
    {
        survivalFunctionEstimate: 0.5148700023491594,
        standardError: 0.030468050982238596,
        low95ConfidenceInterval: 0.45848667200506266,
        high95ConfidenceInterval: 0.5781871873389947,
        atRisk: 309,
    },
    {
        survivalFunctionEstimate: 0.5131983464973764,
        standardError: 0.030414952585934445,
        low95ConfidenceInterval: 0.45691810823890655,
        high95ConfidenceInterval: 0.576410823074521,
        atRisk: 308,
    },
    {
        survivalFunctionEstimate: 0.5115266906455934,
        standardError: 0.030361784804302835,
        low95ConfidenceInterval: 0.4553496796160786,
        high95ConfidenceInterval: 0.5746343238090056,
        atRisk: 307,
    },
    {
        survivalFunctionEstimate: 0.5098604799268781,
        standardError: 0.030308571613052324,
        low95ConfidenceInterval: 0.45378675530909535,
        high95ConfidenceInterval: 0.5728631476125766,
        atRisk: 307,
    },
    {
        survivalFunctionEstimate: 0.5081942692081628,
        standardError: 0.030255289212433582,
        low95ConfidenceInterval: 0.4522239657977828,
        high95ConfidenceInterval: 0.5710918367636961,
        atRisk: 306,
    },
    {
        survivalFunctionEstimate: 0.5065280584894475,
        standardError: 0.03020193723614813,
        low95ConfidenceInterval: 0.45066131179306473,
        high95ConfidenceInterval: 0.569320390552854,
        atRisk: 305,
    },
    {
        survivalFunctionEstimate: 0.5048618477707322,
        standardError: 0.030148515314824625,
        low95ConfidenceInterval: 0.44909879401180747,
        high95ConfidenceInterval: 0.5675488082646167,
        atRisk: 304,
    },
    {
        survivalFunctionEstimate: 0.5031956370520169,
        standardError: 0.030095023075983027,
        low95ConfidenceInterval: 0.44753641317688836,
        high95ConfidenceInterval: 0.5657770891775586,
        atRisk: 303,
    },
    {
        survivalFunctionEstimate: 0.5015403882459247,
        standardError: 0.030041511764781,
        low95ConfidenceInterval: 0.44598497207957205,
        high95ConfidenceInterval: 0.5640162265310432,
        atRisk: 304,
    },
    {
        survivalFunctionEstimate: 0.49989058433722106,
        standardError: 0.029987958315606984,
        low95ConfidenceInterval: 0.4444390286795707,
        high95ConfidenceInterval: 0.5622606931066199,
        atRisk: 304,
    },
    {
        survivalFunctionEstimate: 0.4982407804285173,
        standardError: 0.029934336186436927,
        low95ConfidenceInterval: 0.4428932190292589,
        high95ConfidenceInterval: 0.5605050260785732,
        atRisk: 303,
    },
    {
        survivalFunctionEstimate: 0.4982407804285173,
        atRisk: 304,
    },
    {
        survivalFunctionEstimate: 0.49659642141720206,
        standardError: 0.029880672996250322,
        low95ConfidenceInterval: 0.4413529049739559,
        high95ConfidenceInterval: 0.5587546903739619,
        atRisk: 303,
    },
    {
        survivalFunctionEstimate: 0.49659642141720206,
        atRisk: 303,
    },
    {
        survivalFunctionEstimate: 0.4949520624058868,
        standardError: 0.029826941295568562,
        low95ConfidenceInterval: 0.4398127243332399,
        high95ConfidenceInterval: 0.5570042214018001,
        atRisk: 302,
    },
    {
        survivalFunctionEstimate: 0.49330770339457153,
        standardError: 0.029773140713468184,
        low95ConfidenceInterval: 0.4382726778268975,
        high95ConfidenceInterval: 0.5552536184437724,
        atRisk: 301,
    },
    {
        survivalFunctionEstimate: 0.49330770339457153,
        atRisk: 301,
    },
    {
        survivalFunctionEstimate: 0.4916633443832563,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 300,
    },
    {
        survivalFunctionEstimate: 0.4900189853719411,
        standardError: 0.029665331405464616,
        low95ConfidenceInterval: 0.43519299012712925,
        high95ConfidenceInterval: 0.5517520076663064,
        atRisk: 299,
    },
    {
        survivalFunctionEstimate: 0.4900189853719411,
        atRisk: 298,
    },
    {
        survivalFunctionEstimate: 0.4900189853719411,
        atRisk: 297,
    },
    {
        survivalFunctionEstimate: 0.4883635158267656,
        standardError: 0.029611266283072743,
        low95ConfidenceInterval: 0.43364240920878616,
        high95ConfidenceInterval: 0.549989849991469,
        atRisk: 296,
    },
    {
        survivalFunctionEstimate: 0.48671363908410764,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 296,
    },
    {
        survivalFunctionEstimate: 0.48506376234144966,
        standardError: 0.02950297456339488,
        low95ConfidenceInterval: 0.43055268554885046,
        high95ConfidenceInterval: 0.5464763347995579,
        atRisk: 295,
    },
    {
        survivalFunctionEstimate: 0.48341388559879167,
        standardError: 0.029448720697973843,
        low95ConfidenceInterval: 0.4290080340175114,
        high95ConfidenceInterval: 0.5447193671440261,
        atRisk: 294,
    },
    {
        survivalFunctionEstimate: 0.48176400885613363,
        standardError: 0.029394394297412516,
        low95ConfidenceInterval: 0.42746352371624696,
        high95ConfidenceInterval: 0.5429622584199723,
        atRisk: 293,
    },
    {
        survivalFunctionEstimate: 0.48176400885613363,
        atRisk: 292,
    },
    {
        survivalFunctionEstimate: 0.4801084624339476,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 291,
    },
    {
        survivalFunctionEstimate: 0.4784529160117616,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 290,
    },
    {
        survivalFunctionEstimate: 0.4767973695895755,
        standardError: 0.029230889825664227,
        low95ConfidenceInterval: 0.42281410187905144,
        high95ConfidenceInterval: 0.5376730119388711,
        atRisk: 289,
    },
    {
        survivalFunctionEstimate: 0.4767973695895755,
        atRisk: 288,
    },
    {
        survivalFunctionEstimate: 0.47513605471295683,
        standardError: 0.02917621160961467,
        low95ConfidenceInterval: 0.42125890316196135,
        high95ConfidenceInterval: 0.5359038557848549,
        atRisk: 287,
    },
    {
        survivalFunctionEstimate: 0.4734862767452035,
        standardError: 0.02912151156942743,
        low95ConfidenceInterval: 0.41971521794838673,
        high95ConfidenceInterval: 0.5341461178412751,
        atRisk: 288,
    },
    {
        survivalFunctionEstimate: 0.47184791592601594,
        standardError: 0.029066795213277485,
        low95ConfidenceInterval: 0.4181829171411335,
        high95ConfidenceInterval: 0.5323996907520381,
        atRisk: 289,
    },
    {
        survivalFunctionEstimate: 0.4702095551068284,
        standardError: 0.02901200550907545,
        low95ConfidenceInterval: 0.4166507591262242,
        high95ConfidenceInterval: 0.5306531210393888,
        atRisk: 288,
    },
    {
        survivalFunctionEstimate: 0.4685768830404852,
        standardError: 0.02895717285153187,
        low95ConfidenceInterval: 0.41512434145657806,
        high95ConfidenceInterval: 0.5289121195580457,
        atRisk: 288,
    },
    {
        survivalFunctionEstimate: 0.4685768830404852,
        atRisk: 287,
    },
    {
        survivalFunctionEstimate: 0.46694421097414207,
        standardError: 0.028902267034743058,
        low95ConfidenceInterval: 0.41359806620580575,
        high95ConfidenceInterval: 0.5271709758279413,
        atRisk: 287,
    },
    {
        survivalFunctionEstimate: 0.4653115389077989,
        standardError: 0.028847287640971157,
        low95ConfidenceInterval: 0.41207193418412724,
        high95ConfidenceInterval: 0.5254296890406474,
        atRisk: 286,
    },
    {
        survivalFunctionEstimate: 0.46367886684145576,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 285,
    },
    {
        survivalFunctionEstimate: 0.4620461947751126,
        standardError: 0.028737106432716646,
        low95ConfidenceInterval: 0.4090201031052722,
        high95ConfidenceInterval: 0.5219466830245623,
        atRisk: 284,
    },
    {
        survivalFunctionEstimate: 0.46041352270876945,
        standardError: 0.028681903763807408,
        low95ConfidenceInterval: 0.40749440570514883,
        high95ConfidenceInterval: 0.5202049621424291,
        atRisk: 283,
    },
    {
        survivalFunctionEstimate: 0.4587808506424263,
        standardError: 0.02862662580896613,
        low95ConfidenceInterval: 0.4059688548484095,
        high95ConfidenceInterval: 0.5184630948962388,
        atRisk: 282,
    },
    {
        survivalFunctionEstimate: 0.45714817857608314,
        standardError: 0.02857127213121692,
        low95ConfidenceInterval: 0.40444345138244003,
        high95ConfidenceInterval: 0.5167210804405277,
        atRisk: 281,
    },
    {
        survivalFunctionEstimate: 0.45714817857608314,
        atRisk: 281,
    },
    {
        survivalFunctionEstimate: 0.45552131673061663,
        standardError: 0.028515874534947323,
        low95ConfidenceInterval: 0.4029239101648695,
        high95ConfidenceInterval: 0.5149847521113589,
        atRisk: 281,
    },
    {
        survivalFunctionEstimate: 0.4538944548851501,
        standardError: 0.02846040141475361,
        low95ConfidenceInterval: 0.4014045159443631,
        high95ConfidenceInterval: 0.513248276967674,
        atRisk: 280,
    },
    {
        survivalFunctionEstimate: 0.4522675930396836,
        standardError: 0.028404852328153533,
        low95ConfidenceInterval: 0.3998852695789147,
        high95ConfidenceInterval: 0.5115116541534498,
        atRisk: 279,
    },
    {
        survivalFunctionEstimate: 0.45065235163597045,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 280,
    },
    {
        survivalFunctionEstimate: 0.4490371102322573,
        standardError: 0.028293662109858238,
        low95ConfidenceInterval: 0.3968700630340234,
        high95ConfidenceInterval: 0.5080613156464019,
        atRisk: 279,
    },
    {
        survivalFunctionEstimate: 0.4474218688285441,
        standardError: 0.02823795452361868,
        low95ConfidenceInterval: 0.39536267866532915,
        high95ConfidenceInterval: 0.5063359277659155,
        atRisk: 278,
    },
    {
        survivalFunctionEstimate: 0.4474218688285441,
        atRisk: 277,
    },
    {
        survivalFunctionEstimate: 0.4458007751009045,
        standardError: 0.02818213681851982,
        low95ConfidenceInterval: 0.393849690246254,
        high95ConfidenceInterval: 0.5046045128442436,
        atRisk: 276,
    },
    {
        survivalFunctionEstimate: 0.4441796813732648,
        standardError: 0.02812624197819492,
        low95ConfidenceInterval: 0.3923368519405166,
        high95ConfidenceInterval: 0.5028729480012437,
        atRisk: 275,
    },
    {
        survivalFunctionEstimate: 0.4425585876456251,
        standardError: 0.02807026954185812,
        low95ConfidenceInterval: 0.3908241646414307,
        high95ConfidenceInterval: 0.5011412323457131,
        atRisk: 274,
    },
    {
        survivalFunctionEstimate: 0.4409434103184513,
        standardError: 0.028014253817063545,
        low95ConfidenceInterval: 0.3893174432439647,
        high95ConfidenceInterval: 0.4994153087084423,
        atRisk: 274,
    },
    {
        survivalFunctionEstimate: 0.4393282329912775,
        standardError: 0.027958160702115403,
        low95ConfidenceInterval: 0.38781087244565793,
        high95ConfidenceInterval: 0.4976892346675599,
        atRisk: 273,
    },
    {
        survivalFunctionEstimate: 0.4393282329912775,
        atRisk: 272,
    },
    {
        survivalFunctionEstimate: 0.4377070956001658,
        standardError: 0.02790195481764576,
        low95ConfidenceInterval: 0.386298596296448,
        high95ConfidenceInterval: 0.4959570223022691,
        atRisk: 271,
    },
    {
        survivalFunctionEstimate: 0.4360859582090541,
        standardError: 0.027845669442263948,
        low95ConfidenceInterval: 0.38478647482724926,
        high95ConfidenceInterval: 0.49422465545985356,
        atRisk: 270,
    },
    {
        survivalFunctionEstimate: 0.43446482081794235,
        standardError: 0.027789304092959398,
        low95ConfidenceInterval: 0.3832745089742857,
        high95ConfidenceInterval: 0.49249213320636165,
        atRisk: 269,
    },
    {
        survivalFunctionEstimate: 0.43286162959721197,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 271,
    },
    {
        survivalFunctionEstimate: 0.4312584383764816,
        standardError: 0.027676552054147194,
        low95ConfidenceInterval: 0.38028629786436813,
        high95ConfidenceInterval: 0.48906269228047244,
        atRisk: 270,
    },
    {
        survivalFunctionEstimate: 0.4296611849010132,
        standardError: 0.027620097743874802,
        low95ConfidenceInterval: 0.37879824738083334,
        high95ConfidenceInterval: 0.48735371688492024,
        atRisk: 270,
    },
    {
        survivalFunctionEstimate: 0.4280639314255447,
        standardError: 0.027563566207900767,
        low95ConfidenceInterval: 0.377310347151821,
        high95ConfidenceInterval: 0.485644591436986,
        atRisk: 269,
    },
    {
        survivalFunctionEstimate: 0.4264666779500763,
        standardError: 0.027506956970088738,
        low95ConfidenceInterval: 0.37582259810015095,
        high95ConfidenceInterval: 0.48393531501612236,
        atRisk: 268,
    },
    {
        survivalFunctionEstimate: 0.42486942447460785,
        standardError: 0.027450269549717545,
        low95ConfidenceInterval: 0.37433500115749313,
        high95ConfidenceInterval: 0.4822258866929658,
        atRisk: 267,
    },
    {
        survivalFunctionEstimate: 0.42486942447460785,
        atRisk: 266,
    },
    {
        survivalFunctionEstimate: 0.42326614362753384,
        standardError: 0.02739346481969684,
        low95ConfidenceInterval: 0.37284164118513274,
        high95ConfidenceInterval: 0.48051024497117784,
        atRisk: 265,
    },
    {
        survivalFunctionEstimate: 0.42326614362753384,
        atRisk: 264,
    },
    {
        survivalFunctionEstimate: 0.42326614362753384,
        atRisk: 263,
    },
    {
        survivalFunctionEstimate: 0.42326614362753384,
        atRisk: 262,
    },
    {
        survivalFunctionEstimate: 0.4216444342649762,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 261,
    },
    {
        survivalFunctionEstimate: 0.4200227249024186,
        standardError: 0.027279390428001947,
        low95ConfidenceInterval: 0.369819194053226,
        high95ConfidenceInterval: 0.4770414631563494,
        atRisk: 260,
    },
    {
        survivalFunctionEstimate: 0.4200227249024186,
        atRisk: 259,
    },
    {
        survivalFunctionEstimate: 0.4183947298446573,
        standardError: 0.027222191222411287,
        low95ConfidenceInterval: 0.3683020393328773,
        high95ConfidenceInterval: 0.47530051769158693,
        atRisk: 258,
    },
    {
        survivalFunctionEstimate: 0.416766734786896,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 257,
    },
    {
        survivalFunctionEstimate: 0.41513873972913473,
        standardError: 0.027107531099346417,
        low95ConfidenceInterval: 0.36526823900106586,
        high95ConfidenceInterval: 0.47181811836476534,
        atRisk: 256,
    },
    {
        survivalFunctionEstimate: 0.41513873972913473,
        atRisk: 255,
    },
    {
        survivalFunctionEstimate: 0.41350433524201213,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 254,
    },
    {
        survivalFunctionEstimate: 0.4118699307548896,
        standardError: 0.02699244985148736,
        low95ConfidenceInterval: 0.3622225209371448,
        high95ConfidenceInterval: 0.4683221778180767,
        atRisk: 253,
    },
    {
        survivalFunctionEstimate: 0.410235526267767,
        standardError: 0.026934773452087817,
        low95ConfidenceInterval: 0.36069992600645145,
        high95ConfidenceInterval: 0.46657394381938866,
        atRisk: 252,
    },
    {
        survivalFunctionEstimate: 0.410235526267767,
        atRisk: 251,
    },
    {
        survivalFunctionEstimate: 0.4085945841626959,
        standardError: 0.02687697327990139,
        low95ConfidenceInterval: 0.35917107686795857,
        high95ConfidenceInterval: 0.464818981703423,
        atRisk: 250,
    },
    {
        survivalFunctionEstimate: 0.4069536420576248,
        standardError: 0.026819079744810473,
        low95ConfidenceInterval: 0.35764240932121727,
        high95ConfidenceInterval: 0.46306383825756325,
        atRisk: 249,
    },
    {
        survivalFunctionEstimate: 0.40531929008148976,
        standardError: 0.026761124884347898,
        low95ConfidenceInterval: 0.3561204078121091,
        high95ConfidenceInterval: 0.4613151150799528,
        atRisk: 249,
    },
    {
        survivalFunctionEstimate: 0.40369150176790947,
        standardError: 0.026703111012469406,
        low95ConfidenceInterval: 0.3546050417223644,
        high95ConfidenceInterval: 0.4595727906407596,
        atRisk: 249,
    },
    {
        survivalFunctionEstimate: 0.4020637134543292,
        standardError: 0.02664500495807485,
        low95ConfidenceInterval: 0.35308985491369005,
        high95ConfidenceInterval: 0.45783028718341456,
        atRisk: 248,
    },
    {
        survivalFunctionEstimate: 0.4020637134543292,
        atRisk: 247,
    },
    {
        survivalFunctionEstimate: 0.4004489997456772,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 249,
    },
    {
        survivalFunctionEstimate: 0.3988342860370253,
        standardError: 0.026528661635646475,
        low95ConfidenceInterval: 0.3500857150170935,
        high95ConfidenceInterval: 0.4543709751507482,
        atRisk: 248,
    },
    {
        survivalFunctionEstimate: 0.3988342860370253,
        atRisk: 247,
    },
    {
        survivalFunctionEstimate: 0.3988342860370253,
        atRisk: 246,
    },
    {
        survivalFunctionEstimate: 0.3988342860370253,
        atRisk: 245,
    },
    {
        survivalFunctionEstimate: 0.3971997192909719,
        standardError: 0.026470246728425,
        low95ConfidenceInterval: 0.34856439632394615,
        high95ConfidenceInterval: 0.4526211473939582,
        atRisk: 244,
    },
    {
        survivalFunctionEstimate: 0.3971997192909719,
        atRisk: 244,
    },
    {
        survivalFunctionEstimate: 0.3971997192909719,
        atRisk: 243,
    },
    {
        survivalFunctionEstimate: 0.3971997192909719,
        atRisk: 242,
    },
    {
        survivalFunctionEstimate: 0.3971997192909719,
        atRisk: 241,
    },
    {
        survivalFunctionEstimate: 0.3955515876756567,
        standardError: 0.026411671106473023,
        low95ConfidenceInterval: 0.3470299172813519,
        high95ConfidenceInterval: 0.45085755066438,
        atRisk: 241,
    },
    {
        survivalFunctionEstimate: 0.39390345606034144,
        standardError: 0.026352995364143254,
        low95ConfidenceInterval: 0.34549563293030694,
        high95ConfidenceInterval: 0.449093759536984,
        atRisk: 240,
    },
    {
        survivalFunctionEstimate: 0.3922690018858172,
        standardError: 0.026294283691867273,
        low95ConfidenceInterval: 0.3439750023398316,
        high95ConfidenceInterval: 0.4473434662222164,
        atRisk: 241,
    },
    {
        survivalFunctionEstimate: 0.3922690018858172,
        atRisk: 240,
    },
    {
        survivalFunctionEstimate: 0.39063454771129297,
        standardError: 0.02623547387880238,
        low95ConfidenceInterval: 0.342454562580195,
        high95ConfidenceInterval: 0.44559298236790773,
        atRisk: 240,
    },
    {
        survivalFunctionEstimate: 0.39063454771129297,
        atRisk: 240,
    },
    {
        survivalFunctionEstimate: 0.3890069037624959,
        standardError: 0.026176600077293596,
        low95ConfidenceInterval: 0.34094101108337027,
        high95ConfidenceInterval: 0.44384913007100785,
        atRisk: 240,
    },
    {
        survivalFunctionEstimate: 0.3890069037624959,
        atRisk: 239,
    },
    {
        survivalFunctionEstimate: 0.3890069037624959,
        atRisk: 238,
    },
    {
        survivalFunctionEstimate: 0.3873655244217259,
        standardError: 0.026117560172283092,
        low95ConfidenceInterval: 0.3394141415247113,
        high95ConfidenceInterval: 0.44209133077501456,
        atRisk: 237,
    },
    {
        survivalFunctionEstimate: 0.3873655244217259,
        atRisk: 236,
    },
    {
        survivalFunctionEstimate: 0.3857171604880164,
        standardError: 0.02605838649852581,
        low95ConfidenceInterval: 0.3378805962839399,
        high95ConfidenceInterval: 0.44032634466500103,
        atRisk: 235,
    },
    {
        survivalFunctionEstimate: 0.3840687965543069,
        standardError: 0.025999108517988895,
        low95ConfidenceInterval: 0.336347253839118,
        high95ConfidenceInterval: 0.43856115607600643,
        atRisk: 234,
    },
    {
        survivalFunctionEstimate: 0.38242043262059744,
        standardError: 0.025939725515580526,
        low95ConfidenceInterval: 0.33481411557419427,
        high95ConfidenceInterval: 0.43679576362818306,
        atRisk: 233,
    },
    {
        survivalFunctionEstimate: 0.3807791432102086,
        standardError: 0.02588026894756829,
        low95ConfidenceInterval: 0.3332881447420192,
        high95ConfidenceInterval: 0.43503724387236103,
        atRisk: 233,
    },
    {
        survivalFunctionEstimate: 0.3791378537998198,
        standardError: 0.025820707720441035,
        low95ConfidenceInterval: 0.3317623773717209,
        high95ConfidenceInterval: 0.43327852097850994,
        atRisk: 232,
    },
    {
        survivalFunctionEstimate: 0.37749656438943097,
        standardError: 0.025761041108262436,
        low95ConfidenceInterval: 0.33023681486807255,
        high95ConfidenceInterval: 0.43151959354608327,
        atRisk: 231,
    },
    {
        survivalFunctionEstimate: 0.37749656438943097,
        atRisk: 231,
    },
    {
        survivalFunctionEstimate: 0.37749656438943097,
        atRisk: 230,
    },
    {
        survivalFunctionEstimate: 0.37584810777637667,
        standardError: 0.025701235973592626,
        low95ConfidenceInterval: 0.3287044058907873,
        high95ConfidenceInterval: 0.42975329076062757,
        atRisk: 229,
    },
    {
        survivalFunctionEstimate: 0.3741996511633224,
        standardError: 0.025641322098534116,
        low95ConfidenceInterval: 0.3271722082892681,
        high95ConfidenceInterval: 0.4279867769420966,
        atRisk: 228,
    },
    {
        survivalFunctionEstimate: 0.3725584246231324,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 228,
    },
    {
        survivalFunctionEstimate: 0.3709171980829424,
        standardError: 0.02552123216153841,
        low95ConfidenceInterval: 0.32412267812033635,
        high95ConfidenceInterval: 0.42446757700373516,
        atRisk: 227,
    },
    {
        survivalFunctionEstimate: 0.3692759715427524,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 226,
    },
    {
        survivalFunctionEstimate: 0.3676347450025624,
        standardError: 0.02540070572809008,
        low95ConfidenceInterval: 0.3210739963544177,
        high95ConfidenceInterval: 0.42094753006377944,
        atRisk: 225,
    },
    {
        survivalFunctionEstimate: 0.3676347450025624,
        atRisk: 224,
    },
    {
        survivalFunctionEstimate: 0.36598615870210244,
        standardError: 0.025340243981161175,
        low95ConfidenceInterval: 0.3195427358599843,
        high95ConfidenceInterval: 0.41917982582527646,
        atRisk: 223,
    },
    {
        survivalFunctionEstimate: 0.36598615870210244,
        atRisk: 222,
    },
    {
        survivalFunctionEstimate: 0.3643301127351246,
        standardError: 0.025279637622577452,
        low95ConfidenceInterval: 0.3180043525393298,
        high95ConfidenceInterval: 0.41740444740979493,
        atRisk: 221,
    },
    {
        survivalFunctionEstimate: 0.3626740667681468,
        standardError: 0.025218914872825696,
        low95ConfidenceInterval: 0.3164661954134745,
        high95ConfidenceInterval: 0.4156288431827427,
        atRisk: 220,
    },
    {
        survivalFunctionEstimate: 0.3610180208011689,
        standardError: 0.025158074889123805,
        low95ConfidenceInterval: 0.314928266112288,
        high95ConfidenceInterval: 0.41385301151952647,
        atRisk: 219,
    },
    {
        survivalFunctionEstimate: 0.3593619748341911,
        standardError: 0.025097116818882447,
        low95ConfidenceInterval: 0.3133905662845119,
        high95ConfidenceInterval: 0.4120769507767793,
        atRisk: 218,
    },
    {
        survivalFunctionEstimate: 0.3593619748341911,
        atRisk: 217,
    },
    {
        survivalFunctionEstimate: 0.35769826198773647,
        standardError: 0.025036010322297426,
        low95ConfidenceInterval: 0.31184554695633543,
        high95ConfidenceInterval: 0.41029300523237106,
        atRisk: 216,
    },
    {
        survivalFunctionEstimate: 0.3560345491412819,
        standardError: 0.024974781769118968,
        low95ConfidenceInterval: 0.31030076479933566,
        high95ConfidenceInterval: 0.4085088229292926,
        atRisk: 215,
    },
    {
        survivalFunctionEstimate: 0.3560345491412819,
        atRisk: 214,
    },
    {
        survivalFunctionEstimate: 0.35437083629482735,
        standardError: 0.02491343025942902,
        low95ConfidenceInterval: 0.30875622155339577,
        high95ConfidenceInterval: 0.406724402133474,
        atRisk: 214,
    },
    {
        survivalFunctionEstimate: 0.35437083629482735,
        atRisk: 214,
    },
    {
        survivalFunctionEstimate: 0.35437083629482735,
        atRisk: 213,
    },
    {
        survivalFunctionEstimate: 0.35437083629482735,
        atRisk: 212,
    },
    {
        survivalFunctionEstimate: 0.3526992757462668,
        standardError: 0.024851927287285063,
        low95ConfidenceInterval: 0.3072041866762467,
        high95ConfidenceInterval: 0.40493191338905543,
        atRisk: 212,
    },
    {
        survivalFunctionEstimate: 0.3526992757462668,
        atRisk: 211,
    },
    {
        survivalFunctionEstimate: 0.3510197553855703,
        standardError: 0.024790271725137633,
        low95ConfidenceInterval: 0.3056445520152158,
        high95ConfidenceInterval: 0.403131244638745,
        atRisk: 210,
    },
    {
        survivalFunctionEstimate: 0.3493482327408771,
        standardError: 0.024728511110459064,
        low95ConfidenceInterval: 0.3040930550034453,
        high95ConfidenceInterval: 0.4013382933648101,
        atRisk: 210,
    },
    {
        survivalFunctionEstimate: 0.3493482327408771,
        atRisk: 209,
    },
    {
        survivalFunctionEstimate: 0.3493482327408771,
        atRisk: 208,
    },
    {
        survivalFunctionEstimate: 0.3493482327408771,
        atRisk: 207,
    },
    {
        survivalFunctionEstimate: 0.34766056011894053,
        standardError: 0.024666572656349795,
        low95ConfidenceInterval: 0.30252588433805844,
        high95ConfidenceInterval: 0.3995290033667047,
        atRisk: 207,
    },
    {
        survivalFunctionEstimate: 0.34597288749700394,
        standardError: 0.02460449906665149,
        low95ConfidenceInterval: 0.30095897619812556,
        high95ConfidenceInterval: 0.39771945131889397,
        atRisk: 206,
    },
    {
        survivalFunctionEstimate: 0.34428521487506736,
        standardError: 0.02454228931598902,
        low95ConfidenceInterval: 0.2993923325650634,
        high95ConfidenceInterval: 0.39590963524696166,
        atRisk: 205,
    },
    {
        survivalFunctionEstimate: 0.34259754225313077,
        standardError: 0.024479942366303645,
        low95ConfidenceInterval: 0.29782595544466745,
        high95ConfidenceInterval: 0.3940995531522513,
        atRisk: 204,
    },
    {
        survivalFunctionEstimate: 0.34259754225313077,
        atRisk: 203,
    },
    {
        survivalFunctionEstimate: 0.34259754225313077,
        atRisk: 202,
    },
    {
        survivalFunctionEstimate: 0.34259754225313077,
        atRisk: 201,
    },
    {
        survivalFunctionEstimate: 0.34259754225313077,
        atRisk: 200,
    },
    {
        survivalFunctionEstimate: 0.3408759465634165,
        standardError: 0.02441738968347025,
        low95ConfidenceInterval: 0.29622634333194786,
        high95ConfidenceInterval: 0.392255495033056,
        atRisk: 199,
    },
    {
        survivalFunctionEstimate: 0.3408759465634165,
        atRisk: 200,
    },
    {
        survivalFunctionEstimate: 0.33916300210832395,
        standardError: 0.024354698910229296,
        low95ConfidenceInterval: 0.2946355756825521,
        high95ConfidenceInterval: 0.3904197303148106,
        atRisk: 199,
    },
    {
        survivalFunctionEstimate: 0.33916300210832395,
        atRisk: 199,
    },
    {
        survivalFunctionEstimate: 0.33916300210832395,
        atRisk: 198,
    },
    {
        survivalFunctionEstimate: 0.33916300210832395,
        atRisk: 197,
    },
    {
        survivalFunctionEstimate: 0.33916300210832395,
        atRisk: 196,
    },
    {
        survivalFunctionEstimate: 0.3374237046616146,
        standardError: 0.02429182982607976,
        low95ConfidenceInterval: 0.2930190332316098,
        high95ConfidenceInterval: 0.38855754594471953,
        atRisk: 195,
    },
    {
        survivalFunctionEstimate: 0.3374237046616146,
        atRisk: 194,
    },
    {
        survivalFunctionEstimate: 0.33567539531103624,
        standardError: 0.02422879760062402,
        low95ConfidenceInterval: 0.29139387698546426,
        high95ConfidenceInterval: 0.3866861314414003,
        atRisk: 193,
    },
    {
        survivalFunctionEstimate: 0.33567539531103624,
        atRisk: 192,
    },
    {
        survivalFunctionEstimate: 0.33567539531103624,
        atRisk: 191,
    },
    {
        survivalFunctionEstimate: 0.33567539531103624,
        atRisk: 190,
    },
    {
        survivalFunctionEstimate: 0.33389933501838526,
        standardError: 0.024165611116212623,
        low95ConfidenceInterval: 0.289741525913494,
        high95ConfidenceInterval: 0.3847869772005213,
        atRisk: 189,
    },
    {
        survivalFunctionEstimate: 0.3321232747257343,
        standardError: 0.024102248360646082,
        low95ConfidenceInterval: 0.2880895171626195,
        high95ConfidenceInterval: 0.3828874812972824,
        atRisk: 188,
    },
    {
        survivalFunctionEstimate: 0.3303472144330833,
        standardError: 0.02403870794004426,
        low95ConfidenceInterval: 0.2864378534232696,
        high95ConfidenceInterval: 0.3809876410518864,
        atRisk: 187,
    },
    {
        survivalFunctionEstimate: 0.3303472144330833,
        atRisk: 186,
    },
    {
        survivalFunctionEstimate: 0.3285615538145261,
        standardError: 0.02397499910968822,
        low95ConfidenceInterval: 0.2847770082760821,
        high95ConfidenceInterval: 0.37907798560886313,
        atRisk: 185,
    },
    {
        survivalFunctionEstimate: 0.3285615538145261,
        atRisk: 184,
    },
    {
        survivalFunctionEstimate: 0.3267661354876708,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 183,
    },
    {
        survivalFunctionEstimate: 0.3249707171608155,
        standardError: 0.023847051912870152,
        low95ConfidenceInterval: 0.2814370204283803,
        high95ConfidenceInterval: 0.37523836363556584,
        atRisk: 182,
    },
    {
        survivalFunctionEstimate: 0.3231752988339602,
        standardError: 0.02378279174382621,
        low95ConfidenceInterval: 0.27976758290075315,
        high95ConfidenceInterval: 0.3733179973659424,
        atRisk: 181,
    },
    {
        survivalFunctionEstimate: 0.32137988050710486,
        standardError: 0.023718338458813347,
        low95ConfidenceInterval: 0.2780985202951502,
        high95ConfidenceInterval: 0.3713972569330576,
        atRisk: 180,
    },
    {
        survivalFunctionEstimate: 0.32137988050710486,
        atRisk: 179,
    },
    {
        survivalFunctionEstimate: 0.32137988050710486,
        atRisk: 178,
    },
    {
        survivalFunctionEstimate: 0.32137988050710486,
        atRisk: 177,
    },
    {
        survivalFunctionEstimate: 0.32137988050710486,
        atRisk: 176,
    },
    {
        survivalFunctionEstimate: 0.32137988050710486,
        atRisk: 175,
    },
    {
        survivalFunctionEstimate: 0.31953286969959277,
        standardError: 0.02365383269412575,
        low95ConfidenceInterval: 0.2763784952591751,
        high95ConfidenceInterval: 0.36942546750140975,
        atRisk: 174,
    },
    {
        survivalFunctionEstimate: 0.3176858588920807,
        standardError: 0.02358910708400764,
        low95ConfidenceInterval: 0.27465889699270274,
        high95ConfidenceInterval: 0.3674532521794857,
        atRisk: 173,
    },
    {
        survivalFunctionEstimate: 0.3158388480845686,
        standardError: 0.023524159813776264,
        low95ConfidenceInterval: 0.2729397289941896,
        high95ConfidenceInterval: 0.3654806074842654,
        atRisk: 172,
    },
    {
        survivalFunctionEstimate: 0.3158388480845686,
        atRisk: 171,
    },
    {
        survivalFunctionEstimate: 0.3139918372770565,
        standardError: 0.023458989042403598,
        low95ConfidenceInterval: 0.2712209948125566,
        high95ConfidenceInterval: 0.3635075298826279,
        atRisk: 171,
    },
    {
        survivalFunctionEstimate: 0.3139918372770565,
        atRisk: 170,
    },
    {
        survivalFunctionEstimate: 0.3139918372770565,
        atRisk: 169,
    },
    {
        survivalFunctionEstimate: 0.3139918372770565,
        atRisk: 168,
    },
    {
        survivalFunctionEstimate: 0.3139918372770565,
        atRisk: 168,
    },
    {
        survivalFunctionEstimate: 0.3121116466346789,
        standardError: 0.023393741459070253,
        low95ConfidenceInterval: 0.2694695916145503,
        high95ConfidenceInterval: 0.36150156825246293,
        atRisk: 167,
    },
    {
        survivalFunctionEstimate: 0.3121116466346789,
        atRisk: 166,
    },
    {
        survivalFunctionEstimate: 0.3102314559923013,
        standardError: 0.023328249840228937,
        low95ConfidenceInterval: 0.26771866203480954,
        high95ConfidenceInterval: 0.35949513401717703,
        atRisk: 166,
    },
    {
        survivalFunctionEstimate: 0.3102314559923013,
        atRisk: 165,
    },
    {
        survivalFunctionEstimate: 0.3102314559923013,
        atRisk: 164,
    },
    {
        survivalFunctionEstimate: 0.3102314559923013,
        atRisk: 163,
    },
    {
        survivalFunctionEstimate: 0.30831644700469446,
        standardError: 0.023262716762285807,
        low95ConfidenceInterval: 0.26593339089364554,
        high95ConfidenceInterval: 0.35745429024223374,
        atRisk: 162,
    },
    {
        survivalFunctionEstimate: 0.30831644700469446,
        atRisk: 161,
    },
    {
        survivalFunctionEstimate: 0.30831644700469446,
        atRisk: 161,
    },
    {
        survivalFunctionEstimate: 0.30638946921091514,
        standardError: 0.023196998528838636,
        low95ConfidenceInterval: 0.26413665023012134,
        high95ConfidenceInterval: 0.3554012923294094,
        atRisk: 160,
    },
    {
        survivalFunctionEstimate: 0.3044624914171358,
        standardError: 0.023131004138264046,
        low95ConfidenceInterval: 0.26234044542333335,
        high95ConfidenceInterval: 0.3533477597415283,
        atRisk: 159,
    },
    {
        survivalFunctionEstimate: 0.30253551362335646,
        standardError: 0.023064731220083033,
        low95ConfidenceInterval: 0.2605447810369972,
        high95ConfidenceInterval: 0.35129368793747284,
        atRisk: 158,
    },
    {
        survivalFunctionEstimate: 0.30060853582957714,
        standardError: 0.022998177366430495,
        low95ConfidenceInterval: 0.25874966170630775,
        high95ConfidenceInterval: 0.3492390723052267,
        atRisk: 157,
    },
    {
        survivalFunctionEstimate: 0.2986815580357978,
        standardError: 0.022931340131221223,
        low95ConfidenceInterval: 0.25695509213952183,
        high95ConfidenceInterval: 0.3471839081603097,
        atRisk: 156,
    },
    {
        survivalFunctionEstimate: 0.2986815580357978,
        atRisk: 155,
    },
    {
        survivalFunctionEstimate: 0.2967420673992017,
        standardError: 0.022864307493247703,
        low95ConfidenceInterval: 0.25514854091900196,
        high95ConfidenceInterval: 0.34511604200122037,
        atRisk: 154,
    },
    {
        survivalFunctionEstimate: 0.2967420673992017,
        atRisk: 153,
    },
    {
        survivalFunctionEstimate: 0.29478981695578593,
        standardError: 0.02279707772740043,
        low95ConfidenceInterval: 0.2533297700731015,
        high95ConfidenceInterval: 0.34303523093929866,
        atRisk: 152,
    },
    {
        survivalFunctionEstimate: 0.29478981695578593,
        atRisk: 151,
    },
    {
        survivalFunctionEstimate: 0.29478981695578593,
        atRisk: 150,
    },
    {
        survivalFunctionEstimate: 0.29281136180843165,
        standardError: 0.022729765658757305,
        low95ConfidenceInterval: 0.25148528579919777,
        high95ConfidenceInterval: 0.3409284695589206,
        atRisk: 149,
    },
    {
        survivalFunctionEstimate: 0.29281136180843165,
        atRisk: 149,
    },
    {
        survivalFunctionEstimate: 0.29281136180843165,
        atRisk: 148,
    },
    {
        survivalFunctionEstimate: 0.29281136180843165,
        atRisk: 147,
    },
    {
        survivalFunctionEstimate: 0.29080580453577115,
        standardError: 0.02266238962234061,
        low95ConfidenceInterval: 0.2496141765667062,
        high95ConfidenceInterval: 0.3387949238896591,
        atRisk: 146,
    },
    {
        survivalFunctionEstimate: 0.29080580453577115,
        atRisk: 145,
    },
    {
        survivalFunctionEstimate: 0.28878631978205055,
        standardError: 0.022594812643187162,
        low95ConfidenceInterval: 0.24772971528663135,
        high95ConfidenceInterval: 0.3366472948017847,
        atRisk: 144,
    },
    {
        survivalFunctionEstimate: 0.28676683502832995,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 143,
    },
    {
        survivalFunctionEstimate: 0.2847473502746093,
        standardError: 0.022458573972128602,
        low95ConfidenceInterval: 0.24396289608321767,
        high95ConfidenceInterval: 0.33234993841339566,
        atRisk: 142,
    },
    {
        survivalFunctionEstimate: 0.2847473502746093,
        atRisk: 141,
    },
    {
        survivalFunctionEstimate: 0.2847473502746093,
        atRisk: 141,
    },
    {
        survivalFunctionEstimate: 0.28271344062979065,
        standardError: 0.022390064370591344,
        low95ConfidenceInterval: 0.2420660164312152,
        high95ConfidenceInterval: 0.3301863297091352,
        atRisk: 140,
    },
    {
        survivalFunctionEstimate: 0.28271344062979065,
        atRisk: 139,
    },
    {
        survivalFunctionEstimate: 0.28271344062979065,
        atRisk: 138,
    },
    {
        survivalFunctionEstimate: 0.2806498388733688,
        standardError: 0.02232152759043784,
        low95ConfidenceInterval: 0.24013991529373785,
        high95ConfidenceInterval: 0.32799350313463616,
        atRisk: 137,
    },
    {
        survivalFunctionEstimate: 0.27858623711694697,
        standardError: 0.022252583169218954,
        low95ConfidenceInterval: 0.23821460436255598,
        high95ConfidenceInterval: 0.32579988837652923,
        atRisk: 136,
    },
    {
        survivalFunctionEstimate: 0.27858623711694697,
        atRisk: 135,
    },
    {
        survivalFunctionEstimate: 0.27858623711694697,
        atRisk: 134,
    },
    {
        survivalFunctionEstimate: 0.27858623711694697,
        atRisk: 133,
    },
    {
        survivalFunctionEstimate: 0.27858623711694697,
        atRisk: 133,
    },
    {
        survivalFunctionEstimate: 0.2765072353474175,
        standardError: 0.02218342442955763,
        low95ConfidenceInterval: 0.2362745323248795,
        high95ConfidenceInterval: 0.32359074186778686,
        atRisk: 134,
    },
    {
        survivalFunctionEstimate: 0.27444374851646663,
        standardError: 0.022113640290601748,
        low95ConfidenceInterval: 0.23435096080057485,
        high95ConfidenceInterval: 0.3213956146903274,
        atRisk: 134,
    },
    {
        survivalFunctionEstimate: 0.27444374851646663,
        atRisk: 133,
    },
    {
        survivalFunctionEstimate: 0.27444374851646663,
        atRisk: 132,
    },
    {
        survivalFunctionEstimate: 0.27444374851646663,
        atRisk: 131,
    },
    {
        survivalFunctionEstimate: 0.27444374851646663,
        atRisk: 130,
    },
    {
        survivalFunctionEstimate: 0.27231627759773436,
        standardError: 0.022044317202100407,
        low95ConfidenceInterval: 0.23236347022990414,
        high95ConfidenceInterval: 0.3191386106056819,
        atRisk: 129,
    },
    {
        survivalFunctionEstimate: 0.27231627759773436,
        atRisk: 128,
    },
    {
        survivalFunctionEstimate: 0.27231627759773436,
        atRisk: 127,
    },
    {
        survivalFunctionEstimate: 0.27015503729933965,
        standardError: 0.021975051674307373,
        low95ConfidenceInterval: 0.2303426276603091,
        high95ConfidenceInterval: 0.31684862207024134,
        atRisk: 126,
    },
    {
        survivalFunctionEstimate: 0.27015503729933965,
        atRisk: 126,
    },
    {
        survivalFunctionEstimate: 0.26799379700094494,
        standardError: 0.02190527213907189,
        low95ConfidenceInterval: 0.22832278100610376,
        high95ConfidenceInterval: 0.3145576403480461,
        atRisk: 125,
    },
    {
        survivalFunctionEstimate: 0.26799379700094494,
        atRisk: 124,
    },
    {
        survivalFunctionEstimate: 0.26799379700094494,
        atRisk: 123,
    },
    {
        survivalFunctionEstimate: 0.26799379700094494,
        atRisk: 122,
    },
    {
        survivalFunctionEstimate: 0.26799379700094494,
        atRisk: 121,
    },
    {
        survivalFunctionEstimate: 0.2657789722323421,
        standardError: 0.02183591934853226,
        low95ConfidenceInterval: 0.22624940168870578,
        high95ConfidenceInterval: 0.3122150226857652,
        atRisk: 121,
    },
    {
        survivalFunctionEstimate: 0.26356414746373924,
        standardError: 0.021765988740958045,
        low95ConfidenceInterval: 0.22417714167378003,
        high95ConfidenceInterval: 0.3098712888817803,
        atRisk: 120,
    },
    {
        survivalFunctionEstimate: 0.2613677795682081,
        standardError: 0.02169513831068251,
        low95ConfidenceInterval: 0.22212480955988761,
        high95ConfidenceInterval: 0.3075438368715736,
        atRisk: 120,
    },
    {
        survivalFunctionEstimate: 0.259171411672677,
        standardError: 0.021623718135295688,
        low95ConfidenceInterval: 0.2200735808566425,
        high95ConfidenceInterval: 0.30521528466500986,
        atRisk: 119,
    },
    {
        survivalFunctionEstimate: 0.259171411672677,
        atRisk: 119,
    },
    {
        survivalFunctionEstimate: 0.259171411672677,
        atRisk: 118,
    },
    {
        survivalFunctionEstimate: 0.25695627140197036,
        standardError: 0.021552061203618106,
        low95ConfidenceInterval: 0.21800435837154475,
        high95ConfidenceInterval: 0.3028679146876232,
        atRisk: 117,
    },
    {
        survivalFunctionEstimate: 0.25474113113126373,
        standardError: 0.021479802978627366,
        low95ConfidenceInterval: 0.21593630011714857,
        high95ConfidenceInterval: 0.3005193839795824,
        atRisk: 116,
    },
    {
        survivalFunctionEstimate: 0.25254508689737354,
        standardError: 0.021406597097147703,
        low95ConfidenceInterval: 0.2138888425094751,
        high95ConfidenceInterval: 0.29818769491529973,
        atRisk: 116,
    },
    {
        survivalFunctionEstimate: 0.25034904266348335,
        standardError: 0.021332798468394962,
        low95ConfidenceInterval: 0.21184253230196545,
        high95ConfidenceInterval: 0.2958548620122453,
        atRisk: 115,
    },
    {
        survivalFunctionEstimate: 0.24815299842959312,
        standardError: 0.02125840091919604,
        low95ConfidenceInterval: 0.20979738129862677,
        high95ConfidenceInterval: 0.29352087356106915,
        atRisk: 114,
    },
    {
        survivalFunctionEstimate: 0.24815299842959312,
        atRisk: 113,
    },
    {
        survivalFunctionEstimate: 0.24593734665790035,
        standardError: 0.021183741991818134,
        low95ConfidenceInterval: 0.20773347461876418,
        high95ConfidenceInterval: 0.291167220844554,
        atRisk: 112,
    },
    {
        survivalFunctionEstimate: 0.24593734665790035,
        atRisk: 111,
    },
    {
        survivalFunctionEstimate: 0.243701552597374,
        standardError: 0.021108818905312174,
        low95ConfidenceInterval: 0.20565030005772872,
        high95ConfidenceInterval: 0.28879338722918946,
        atRisk: 110,
    },
    {
        survivalFunctionEstimate: 0.24146575853684765,
        standardError: 0.02103323164103721,
        low95ConfidenceInterval: 0.2035684104954722,
        high95ConfidenceInterval: 0.2864182728737869,
        atRisk: 109,
    },
    {
        survivalFunctionEstimate: 0.24146575853684765,
        atRisk: 109,
    },
    {
        survivalFunctionEstimate: 0.24146575853684765,
        atRisk: 108,
    },
    {
        survivalFunctionEstimate: 0.24146575853684765,
        atRisk: 109,
    },
    {
        survivalFunctionEstimate: 0.24146575853684765,
        atRisk: 109,
    },
    {
        survivalFunctionEstimate: 0.23922996447632128,
        standardError: 0.020956973012358233,
        low95ConfidenceInterval: 0.2014878196550264,
        high95ConfidenceInterval: 0.28404186417486127,
        atRisk: 108,
    },
    {
        survivalFunctionEstimate: 0.2369941704157949,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 107,
    },
    {
        survivalFunctionEstimate: 0.23475837635526853,
        standardError: 0.020802412063312077,
        low95ConfidenceInterval: 0.19733059064355965,
        high95ConfidenceInterval: 0.2792851077434334,
        atRisk: 106,
    },
    {
        survivalFunctionEstimate: 0.23475837635526853,
        atRisk: 105,
    },
    {
        survivalFunctionEstimate: 0.23475837635526853,
        atRisk: 105,
    },
    {
        survivalFunctionEstimate: 0.23252258229474218,
        standardError: 0.020724094501033567,
        low95ConfidenceInterval: 0.19525398156175333,
        high95ConfidenceInterval: 0.2769047311842671,
        atRisk: 105,
    },
    {
        survivalFunctionEstimate: 0.23252258229474218,
        atRisk: 104,
    },
    {
        survivalFunctionEstimate: 0.23252258229474218,
        atRisk: 103,
    },
    {
        survivalFunctionEstimate: 0.23024294913498983,
        standardError: 0.02064591528180684,
        low95ConfidenceInterval: 0.1931341115074957,
        high95ConfidenceInterval: 0.27448188832411446,
        atRisk: 102,
    },
    {
        survivalFunctionEstimate: 0.23024294913498983,
        atRisk: 101,
    },
    {
        survivalFunctionEstimate: 0.22794051964363993,
        standardError: 0.020567438746617626,
        low95ConfidenceInterval: 0.19099247353423704,
        high95ConfidenceInterval: 0.2720362720790612,
        atRisk: 100,
    },
    {
        survivalFunctionEstimate: 0.22794051964363993,
        atRisk: 99,
    },
    {
        survivalFunctionEstimate: 0.22794051964363993,
        atRisk: 98,
    },
    {
        survivalFunctionEstimate: 0.22559061737927252,
        standardError: 0.020489205834422562,
        low95ConfidenceInterval: 0.18880386762474724,
        high95ConfidenceInterval: 0.2695449372398918,
        atRisk: 97,
    },
    {
        survivalFunctionEstimate: 0.22326494091144497,
        standardError: 0.0204095408571633,
        low95ConfidenceInterval: 0.18664174894791935,
        high95ConfidenceInterval: 0.26707440388431214,
        atRisk: 97,
    },
    {
        survivalFunctionEstimate: 0.22326494091144497,
        atRisk: 96,
    },
    {
        survivalFunctionEstimate: 0.22093926444361742,
        standardError: 0.020329015582200235,
        low95ConfidenceInterval: 0.1844812923842204,
        high95ConfidenceInterval: 0.2646022149022088,
        atRisk: 96,
    },
    {
        survivalFunctionEstimate: 0.22093926444361742,
        atRisk: 95,
    },
    {
        survivalFunctionEstimate: 0.22093926444361742,
        atRisk: 94,
    },
    {
        survivalFunctionEstimate: 0.22093926444361742,
        atRisk: 93,
    },
    {
        survivalFunctionEstimate: 0.21853775069966505,
        standardError: 0.020249399393557552,
        low95ConfidenceInterval: 0.1822448798610344,
        high95ConfidenceInterval: 0.26205810839396976,
        atRisk: 92,
    },
    {
        survivalFunctionEstimate: 0.21853775069966505,
        atRisk: 91,
    },
    {
        survivalFunctionEstimate: 0.21853775069966505,
        atRisk: 90,
    },
    {
        survivalFunctionEstimate: 0.21853775069966505,
        atRisk: 89,
    },
    {
        survivalFunctionEstimate: 0.21605436716898704,
        standardError: 0.020170998674026542,
        low95ConfidenceInterval: 0.17992612680386572,
        high95ConfidenceInterval: 0.2594369722840527,
        atRisk: 88,
    },
    {
        survivalFunctionEstimate: 0.21605436716898704,
        atRisk: 87,
    },
    {
        survivalFunctionEstimate: 0.21354210708562674,
        standardError: 0.020092291735959132,
        low95ConfidenceInterval: 0.17757977958526802,
        high95ConfidenceInterval: 0.2567872964200495,
        atRisk: 86,
    },
    {
        survivalFunctionEstimate: 0.21354210708562674,
        atRisk: 85,
    },
    {
        survivalFunctionEstimate: 0.21354210708562674,
        atRisk: 84,
    },
    {
        survivalFunctionEstimate: 0.21096931061471555,
        standardError: 0.02001426001236488,
        low95ConfidenceInterval: 0.1751730941321834,
        high95ConfidenceInterval: 0.2540804011126453,
        atRisk: 83,
    },
    {
        survivalFunctionEstimate: 0.21096931061471555,
        atRisk: 82,
    },
    {
        survivalFunctionEstimate: 0.21096931061471555,
        atRisk: 81,
    },
    {
        survivalFunctionEstimate: 0.20833219423203161,
        standardError: 0.01993706048879716,
        low95ConfidenceInterval: 0.1727021525790966,
        high95ConfidenceInterval: 0.2513130410094625,
        atRisk: 80,
    },
    {
        survivalFunctionEstimate: 0.20833219423203161,
        atRisk: 79,
    },
    {
        survivalFunctionEstimate: 0.20833219423203161,
        atRisk: 79,
    },
    {
        survivalFunctionEstimate: 0.20833219423203161,
        atRisk: 78,
    },
    {
        survivalFunctionEstimate: 0.2056612686649543,
        standardError: 0.01985956040223392,
        low95ConfidenceInterval: 0.17019889266416158,
        high95ConfidenceInterval: 0.24851253005704674,
        atRisk: 78,
    },
    {
        survivalFunctionEstimate: 0.20299034309787697,
        standardError: 0.019780435912266373,
        low95ConfidenceInterval: 0.1676987662049021,
        high95ConfidenceInterval: 0.2457089000920111,
        atRisk: 77,
    },
    {
        survivalFunctionEstimate: 0.20299034309787697,
        atRisk: 76,
    },
    {
        survivalFunctionEstimate: 0.20299034309787697,
        atRisk: 75,
    },
    {
        survivalFunctionEstimate: 0.20299034309787697,
        atRisk: 74,
    },
    {
        survivalFunctionEstimate: 0.20299034309787697,
        atRisk: 73,
    },
    {
        survivalFunctionEstimate: 0.20299034309787697,
        atRisk: 72,
    },
    {
        survivalFunctionEstimate: 0.20299034309787697,
        atRisk: 74,
    },
    {
        survivalFunctionEstimate: 0.20020965346639918,
        standardError: 0.019703953256532104,
        low95ConfidenceInterval: 0.16508691529454364,
        high95ConfidenceInterval: 0.24280485991042372,
        atRisk: 73,
    },
    {
        survivalFunctionEstimate: 0.20020965346639918,
        atRisk: 72,
    },
    {
        survivalFunctionEstimate: 0.20020965346639918,
        atRisk: 73,
    },
    {
        survivalFunctionEstimate: 0.20020965346639918,
        atRisk: 72,
    },
    {
        survivalFunctionEstimate: 0.19746705547370877,
        standardError: 0.019623979298001238,
        low95ConfidenceInterval: 0.16251876736370924,
        high95ConfidenceInterval: 0.2399306777302326,
        atRisk: 73,
    },
    {
        survivalFunctionEstimate: 0.19483416140072599,
        standardError: 0.019538151914514402,
        low95ConfidenceInterval: 0.16006847957864145,
        high95ConfidenceInterval: 0.2371506904335545,
        atRisk: 75,
    },
    {
        survivalFunctionEstimate: 0.19483416140072599,
        atRisk: 74,
    },
    {
        survivalFunctionEstimate: 0.19483416140072599,
        atRisk: 73,
    },
    {
        survivalFunctionEstimate: 0.19212813138127147,
        standardError: 0.0194532784452622,
        low95ConfidenceInterval: 0.1575452997058335,
        high95ConfidenceInterval: 0.2343022542531131,
        atRisk: 72,
    },
    {
        survivalFunctionEstimate: 0.19212813138127147,
        atRisk: 73,
    },
    {
        survivalFunctionEstimate: 0.19212813138127147,
        atRisk: 72,
    },
    {
        survivalFunctionEstimate: 0.18942210136181695,
        standardError: 0.019366583973062318,
        low95ConfidenceInterval: 0.15502562728999333,
        high95ConfidenceInterval: 0.23145032928786288,
        atRisk: 71,
    },
    {
        survivalFunctionEstimate: 0.18942210136181695,
        atRisk: 70,
    },
    {
        survivalFunctionEstimate: 0.18667685351599353,
        standardError: 0.019279498933253477,
        low95ConfidenceInterval: 0.1524686640831023,
        high95ConfidenceInterval: 0.22856006411676716,
        atRisk: 69,
    },
    {
        survivalFunctionEstimate: 0.1839316056701701,
        standardError: 0.019190432369878673,
        low95ConfidenceInterval: 0.14991551498391137,
        high95ConfidenceInterval: 0.2256660064039244,
        atRisk: 68,
    },
    {
        survivalFunctionEstimate: 0.1839316056701701,
        atRisk: 67,
    },
    {
        survivalFunctionEstimate: 0.18114476316001601,
        standardError: 0.019100949629939534,
        low95ConfidenceInterval: 0.1473228725476331,
        high95ConfidenceInterval: 0.22273136990109194,
        atRisk: 66,
    },
    {
        survivalFunctionEstimate: 0.17835792064986195,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 65,
    },
    {
        survivalFunctionEstimate: 0.17557107813970785,
        standardError: 0.018915460851455537,
        low95ConfidenceInterval: 0.14215013327899456,
        high95ConfidenceInterval: 0.2168496277005913,
        atRisk: 64,
    },
    {
        survivalFunctionEstimate: 0.17282778004377491,
        standardError: 0.018817784778174064,
        low95ConfidenceInterval: 0.1396153820917092,
        high95ConfidenceInterval: 0.21394090756589484,
        atRisk: 64,
    },
    {
        survivalFunctionEstimate: 0.17282778004377491,
        atRisk: 64,
    },
    {
        survivalFunctionEstimate: 0.17282778004377491,
        atRisk: 63,
    },
    {
        survivalFunctionEstimate: 0.17016889112002453,
        standardError: 0.01871518368361226,
        low95ConfidenceInterval: 0.137171844324068,
        high95ConfidenceInterval: 0.2111034640360079,
        atRisk: 65,
    },
    {
        survivalFunctionEstimate: 0.17016889112002453,
        atRisk: 64,
    },
    {
        survivalFunctionEstimate: 0.17016889112002453,
        atRisk: 63,
    },
    {
        survivalFunctionEstimate: 0.16742423158583059,
        standardError: 0.018613495735342708,
        low95ConfidenceInterval: 0.13464351471384414,
        high95ConfidenceInterval: 0.2081858408232987,
        atRisk: 62,
    },
    {
        survivalFunctionEstimate: 0.16742423158583059,
        atRisk: 61,
    },
    {
        survivalFunctionEstimate: 0.16742423158583059,
        atRisk: 61,
    },
    {
        survivalFunctionEstimate: 0.16742423158583059,
        atRisk: 60,
    },
    {
        survivalFunctionEstimate: 0.16742423158583059,
        atRisk: 59,
    },
    {
        survivalFunctionEstimate: 0.16742423158583059,
        atRisk: 58,
    },
    {
        survivalFunctionEstimate: 0.16742423158583059,
        atRisk: 60,
    },
    {
        survivalFunctionEstimate: 0.16458653274539278,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 59,
    },
    {
        survivalFunctionEstimate: 0.16174883390495498,
        standardError: 0.018410065510089793,
        low95ConfidenceInterval: 0.12940718247098093,
        high95ConfidenceInterval: 0.20217336294667884,
        atRisk: 58,
    },
    {
        survivalFunctionEstimate: 0.15891113506451718,
        standardError: 0.01830447469871154,
        low95ConfidenceInterval: 0.12679645060418224,
        high95ConfidenceInterval: 0.1991597456172034,
        atRisk: 57,
    },
    {
        survivalFunctionEstimate: 0.15891113506451718,
        atRisk: 56,
    },
    {
        survivalFunctionEstimate: 0.1560218416997078,
        standardError: 0.018198269568549513,
        low95ConfidenceInterval: 0.12413718591785192,
        high95ConfidenceInterval: 0.19609607634796547,
        atRisk: 55,
    },
    {
        survivalFunctionEstimate: 0.1560218416997078,
        atRisk: 54,
    },
    {
        survivalFunctionEstimate: 0.1560218416997078,
        atRisk: 53,
    },
    {
        survivalFunctionEstimate: 0.1530214216670211,
        standardError: 0.018093957816521587,
        low95ConfidenceInterval: 0.12136745379786083,
        high95ConfidenceInterval: 0.1929310927787543,
        atRisk: 52,
    },
    {
        survivalFunctionEstimate: 0.1500210016343344,
        standardError: 0.01798622545929652,
        low95ConfidenceInterval: 0.11860427141544422,
        high95ConfidenceInterval: 0.18975961542341457,
        atRisk: 51,
    },
    {
        survivalFunctionEstimate: 0.1500210016343344,
        atRisk: 50,
    },
    {
        survivalFunctionEstimate: 0.14695934853975615,
        standardError: 0.017877841185787505,
        low95ConfidenceInterval: 0.1157836352393814,
        high95ConfidenceInterval: 0.18652938369552707,
        atRisk: 49,
    },
    {
        survivalFunctionEstimate: 0.14695934853975615,
        atRisk: 48,
    },
    {
        survivalFunctionEstimate: 0.1438325538899741,
        standardError: 0.017768792552239284,
        low95ConfidenceInterval: 0.11290194327217636,
        high95ConfidenceInterval: 0.18323691301432737,
        atRisk: 47,
    },
    {
        survivalFunctionEstimate: 0.1407722867859321,
        standardError: 0.017652296548891496,
        low95ConfidenceInterval: 0.11009809560660297,
        high95ConfidenceInterval: 0.17999254771625883,
        atRisk: 47,
    },
    {
        survivalFunctionEstimate: 0.1407722867859321,
        atRisk: 47,
    },
    {
        survivalFunctionEstimate: 0.1407722867859321,
        atRisk: 47,
    },
    {
        survivalFunctionEstimate: 0.1377120196818901,
        standardError: 0.017531813806676743,
        low95ConfidenceInterval: 0.10730186110045209,
        high95ConfidenceInterval: 0.17674064709009393,
        atRisk: 46,
    },
    {
        survivalFunctionEstimate: 0.1377120196818901,
        atRisk: 45,
    },
    {
        survivalFunctionEstimate: 0.13458220105275623,
        standardError: 0.017410493336373924,
        low95ConfidenceInterval: 0.10444081473986605,
        high95ConfidenceInterval: 0.17342232426391477,
        atRisk: 44,
    },
    {
        survivalFunctionEstimate: 0.13458220105275623,
        atRisk: 44,
    },
    {
        survivalFunctionEstimate: 0.13458220105275623,
        atRisk: 43,
    },
    {
        survivalFunctionEstimate: 0.13458220105275623,
        atRisk: 42,
    },
    {
        survivalFunctionEstimate: 0.13458220105275623,
        atRisk: 41,
    },
    {
        survivalFunctionEstimate: 0.13458220105275623,
        atRisk: 40,
    },
    {
        survivalFunctionEstimate: 0.13458220105275623,
        atRisk: 40,
    },
    {
        survivalFunctionEstimate: 0.13458220105275623,
        atRisk: 39,
    },
    {
        survivalFunctionEstimate: 0.13104056418294685,
        standardError: 0.01730879384828309,
        low95ConfidenceInterval: 0.10115162023860948,
        high95ConfidenceInterval: 0.1697612892495282,
        atRisk: 38,
    },
    {
        survivalFunctionEstimate: 0.13104056418294685,
        atRisk: 37,
    },
    {
        survivalFunctionEstimate: 0.13104056418294685,
        atRisk: 36,
    },
    {
        survivalFunctionEstimate: 0.12729654806343407,
        standardError: 0.017214423797866696,
        low95ConfidenceInterval: 0.09765801576896116,
        high95ConfidenceInterval: 0.1659301699023099,
        atRisk: 35,
    },
    {
        survivalFunctionEstimate: 0.12729654806343407,
        atRisk: 34,
    },
    {
        survivalFunctionEstimate: 0.12343907690999668,
        standardError: 0.017119517973568827,
        low95ConfidenceInterval: 0.09405918919561472,
        high95ConfidenceInterval: 0.1619959287199817,
        atRisk: 33,
    },
    {
        survivalFunctionEstimate: 0.12343907690999668,
        atRisk: 32,
    },
    {
        survivalFunctionEstimate: 0.11958160575655928,
        standardError: 0.01701357743497364,
        low95ConfidenceInterval: 0.09048132206526299,
        high95ConfidenceInterval: 0.15804102005718865,
        atRisk: 32,
    },
    {
        survivalFunctionEstimate: 0.11572413460312189,
        standardError: 0.016896394620097312,
        low95ConfidenceInterval: 0.08692477619828849,
        high95ConfidenceInterval: 0.15406511141417423,
        atRisk: 31,
    },
    {
        survivalFunctionEstimate: 0.1118666634496845,
        standardError: 0.01676773382762184,
        low95ConfidenceInterval: 0.08338996354663711,
        high95ConfidenceInterval: 0.15006782422163126,
        atRisk: 30,
    },
    {
        survivalFunctionEstimate: 0.1118666634496845,
        atRisk: 29,
    },
    {
        survivalFunctionEstimate: 0.10787142546933863,
        standardError: 0.016638050918610765,
        low95ConfidenceInterval: 0.07972919690981095,
        high95ConfidenceInterval: 0.14594709195365296,
        atRisk: 28,
    },
    {
        survivalFunctionEstimate: 0.10415172114280971,
        standardError: 0.016474878707503363,
        low95ConfidenceInterval: 0.07638746327053812,
        high95ConfidenceInterval: 0.142007347181974,
        atRisk: 29,
    },
    {
        survivalFunctionEstimate: 0.10415172114280971,
        atRisk: 30,
    },
    {
        survivalFunctionEstimate: 0.10067999710471605,
        standardError: 0.016287404284366914,
        low95ConfidenceInterval: 0.0733231187628414,
        high95ConfidenceInterval: 0.13824373523705835,
        atRisk: 30,
    },
    {
        survivalFunctionEstimate: 0.0972082730666224,
        standardError: undefined,
        low95ConfidenceInterval: undefined,
        high95ConfidenceInterval: undefined,
        atRisk: 29,
    },
    {
        survivalFunctionEstimate: 0.09373654902852875,
        standardError: 0.015886921101234703,
        low95ConfidenceInterval: 0.06724240370962643,
        high95ConfidenceInterval: 0.13066963908251675,
        atRisk: 28,
    },
    {
        survivalFunctionEstimate: 0.09373654902852875,
        atRisk: 27,
    },
    {
        survivalFunctionEstimate: 0.09373654902852875,
        atRisk: 26,
    },
    {
        survivalFunctionEstimate: 0.09373654902852875,
        atRisk: 25,
    },
    {
        survivalFunctionEstimate: 0.09373654902852875,
        atRisk: 24,
    },
    {
        survivalFunctionEstimate: 0.09373654902852875,
        atRisk: 23,
    },
    {
        survivalFunctionEstimate: 0.08966104689685359,
        standardError: 0.015710238914048182,
        low95ConfidenceInterval: 0.06360011227566292,
        high95ConfidenceInterval: 0.1264007726243591,
        atRisk: 23,
    },
    {
        survivalFunctionEstimate: 0.08558554476517842,
        standardError: 0.01551576165513586,
        low95ConfidenceInterval: 0.05999105602069649,
        high95ConfidenceInterval: 0.12209962548792813,
        atRisk: 22,
    },
    {
        survivalFunctionEstimate: 0.08151004263350325,
        standardError: 0.015302810888342955,
        low95ConfidenceInterval: 0.0564163956795666,
        high95ConfidenceInterval: 0.11776518102736336,
        atRisk: 21,
    },
    {
        survivalFunctionEstimate: 0.07743454050182809,
        standardError: 0.015070603530472303,
        low95ConfidenceInterval: 0.052877468791622584,
        high95ConfidenceInterval: 0.11339627633006565,
        atRisk: 20,
    },
    {
        survivalFunctionEstimate: 0.07743454050182809,
        atRisk: 19,
    },
    {
        survivalFunctionEstimate: 0.07335903837015292,
        standardError: 0.014818234334895432,
        low95ConfidenceInterval: 0.04937581752225301,
        high95ConfidenceInterval: 0.1089915829377038,
        atRisk: 19,
    },
    {
        survivalFunctionEstimate: 0.06949803635067119,
        standardError: 0.014532631293769515,
        low95ConfidenceInterval: 0.04612953127972351,
        high95ConfidenceInterval: 0.10470466364184056,
        atRisk: 19,
    },
    {
        survivalFunctionEstimate: 0.06563703433118945,
        standardError: 0.014228913963762982,
        low95ConfidenceInterval: 0.04291646613874289,
        high95ConfidenceInterval: 0.10038618421810114,
        atRisk: 18,
    },
    {
        survivalFunctionEstimate: 0.06563703433118945,
        atRisk: 17,
    },
    {
        survivalFunctionEstimate: 0.06153471968549011,
        standardError: 0.013918415302567492,
        low95ConfidenceInterval: 0.03949923187389412,
        high95ConfidenceInterval: 0.09586317371590296,
        atRisk: 16,
    },
    {
        survivalFunctionEstimate: 0.06153471968549011,
        atRisk: 15,
    },
    {
        survivalFunctionEstimate: 0.05768879970514698,
        standardError: 0.013569465109432945,
        low95ConfidenceInterval: 0.03638088573066293,
        high95ConfidenceInterval: 0.09147654172189731,
        atRisk: 16,
    },
    {
        survivalFunctionEstimate: 0.05384287972480385,
        standardError: 0.013198600303195734,
        low95ConfidenceInterval: 0.033302109182732395,
        high95ConfidenceInterval: 0.0870532157933977,
        atRisk: 15,
    },
    {
        survivalFunctionEstimate: 0.05025335440981693,
        standardError: 0.012797496822965089,
        low95ConfidenceInterval: 0.03050692615554526,
        high95ConfidenceInterval: 0.08278118931295943,
        atRisk: 15,
    },
    {
        survivalFunctionEstimate: 0.04666382909483001,
        standardError: 0.01237656303372325,
        low95ConfidenceInterval: 0.027747108110025058,
        high95ConfidenceInterval: 0.07847711326012984,
        atRisk: 14,
    },
    {
        survivalFunctionEstimate: 0.043330698445199295,
        standardError: 0.011932907707130624,
        low95ConfidenceInterval: 0.025257012951451562,
        high95ConfidenceInterval: 0.07433774656400496,
        atRisk: 14,
    },
    {
        survivalFunctionEstimate: 0.043330698445199295,
        atRisk: 13,
    },
    {
        survivalFunctionEstimate: 0.043330698445199295,
        atRisk: 12,
    },
    {
        survivalFunctionEstimate: 0.043330698445199295,
        atRisk: 11,
    },
    {
        survivalFunctionEstimate: 0.03899762860067937,
        standardError: 0.011499448542896572,
        low95ConfidenceInterval: 0.021879613500092612,
        high95ConfidenceInterval: 0.06950831359384334,
        atRisk: 10,
    },
    {
        survivalFunctionEstimate: 0.03899762860067937,
        atRisk: 9,
    },
    {
        survivalFunctionEstimate: 0.03466455875615944,
        standardError: 0.01100786691898095,
        low95ConfidenceInterval: 0.018603012584219737,
        high95ConfidenceInterval: 0.06459338928677234,
        atRisk: 9,
    },
    {
        survivalFunctionEstimate: 0.03033148891163951,
        standardError: 0.010449963554611303,
        low95ConfidenceInterval: 0.015439502846095559,
        high95ConfidenceInterval: 0.059587360342341945,
        atRisk: 8,
    },
    {
        survivalFunctionEstimate: 0.02599841906711958,
        standardError: 0.009814434744337252,
        low95ConfidenceInterval: 0.012405587161558741,
        high95ConfidenceInterval: 0.05448494982035487,
        atRisk: 7,
    },
    {
        survivalFunctionEstimate: 0.02599841906711958,
        atRisk: 7,
    },
    {
        survivalFunctionEstimate: 0.02599841906711958,
        atRisk: 6,
    },
    {
        survivalFunctionEstimate: 0.02599841906711958,
        atRisk: 5,
    },
    {
        survivalFunctionEstimate: 0.02599841906711958,
        atRisk: 4,
    },
    {
        survivalFunctionEstimate: 0.02166534922259965,
        standardError: 0.009085004528366747,
        low95ConfidenceInterval: 0.00952419892262283,
        high95ConfidenceInterval: 0.049283657423645724,
        atRisk: 6,
    },
    {
        survivalFunctionEstimate: 0.02166534922259965,
        atRisk: 5,
    },
    {
        survivalFunctionEstimate: 0.02166534922259965,
        atRisk: 4,
    },
    {
        survivalFunctionEstimate: 0.014443566148399766,
        standardError: 0.008452968799622852,
        low95ConfidenceInterval: 0.0045868685252765775,
        high95ConfidenceInterval: 0.045481269396231594,
        atRisk: 3,
    },
    {
        survivalFunctionEstimate: 0.014443566148399766,
        atRisk: 3,
    },
    {
        survivalFunctionEstimate: 0.009629044098933177,
        standardError: 0.006870940914851225,
        low95ConfidenceInterval: 0.002377914571482785,
        high95ConfidenceInterval: 0.03899151440137049,
        atRisk: 3,
    },
    {
        survivalFunctionEstimate: 0.009629044098933177,
        atRisk: 3,
    },
    {
        survivalFunctionEstimate: 0.009629044098933177,
        atRisk: 2,
    },
    {
        survivalFunctionEstimate: 0.009629044098933177,
        atRisk: 1,
    },
];

const examplePatientSurvivals = _.sortBy(
    exampleAlteredPatientSurvivals.concat(exampleUnalteredPatientSurvivals),
    s => s.months
);

const exampleAlteredEstimates = [1, 0.8];
const exampleUnalteredEstimates = [0.8, 0.8, 0.4, 0];
const exampleEstimates = exampleAlteredEstimates.concat(
    exampleUnalteredEstimates
);

const exampleAlteredSurvivalSummaries = [
    {
        atRisk: 6,
        survivalFunctionEstimate: 1,
    },
    {
        atRisk: 5,
        survivalFunctionEstimate: 0.8,
        high95ConfidenceInterval: 1,
        low95ConfidenceInterval: 0.516125760342172,
        standardError: 0.1788854381999832,
    },
];

const exampleUnalteredSurvivalSummaries = [
    {
        atRisk: 4,
        survivalFunctionEstimate: 0.8,
    },
    {
        atRisk: 3,
        survivalFunctionEstimate: 0.8,
    },
    {
        atRisk: 2,
        survivalFunctionEstimate: 0.4,
        high95ConfidenceInterval: 1,
        low95ConfidenceInterval: 0.09349578484597448,
        standardError: 0.29664793948382656,
    },
    {
        atRisk: 1,
        survivalFunctionEstimate: 0,
        high95ConfidenceInterval: NaN,
        low95ConfidenceInterval: NaN,
        standardError: NaN,
    },
];

const exampleSurvivalSummaries = exampleAlteredSurvivalSummaries.concat(
    exampleUnalteredSurvivalSummaries
);

describe('SurvivalUtil', () => {
    describe('#getNumPatientsAtRisk()', () => {
        it('returns empty for empty', () => {
            assert.deepEqual(getNumPatientsAtRisk([]), []);
        });
        it('returns correct when all patients start at 0', () => {
            assert.deepEqual(
                getNumPatientsAtRisk([
                    { entryMonths: 0, months: 5 },
                    { entryMonths: 0, months: 10 },
                    { entryMonths: 0, months: 15 },
                    { entryMonths: 0, months: 20 },
                ]),
                [4, 3, 2, 1]
            );
        });
        it('returns correct when some patients enter is equal to their exit', () => {
            assert.deepEqual(
                getNumPatientsAtRisk([
                    { entryMonths: 0, months: 5 },
                    { entryMonths: 10, months: 10 },
                    { entryMonths: 15, months: 15 },
                    { entryMonths: 10, months: 20 },
                ]),
                [1, 0, 1, 1]
            );
        });
        it('returns correct when some patients enter at the same time', () => {
            assert.deepEqual(
                getNumPatientsAtRisk([
                    { entryMonths: 0, months: 5 },
                    { entryMonths: 10, months: 20 },
                    { entryMonths: 10, months: 25 },
                    { entryMonths: 10, months: 30 },
                ]),
                [1, 3, 2, 1]
            );
        });
        it('returns correct when some patients exit at the same time', () => {
            assert.deepEqual(
                getNumPatientsAtRisk([
                    { entryMonths: 0, months: 5 },
                    { entryMonths: 0, months: 30 },
                    { entryMonths: 10, months: 30 },
                    { entryMonths: 15, months: 30 },
                ]),
                [2, 3, 2, 1]
            );
        });
        it('returns correct when some patients enter at 0 and some enter throughout', () => {
            assert.deepEqual(
                getNumPatientsAtRisk([
                    { entryMonths: 0, months: 5 },
                    { entryMonths: 10, months: 30 },
                    { entryMonths: 15, months: 30 },
                    { entryMonths: 0, months: 45 },
                    { entryMonths: 20, months: 90 },
                ]),
                [2, 4, 3, 2, 1]
            );
        });
        it('returns correct number of risk when calculating example data from github repository: delayed_entry_clin_genom_studies', () => {
            assert.deepEqual(
                getNumPatientsAtRisk(
                    examplePatientSurvivalsFromDelayedEntryRepo
                ),
                numberOfRiskFoPatientSurvivalsFromDelayedEntryRepo
            );
        });
    });
    describe('#getSurvivalSummaries()', () => {
        it('returns empty list for empty list', () => {
            assert.deepEqual(getSurvivalSummaries([]), []);
        });

        it('returns correct survival summaries for the example data, compare with R survival package result', () => {
            assert.deepEqual(
                getSurvivalSummaries(examplePatientSurvivals),
                exampleSurvivalSummaries
            );
        });

        it('returns correct survival summaries for the example data from cbioportal, compare with R survival package result', () => {
            assert.deepEqual(
                getSurvivalSummaries(cbioExamplePatientSurvivals),
                cbioExampleSurvivalSummaries
            );
        });

        it('returns correct survival summaries for the large example data from R survival package, compare with R survival package result', () => {
            assert.deepEqual(
                getSurvivalSummaries(
                    largeExamplePatientSurvivalsFromRSurvivalPackage
                ),
                largeExampleSurvivalSummaries
            );
        });

        it('returns correct survival summaries for example data from github repository: delayed_entry_clin_genom_studies', () => {
            assert.deepEqual(
                getSurvivalSummaries(
                    examplePatientSurvivalsFromDelayedEntryRepo
                ),
                delayedEntrySurvivalSummaries
            );
        });
    });

    describe('#getMedian()', () => {
        it('returns NA for empty list', () => {
            assert.equal(getMedian([], []), 'NA');
        });

        it('returns 63.83 for the example data', () => {
            assert.equal(
                getMedian(examplePatientSurvivals, exampleSurvivalSummaries),
                '63.83 (63.83 - NA)'
            );
        });
    });

    // describe('#getLineData()', () => {
    //     it('returns [{x: 0, y: 100}] for empty list', () => {
    //         assert.deepEqual(getLineData([], []), [{ x: 0, y: 100 }]);
    //     });

    //     it('returns correct line data for the example data', () => {
    //         assert.deepEqual(
    //             getLineData(examplePatientSurvivals, exampleEstimates),
    //             [
    //                 { x: 0, y: 100 },
    //                 { x: 0, y: 100 },
    //                 { x: 0.09, y: 80 },
    //                 { x: 0.13, y: 80 },
    //                 { x: 5.09, y: 80 },
    //                 { x: 63.83, y: 40 },
    //                 { x: 182.19, y: 0 },
    //             ]
    //         );
    //     });
    // });

    describe('#getScatterData()', () => {
        it('returns empty list for empty list', () => {
            assert.deepEqual(getScatterData([], []), []);
        });

        //     it('returns correct scatter data for the example data', () => {
        //         assert.deepEqual(
        //             getScatterData(examplePatientSurvivals, exampleEstimates),
        //             [
        //                 {
        //                     x: 0,
        //                     y: 100,
        //                     patientId: 'TCGA-OR-A5J3',
        //                     uniquePatientKey: 'TCGA-OR-A5J3',
        //                     studyId: 'acc_tcga',
        //                     status: false,
        //                 },
        //                 {
        //                     x: 0.09,
        //                     y: 80,
        //                     patientId: 'TCGA-OR-A5J2',
        //                     uniquePatientKey: 'TCGA-OR-A5J2',
        //                     studyId: 'acc_tcga',
        //                     status: true,
        //                 },
        //                 {
        //                     x: 0.13,
        //                     y: 80,
        //                     patientId: 'TCGA-2F-A9KP',
        //                     uniquePatientKey: 'TCGA-2F-A9KP',
        //                     studyId: 'blca_tcga',
        //                     status: false,
        //                 },
        //                 {
        //                     x: 5.09,
        //                     y: 80,
        //                     patientId: 'TCGA-OR-A5J1',
        //                     uniquePatientKey: 'TCGA-OR-A5J1',
        //                     studyId: 'acc_tcga',
        //                     status: false,
        //                 },
        //                 {
        //                     x: 63.83,
        //                     y: 40,
        //                     patientId: 'TCGA-2F-A9KO',
        //                     uniquePatientKey: 'TCGA-2F-A9KO',
        //                     studyId: 'blca_tcga',
        //                     status: true,
        //                 },
        //                 {
        //                     x: 182.19,
        //                     y: 0,
        //                     patientId: 'TCGA-2F-A9KQ',
        //                     uniquePatientKey: 'TCGA-2F-A9KQ',
        //                     studyId: 'blca_tcga',
        //                     status: true,
        //                 },
        //             ]
        //         );
        //     });
    });

    describe('#getScatterDataWithOpacity()', () => {
        it('returns empty list for empty list', () => {
            assert.deepEqual(getScatterDataWithOpacity([], []), []);
        });

        //     it('returns correct scatter data with opacity for the example data', () => {
        //         assert.deepEqual(
        //             getScatterDataWithOpacity(
        //                 examplePatientSurvivals,
        //                 exampleEstimates
        //             ),
        //             [
        //                 {
        //                     x: 0,
        //                     y: 100,
        //                     patientId: 'TCGA-OR-A5J3',
        //                     uniquePatientKey: 'TCGA-OR-A5J3',
        //                     studyId: 'acc_tcga',
        //                     status: false,
        //                     opacity: 1,
        //                 },
        //                 {
        //                     x: 0.09,
        //                     y: 80,
        //                     patientId: 'TCGA-OR-A5J2',
        //                     uniquePatientKey: 'TCGA-OR-A5J2',
        //                     studyId: 'acc_tcga',
        //                     status: true,
        //                     opacity: 0,
        //                 },
        //                 {
        //                     x: 0.13,
        //                     y: 80,
        //                     patientId: 'TCGA-2F-A9KP',
        //                     uniquePatientKey: 'TCGA-2F-A9KP',
        //                     studyId: 'blca_tcga',
        //                     status: false,
        //                     opacity: 1,
        //                 },
        //                 {
        //                     x: 5.09,
        //                     y: 80,
        //                     patientId: 'TCGA-OR-A5J1',
        //                     uniquePatientKey: 'TCGA-OR-A5J1',
        //                     studyId: 'acc_tcga',
        //                     status: false,
        //                     opacity: 1,
        //                 },
        //                 {
        //                     x: 63.83,
        //                     y: 40,
        //                     patientId: 'TCGA-2F-A9KO',
        //                     uniquePatientKey: 'TCGA-2F-A9KO',
        //                     studyId: 'blca_tcga',
        //                     status: true,
        //                     opacity: 0,
        //                 },
        //                 {
        //                     x: 182.19,
        //                     y: 0,
        //                     patientId: 'TCGA-2F-A9KQ',
        //                     uniquePatientKey: 'TCGA-2F-A9KQ',
        //                     studyId: 'blca_tcga',
        //                     status: true,
        //                     opacity: 0,
        //                 },
        //             ]
        //         );
        //     });
    });

    describe('#getStats()', () => {
        it('returns [0, 0, "NA"] for empty list', () => {
            assert.deepEqual(getStats([], []), [0, 0, 'NA']);
        });

        it('returns correct stats for the example data', () => {
            assert.deepEqual(
                getStats(examplePatientSurvivals, exampleSurvivalSummaries),
                [6, 3, '63.83 (63.83 - NA)']
            );
        });
    });

    // describe('#getDownloadContent()', () => {
    //     it('returns correct download content for the example data', () => {
    //         const data = [];
    //         data.push({
    //             scatterData: getScatterData(
    //                 exampleAlteredPatientSurvivals,
    //                 exampleAlteredEstimates
    //             ),
    //             title: 'test_altered_title',
    //         });
    //         data.push({
    //             scatterData: getScatterData(
    //                 exampleUnalteredPatientSurvivals,
    //                 exampleUnalteredEstimates
    //             ),
    //             title: 'test_unaltered_title',
    //         });

    //         const targetDownloadContent =
    //             'test_main_title\n\ntest_altered_title\nCase ID\tStudy ID\t' +
    //             'Number at Risk\tStatus\tSurvival Rate\tTime (months)\nTCGA-OR-A5J2\tacc_tcga\t2\t' +
    //             'deceased\t1\t0.09\nTCGA-OR-A5J1\tacc_tcga\t1\tcensored\t0.8\t5.09\n\ntest_unaltered_title\nCase ID\tStudy ID\tNumber at Risk\tStatus\tSurvival Rate\tTime (months)\nTCGA-OR-A5J3\t' +
    //             'acc_tcga\t4\tcensored\t0.8\t0\nTCGA-2F-A9KP\tblca_tcga\t3\tcensored\t' +
    //             '0.8\t0.13\nTCGA-2F-A9KO\tblca_tcga\t2\tdeceased\t0.4\t63.83\nTCGA-2F-A9KQ\tblca_tcga\t1\tdeceased\t0\t182.19';

    //         assert.equal(
    //             getDownloadContent(data, 'test_main_title'),
    //             targetDownloadContent
    //         );
    //     });
    // });

    describe('#convertScatterDataToDownloadData()', () => {
        it('returns empty list for empty list', () => {
            assert.deepEqual(
                convertScatterDataToDownloadData(getScatterData([], [])),
                []
            );
        });

        //     it('returns correct download data for the example data', () => {
        //         assert.deepEqual(
        //             convertScatterDataToDownloadData(
        //                 getScatterData(examplePatientSurvivals, exampleEstimates)
        //             ),
        //             [
        //                 {
        //                     'Time (months)': 0,
        //                     'Survival Rate': 1,
        //                     'Case ID': 'TCGA-OR-A5J3',
        //                     'Study ID': 'acc_tcga',
        //                     Status: 'censored',
        //                     'Number at Risk': 6,
        //                 },
        //                 {
        //                     'Time (months)': 0.09,
        //                     'Survival Rate': 0.8,
        //                     'Case ID': 'TCGA-OR-A5J2',
        //                     'Study ID': 'acc_tcga',
        //                     Status: 'deceased',
        //                     'Number at Risk': 5,
        //                 },
        //                 {
        //                     'Time (months)': 0.13,
        //                     'Survival Rate': 0.8,
        //                     'Case ID': 'TCGA-2F-A9KP',
        //                     'Study ID': 'blca_tcga',
        //                     Status: 'censored',
        //                     'Number at Risk': 4,
        //                 },
        //                 {
        //                     'Time (months)': 5.09,
        //                     'Survival Rate': 0.8,
        //                     'Case ID': 'TCGA-OR-A5J1',
        //                     'Study ID': 'acc_tcga',
        //                     Status: 'censored',
        //                     'Number at Risk': 3,
        //                 },
        //                 {
        //                     'Time (months)': 63.83,
        //                     'Survival Rate': 0.4,
        //                     'Case ID': 'TCGA-2F-A9KO',
        //                     'Study ID': 'blca_tcga',
        //                     Status: 'deceased',
        //                     'Number at Risk': 2,
        //                 },
        //                 {
        //                     'Time (months)': 182.19,
        //                     'Survival Rate': 0,
        //                     'Case ID': 'TCGA-2F-A9KQ',
        //                     'Study ID': 'blca_tcga',
        //                     Status: 'deceased',
        //                     'Number at Risk': 1,
        //                 },
        //             ]
        //         );
        //     });
    });

    describe('#downSampling()', () => {
        it('return empty list for empty list', () => {
            assert.deepEqual(
                downSampling([], {
                    xDenominator: 100,
                    yDenominator: 100,
                    threshold: 100,
                    enableCensoringCross: true,
                }),
                []
            );
        });

        it('when any denominator is 0, return the full data', () => {
            let opts = {
                xDenominator: 0,
                yDenominator: 100,
                threshold: 100,
                enableCensoringCross: true,
            };
            assert.deepEqual(
                downSampling(allScatterData, opts),
                allScatterData
            );
        });

        it('when any denominator is negative value, return the full data', () => {
            let opts = {
                xDenominator: -1,
                yDenominator: 100,
                threshold: 100,
                enableCensoringCross: true,
            };
            assert.deepEqual(
                downSampling(allScatterData, opts),
                allScatterData
            );
        });

        it('Remove the dot which too close from the last dot', () => {
            // In this case, the hypotenuse is always 1, keep the threshold at 2.5
            assert.deepEqual(
                downSampling(
                    [
                        {
                            x: 0,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                        {
                            x: 2,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                        {
                            x: 10,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                    ],
                    {
                        xDenominator: 4,
                        yDenominator: 4,
                        threshold: 100,
                        enableCensoringCross: true,
                    }
                ),
                [
                    {
                        x: 0,
                        y: 10,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        opacity: 1,
                        status: true,
                    },
                    {
                        x: 10,
                        y: 10,
                        opacity: 1,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        status: true,
                    },
                ]
            );
        });

        it('Remove the dot which too close from the last dot, ignore the status', () => {
            // In this case, the hypotenuse is always 1, keep the threshold at 2.5
            assert.deepEqual(
                downSampling(
                    [
                        {
                            x: 0,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                        {
                            x: 2,
                            y: 10,
                            opacity: 0,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: false,
                        },
                        {
                            x: 10,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                    ],
                    {
                        threshold: 100,
                        xDenominator: 4,
                        yDenominator: 4,
                        enableCensoringCross: true,
                    }
                ),
                [
                    {
                        x: 0,
                        y: 10,
                        opacity: 1,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        status: true,
                    },
                    {
                        x: 10,
                        y: 10,
                        opacity: 1,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        status: true,
                    },
                ]
            );
        });

        it('Remove the dot when the distance from last dot equals to the threshold', () => {
            // In this case, the hypotenuse is always 1, keep the threshold at 2
            assert.deepEqual(
                downSampling(
                    [
                        {
                            x: 0,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                        {
                            x: 2,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                        {
                            x: 10,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                    ],
                    {
                        threshold: 100,
                        xDenominator: 5,
                        yDenominator: 5,
                        enableCensoringCross: true,
                    }
                ),
                [
                    {
                        x: 0,
                        y: 10,
                        opacity: 1,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        status: true,
                    },
                    {
                        x: 10,
                        y: 10,
                        opacity: 1,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        status: true,
                    },
                ]
            );
        });

        it('when a hidden dot between two visualized dots, the distance between two vis dots should be calculated separately from the hidden dot', () => {
            // In this case, the hypotenuse is always 1, keep the threshold at 1.25
            assert.deepEqual(
                downSampling(
                    [
                        {
                            x: 0,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                        {
                            x: 1,
                            y: 10,
                            opacity: 0,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: false,
                        },
                        {
                            x: 2,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                        {
                            x: 10,
                            y: 10,
                            opacity: 1,
                            patientId: '',
                            uniquePatientKey: '',
                            studyId: '',
                            status: true,
                        },
                    ],
                    {
                        threshold: 100,
                        xDenominator: 8,
                        yDenominator: 8,
                        enableCensoringCross: true,
                    }
                ),
                [
                    {
                        x: 0,
                        y: 10,
                        opacity: 1,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        status: true,
                    },
                    {
                        x: 2,
                        y: 10,
                        opacity: 1,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        status: true,
                    },
                    {
                        x: 10,
                        y: 10,
                        opacity: 1,
                        patientId: '',
                        uniquePatientKey: '',
                        studyId: '',
                        status: true,
                    },
                ]
            );
        });
    });

    describe('floorScatterData()', () => {
        it('return correct data after do flooring', () => {
            let testScatterData: ScatterData[] = [
                {
                    x: 0,
                    y: 10,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                    opacity: 1,
                },
                {
                    x: 0.5,
                    y: 9,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                    opacity: 1,
                },
                {
                    x: 1,
                    y: 8,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                    opacity: 1,
                },
                {
                    x: 1.2,
                    y: 7,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                    opacity: 1,
                },
                {
                    x: 1.4,
                    y: 6,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                    opacity: 1,
                },
                {
                    x: 1.5,
                    y: 6,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: false,
                    opacity: 0,
                },
            ];
            let result = [
                {
                    x: 0,
                    y: 9,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                    opacity: 1,
                    numberOfEvents: 2,
                    numberOfCensored: 0,
                    atRisk: undefined,
                    group: undefined,
                },
                {
                    x: 1,
                    y: 6,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                    opacity: 1,
                    numberOfEvents: 3,
                    numberOfCensored: 1,
                    atRisk: undefined,
                    group: undefined,
                },
            ];

            assert.deepEqual(floorScatterData(testScatterData), result);
        });
    });

    describe('filterScatterData()', () => {
        it('Return full data if the filers are undefined', () => {
            let testData = {
                altered: {
                    numOfCases: allScatterData.length,
                    line: [],
                    scatter: allScatterData,
                    scatterWithOpacity: allScatterData,
                },
            };
            assert.deepEqual(
                filterScatterData(testData, undefined, {
                    xDenominator: 100,
                    yDenominator: 100,
                    threshold: 100,
                    enableCensoringCross: true,
                }),
                testData
            );
        });

        it('return correct data after filtering and down sampling', () => {
            let testScatterData = [
                {
                    x: 0,
                    y: 10,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                },
                {
                    x: 0.5,
                    y: 9,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                },
                {
                    x: 1,
                    y: 8,
                    patientId: '',
                    uniquePatientKey: '',
                    studyId: '',
                    status: true,
                },
            ];
            let testData = {
                altered: {
                    numOfCases: testScatterData.length,
                    line: [],
                    scatter: testScatterData,
                    scatterWithOpacity: testScatterData,
                },
            };
            var result = filterScatterData(
                testData,
                { x: [0, 10], y: [9, 10] },
                {
                    xDenominator: 2,
                    yDenominator: 2,
                    threshold: 2,
                    enableCensoringCross: true,
                }
            );
            assert.deepEqual(result.altered.numOfCases, 2);
            assert.deepEqual(result.altered.scatter[1], {
                x: 0.5,
                y: 9,
                patientId: '',
                uniquePatientKey: '',
                studyId: '',
                status: true,
            });
        });
    });

    describe('parseSurvivalData()', () => {
        it('survival data do not has boolean status', () => {
            let testData = {
                status: undefined,
                label: 'unknown',
            };
            assert.deepEqual(parseSurvivalData('unknown'), testData);
        });

        it('survival data has boolean status', () => {
            let testData1 = {
                status: '0',
                label: 'LIVING',
            };
            assert.deepEqual(parseSurvivalData('0:LIVING'), testData1);

            let testData2 = {
                status: '1',
                label: 'DECEASED',
            };
            assert.deepEqual(parseSurvivalData('1:DECEASED'), testData2);
        });
    });

    describe('getSurvivalStatusBoolean()', () => {
        it('survival data do not has boolean status, return false', () => {
            const survivalStatus = 'unknown';
            const prefix = 'OS';
            assert.equal(
                getSurvivalStatusBoolean(survivalStatus, prefix),
                false
            );
        });

        it('survival data has boolean status, return status', () => {
            const survivalStatus1 = '1:DECEASED';
            const prefix1 = 'OS';
            const survivalStatus2 = '0:Recurred/Progressed';
            const prefix2 = 'DFS';
            const survivalStatus3 = '1:Event';
            const prefix3 = 'EFS';
            assert.equal(
                getSurvivalStatusBoolean(survivalStatus1, prefix1),
                true
            );
            assert.equal(
                getSurvivalStatusBoolean(survivalStatus2, prefix2),
                false
            );
            assert.equal(
                getSurvivalStatusBoolean(survivalStatus3, prefix3),
                true
            );
        });
    });

    describe('isNullSurvivalClinicalDataValue()', () => {
        it('return true for all null values', () => {
            const nullValues = [
                '[not applicable]',
                '[not available]',
                '[pending]',
                '[discrepancy]',
                '[completed]',
                '[null]',
                '',
                'na',
            ];
            const nullValuesIgnoreCaseAndNotTrimmed = [
                '[Not Applicable]  ',
                '  [Not Available]',
                '[Pending]',
                '[Discrepancy]',
                '  [Completed]',
                '[NULL]  ',
                'NA',
            ];
            nullValues.forEach(nullValue => {
                assert.equal(isNullSurvivalClinicalDataValue(nullValue), true);
            });
            nullValuesIgnoreCaseAndNotTrimmed.forEach(nullValue => {
                assert.equal(isNullSurvivalClinicalDataValue(nullValue), true);
            });
        });

        it('return false for not null values', () => {
            const values = [
                'DECEASED',
                'LIVING',
                'Progressed',
                'Recurred/Progressed',
            ];
            values.forEach(value => {
                assert.equal(isNullSurvivalClinicalDataValue(value), false);
            });
        });
    });

    describe('createSurvivalAttributeIdsDict()', () => {
        it('return empty values for empty list', () => {
            const emptyList: string[] = [];
            const emptyDict: {
                [id: string]: string;
            } = {};
            assert.deepEqual(
                createSurvivalAttributeIdsDict(emptyList),
                emptyDict
            );
        });

        it('return attribute ids', () => {
            const prefixList = ['OS', 'DFS', 'PFS'];
            const result = {
                OS_MONTHS: 'OS_MONTHS',
                OS_STATUS: 'OS_STATUS',
                DFS_MONTHS: 'DFS_MONTHS',
                DFS_STATUS: 'DFS_STATUS',
                PFS_MONTHS: 'PFS_MONTHS',
                PFS_STATUS: 'PFS_STATUS',
            };
            assert.deepEqual(
                createSurvivalAttributeIdsDict(prefixList),
                result
            );
        });

        it('do not return months attributes if months get excluded', () => {
            const prefixList = ['OS', 'DFS', 'PFS'];
            const result = {
                OS_STATUS: 'OS_STATUS',
                DFS_STATUS: 'DFS_STATUS',
                PFS_STATUS: 'PFS_STATUS',
            };
            assert.deepEqual(
                createSurvivalAttributeIdsDict(prefixList, true),
                result
            );
        });

        it('do not return status attributes if status get excluded', () => {
            const prefixList = ['OS', 'DFS', 'PFS'];
            const result = {
                OS_MONTHS: 'OS_MONTHS',
                DFS_MONTHS: 'DFS_MONTHS',
                PFS_MONTHS: 'PFS_MONTHS',
            };
            assert.deepEqual(
                createSurvivalAttributeIdsDict(prefixList, false, true),
                result
            );
        });
    });

    describe('sortPatientSurvivals()', () => {
        it('return small months first', () => {
            const result = [
                {
                    status: true,
                    months: 1,
                },
                {
                    status: true,
                    months: 2,
                },
                {
                    status: true,
                    months: 3,
                },
            ];
            const unsortedPatientSurvivals = [
                {
                    status: true,
                    months: 3,
                },
                {
                    status: true,
                    months: 1,
                },
                {
                    status: true,
                    months: 2,
                },
            ] as PatientSurvival[];
            assert.deepEqual(
                sortPatientSurvivals(unsortedPatientSurvivals),
                result
            );
        });

        it('return true status first', () => {
            const result = [
                {
                    status: true,
                    months: 1,
                },
                {
                    status: false,
                    months: 1,
                },
                {
                    status: false,
                    months: 1,
                },
            ];
            const unsortedPatientSurvivals = [
                {
                    status: false,
                    months: 1,
                },
                {
                    status: true,
                    months: 1,
                },
                {
                    status: false,
                    months: 1,
                },
            ] as PatientSurvival[];
            assert.deepEqual(
                sortPatientSurvivals(unsortedPatientSurvivals),
                result
            );
        });

        it('return small months first, then true status first when months are equal', () => {
            const result = [
                {
                    status: true,
                    months: 1,
                },
                {
                    status: false,
                    months: 1,
                },
                {
                    status: false,
                    months: 2,
                },
            ];
            const unsortedPatientSurvivals = [
                {
                    status: false,
                    months: 1,
                },
                {
                    status: false,
                    months: 2,
                },
                {
                    status: true,
                    months: 1,
                },
            ] as PatientSurvival[];
            assert.deepEqual(
                sortPatientSurvivals(unsortedPatientSurvivals),
                result
            );
        });
    });

    describe('generateStudyViewSurvivalPlotTitle()', () => {
        const correctTitle = 'Overall';
        const originalTitle = 'Overall';
        const removeStatusTitle = 'Overall Status';
        const removeSurvivalTitle = 'Overall Survival';
        const removeStatusAndSurvivalTitle = 'Overall Survival Status';
        it('returns correct title', () => {
            assert.equal(
                generateStudyViewSurvivalPlotTitle(originalTitle),
                correctTitle
            );
            assert.equal(
                generateStudyViewSurvivalPlotTitle(removeStatusTitle),
                correctTitle
            );
            assert.equal(
                generateStudyViewSurvivalPlotTitle(removeSurvivalTitle),
                correctTitle
            );
            assert.equal(
                generateStudyViewSurvivalPlotTitle(
                    removeStatusAndSurvivalTitle
                ),
                correctTitle
            );
        });
    });
});
